/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

(function ()
{
  Ext.BLANK_IMAGE_URL = 'images/s.gif';

  Ext.Loader.setConfig( {
    enabled : true, // enables dynamic (on-demand) class-loading
    disableCaching : true, // false = removes the _dc param from all loader's HTTP requests
    paths : {
      'Ext' : 'js/extjs',
      'QDesk' : 'js/quartzdesk'
    }
  } );

  Ext.require( [
    'QDesk.ApplicationConfig',
    'QDesk.CurrentUser',
    'QDesk.Log',
    'QDesk.Msg',
    'QDesk.Properties',

    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',
    'QDesk.util.constant.EventName',

    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.Viewport'
  ] );


  /*
   * Register a global error handler for errors raised throughout the ExtJS framework and this application.
   * Note: to raise an error the application should use the Ext.Error.raise function.
   */
  Ext.Error.handle = function ( err )
  {
    Ext.log( { level : 'error', stack : true }, err );
    return QDesk.util.UxUtils.handleError( err );
  };

  Ext.application( {
    name : 'QDesk',
    appFolder : 'js/quartzdesk',

    controllers : [
      'HeartbeatController',

      'navigation.ConnectionsController',

      'navigation.NavigationController',

      'navigation.SettingsController',

      'scheduler.SchedulerViewController',

      'security.SecurityController',

      //'top.SwitchLocaleController',
      'top.LicenseInfoController'
    ],

    models : [
      'common.Locale',
      'common.NameValuePair',
      'common.TimeZone',

      'gui.AboutData',
      'gui.ConnectionsTreeNode',
      'gui.JSScriptingTreeNode',
      'gui.SettingsTreeNode',
      'gui.MessageMacroTreeNode',

      'customization.gui.navigation.Link',
      'customization.gui.navigation.Node',
      'customization.gui.navigation.Tab',

      'jmx.RemoteJmxService',

      'message.Message',
      'message.MessageAttachment',
      'message.MessageChannelProfile',
      'message.MessageChannelProfilesTreeNode',

      'scheduler.Scheduler',
      'scheduler.ExecHealthIndicator',
      'scheduler.ExecHealthIndicatorEntry',

      'scheduler.quartz.QuartzCalendar',
      'scheduler.quartz.QuartzExecHistory',
      'scheduler.quartz.QuartzExecNotificationRule',
      'scheduler.quartz.QuartzExecutingJob',
      'scheduler.quartz.QuartzJobChain',
      'scheduler.quartz.QuartzJobDataMapEntry',
      'scheduler.quartz.QuartzJobDetail',
      'scheduler.quartz.QuartzJobGroup',
      'scheduler.quartz.QuartzPlannedJobExecution',
      'scheduler.quartz.QuartzTrigger',
      'scheduler.quartz.QuartzTriggerGroup',
      'scheduler.quartz.QuartzTriggerWithJobDetail',
      'scheduler.quartz.stats.QuartzExecStatData'
    ],

    /* Singleton stores that are automatically instantiated and registered with the Ext.StoreManager */
    stores : [
      'EmptyStore',

      'LocaleStore',
      'TimestampFormatStore',

      'enums.ExecStatusStore',
      'enums.ExpressionLanguageStore',
      'enums.JsLoggingLevelStore',
      'enums.MessageChannelStore',
      'enums.MessageReceiverWSVersionStore',
      'enums.NotificationSeverityStore',
      'enums.ProxyTypeStore',
      'enums.SnmpV3AuthenticationProtocolStore',
      'enums.SnmpV3PrivacyProtocolStore',
      'enums.SnmpV3SecurityLevelStore',
      'enums.WSAuthenticationTypeStore',

      'enums.JmxProtocolStore',

      'enums.scheduler.ExecNotificationRuleConditionTypeStore',
      'enums.scheduler.JobChainConditionTypeStore',
      'enums.scheduler.JobChainTargetExecTypeStore',

      'enums.scheduler.quartz.QuartzExecHistoryExportColumnStore',

      'enums.security.PermissionStore',

      'customization.gui.navigation.TabStore',

      'scheduler.quartz.QuartzJSScriptingDataTypesTreeStore',
      'scheduler.quartz.QuartzJSScriptingObjectsTreeStore',
      'scheduler.quartz.QuartzExecNotificationRuleMessageMacrosTreeStore',
      'scheduler.quartz.stats.QuartzStatisticsTimePeriodLengthStore',
      'scheduler.quartz.stats.QuartzJobStatisticsTypeStore',
      'scheduler.quartz.stats.QuartzSchedulerStatisticsTypeStore',
      'scheduler.quartz.stats.QuartzTriggerStatisticsTypeStore'
    ],

    /*
     * Executed by ExtJS from Ext.onReady
     */
    launch : function ()
    {
      /*
       * Disable all WAI-ARIA compatibility warnings printed in the browser console.
       *
       * This code is applicable to ExtJS 6.0.1 only.
       */
      Ext.enableAria = false;
      Ext.enableAriaButtons = false;
      Ext.enableAriaPanels = false;

      /* Stop propagating the contextmenu event from the application. This prevents the browser context menu from showing up. */
      Ext.getDoc().on( 'contextmenu', function ( event, targetHtmlEl )
      {
        // we need to enable context menus in "Chart preview" windows so that users can save images
        var targetEl = Ext.get( targetHtmlEl.id );
        if ( targetEl && targetEl.hasCls( 'x-chart-image' ) )
        {
          //
          return false;
        }
        event.stopEvent();
      } );

      /* Start of tooltip manager initialization. */
      // Init the singleton.  Any tag-based quick tips will start working.;
      Ext.tip.QuickTipManager.init();

      // Apply a set of config properties to the singleton
      Ext.apply( Ext.tip.QuickTipManager.getQuickTip(), {
        dismissDelay : 0, /* do not dismiss tooltips after 5000 ms period */
        maxWidth : 350,
        showDelay : 50 /* default is 500 and that is too long */
      } );
      /* End of tooltip manager initialization. */


      /* Timeout for all Ext.Ajax.request calls. */
      Ext.Ajax.setTimeout( 1000 * QDesk.ApplicationConfig.get( 'ajaxTimeout' ) );


      /* Start of ExtJS overrides */

      /**
       * JSON encoded Date instances are serialized using the ISO8601 format. Without this override the
       * format would be: yyyy-mm-ddThh:mm:ss (i.e. no time zone).
       */
      Ext.override( Ext.JSON, {
        encodeDate : function ( d )
        {
          return '"' + Ext.Date.format( d, QDesk.util.constant.DateFormat.timestampFormatIso8601 ) + '"';
        }
      } );


      Ext.override( Ext.data.proxy.Server, {
        /* Parameter names used in Ajax data store requests */
        startParam : 'startAt',
        groupParam : 'groupBy',
        sortParam : 'sort',
        filterParam : 'filter',

        noCache : true,  // true is default (a _dc param s appended to all proxy request URLs)

        getNoCache : function ()
        {
          if ( Ext.String.startsWith( this.getUrl(), QDesk.util.constant.ActionUrl.emptyStoreAction ) )
          {
            return false;
          }

          return this.noCache;
        }
      } );


      /* Ext.Ajax.setTimeout() does not seem to affect Ajax proxies in stores, so we need this override. */
      Ext.override( Ext.data.Store, {
        load : function ()
        {
          var me = this;
          var proxy = me.getProxy();
          proxy.timeout = 1000 * QDesk.ApplicationConfig.get( 'ajaxTimeout' );
          me.callParent( arguments );
        }
      } );


      /*
       * Overridden validation messages.
       */
      Ext.override( Ext.form.field.VTypes, {
        emailText : QDesk.Msg.get( 'Ext.form.field.VTypes.emailText' ),
        urlText : QDesk.Msg.get( 'Ext.form.field.VTypes.urlText' ),
        alphaText : QDesk.Msg.get( 'Ext.form.field.VTypes.alphaText' ),
        alphanumText : QDesk.Msg.get( 'Ext.form.field.VTypes.alphanumText' )
      } );


      /*
       * Access permission checks in buttons.
       */
      Ext.override( Ext.button.Button, {
        /*
         * The standard (desired) button state expected by the application.
         */
        _standardStateEnabled : false,

        /*
         * The standard (desired) button icon cls expected by the application.
         */
        _standardIconCls : null,

        /*
         * Flag indicating if the current user is authorized to access the button.
         */
        _authorized : false,

        initComponent : function ()
        {
          var me = this;

          me.callParent();

          // if ( me.getItemId() === 'editFolderOrConnectionButton' )
          //   debugger;

          // save the standard button enabled/disabled state
          me._standardStateEnabled = !me.isDisabled();
          me._standardIconCls = me.iconCls ? me.iconCls : null;

          me._authorized = me._isAuthorized( me.qdRequiresAnyPermissions );

          if ( !me._authorized )
            me._applyNotAuthorized();

          QDesk.getApplication().on( QDesk.util.constant.EventName.CURRENT_USER_PERMISSIONS_CHANGED,
              function ( params )
              {
                if ( me.destroyed )
                {
                  // component has been destroyed, do nothing
                  return;
                }

                var oldEffectivePermissions = params.oldEffectivePermissions;
                var newEffectivePermissions = params.newEffectivePermissions;

                // if ( me.getItemId() === 'editFolderOrConnectionButton' )
                //   debugger;

                var oldAuthorized = me._authorized;
                var newAuthorized = me._isAuthorized( me.qdRequiresAnyPermissions );

                me._authorized = newAuthorized;

                if ( oldAuthorized !== newAuthorized )
                {
                  if ( me._authorized )
                  {
                    me._applyAuthorized();
                  }
                  else
                  {
                    me._applyNotAuthorized();
                  }
                }
              } );
        },

        disable : function ()
        {
          var me = this;

          // if ( me.getItemId() === 'editFolderOrConnectionButton' )
          //   debugger;

          // now the standard button state is 'disabled'
          me._standardStateEnabled = false;

          if ( me._authorized )
          {
            // user is authorized  - invoke the original disable method
            me.callSuper( arguments );
          }
          else
          {
            me._applyNotAuthorized();
          }
        },

        enable : function ()
        {
          var me = this;

          // if ( me.getItemId() === 'editFolderOrConnectionButton' )
          //   debugger;

          // now the standard button state is 'enabled'
          me._standardStateEnabled = true;

          if ( me._authorized )
          {
            // user is authorized  - invoke the original enable method
            me.callSuper( arguments );
          }
          else
          {
            me._applyNotAuthorized();
          }
        },

        /**
         * This method sets the button to the desired (saved) state and restores the original icon
         */
        _applyAuthorized : function ()
        {
          var me = this;

          if ( me.destroyed )
          {
            // do nothing
          }
          else
          {
            // restore the standard button icon (if any)
            me.setIconCls( me._standardIconCls );

            // restore the standard button state (enabled/disabled)
            if ( me._standardStateEnabled )
            {
              me.superclass.enable.call( me );
            }
            else
            {
              me.superclass.disable.call( me );
            }
          }
        },

        /**
         * This method disables the button and makes the button use the configured "NAUTH" icon.
         */
        _applyNotAuthorized : function ()
        {
          var me = this;

          if ( me.destroyed )
          {
            // do nothing
          }
          else
          {
            // user is not authorized to use the button - disable the button and set the new icon CLS
            var newIconCls = me.qdNotAuthorizedIconCls ? me.qdNotAuthorizedIconCls : 'NAUTH';
            me.setIconCls( newIconCls );

            // disable the button if not disabled already
            if ( !me.isDisabled() )
            {
              me.superclass.disable.call( me );
            }
          }
        },

        _isAuthorized : function ( requiredAnyPermissions )
        {
          if ( requiredAnyPermissions == null )
          {
            // authentication / authorization not required
            return true;
          }
          else
          {
            if ( Ext.isEmpty( requiredAnyPermissions ) )
            {
              // required permissions are empty, the user must be at least authenticated
              return QDesk.CurrentUser.isAuthenticated();
            }
            else
            {
              return QDesk.CurrentUser.hasAnyPermissions( requiredAnyPermissions );
            }
          }
        }
      } );


      /*
       * Access permission checks in menu items.
       */
      Ext.override( Ext.menu.Item, {
        /*
         * The standard (desired) menu item state expected by the application.
         */
        _standardStateEnabled : false,

        /*
         * The standard (desired) menu item icon cls expected by the application.
         */
        _standardIconCls : null,

        /*
         * Flag indicating if the current user is authorized to access the menu item.
         */
        _authorized : false,

        initComponent : function ()
        {
          var me = this;

          me.callParent();

          // if ( me.getItemId() === 'addFolderButton' )
          //   debugger;

          // save the standard menu item enabled/disabled state
          me._standardStateEnabled = !me.isDisabled();
          me._standardIconCls = me.iconCls ? me.iconCls : null;

          me._authorized = me._isAuthorized( me.qdRequiresAnyPermissions );

          if ( !me._authorized )
            me._applyNotAuthorized();

          QDesk.getApplication().on( QDesk.util.constant.EventName.CURRENT_USER_PERMISSIONS_CHANGED,
              function ( params )
              {
                if ( me.destroyed )
                {
                  // component has been destroyed, do nothing
                  return;
                }

                var oldEffectivePermissions = params.oldEffectivePermissions;
                var newEffectivePermissions = params.newEffectivePermissions;

                // if ( me.getItemId() === 'addFolderButton' )
                //   debugger;

                var oldAuthorized = me._authorized;
                var newAuthorized = me._isAuthorized( me.qdRequiresAnyPermissions );

                me._authorized = newAuthorized;

                if ( oldAuthorized !== newAuthorized )
                {
                  if ( me._authorized )
                  {
                    me._applyAuthorized();
                  }
                  else
                  {
                    me._applyNotAuthorized();
                  }
                }
              } );
        },

        disable : function ( silent )
        {
          var me = this;

          // if ( me.getItemId() === 'addFolderButton' )
          //   debugger;

          /*
           * Change the standardStateEnabled flag only if the change is not silent. ExtJS uses
           * silent disable calls to disable child menu items when their parent menu it is disabled.
           */
          if ( !silent )
          {
            // now the standard menu item state is 'disabled'
            me._standardStateEnabled = false;
          }

          if ( me._authorized )
          {
            // user is authorized - invoke the original disable method
            me.callSuper( arguments );
          }
          else
          {
            me._applyNotAuthorized();
          }
        },

        enable : function ( silent )
        {
          var me = this;

          // if ( me.getItemId() === 'addFolderButton' )
          //   debugger;

          /*
           * Change the standardStateEnabled flag only if the change is not silent. ExtJS uses
           * silent disable calls to disable child menu items when their parent menu it is disabled.
           */
          if ( !silent )
          {
            // now the standard menu item state is 'enabled'
            me._standardStateEnabled = true;
          }

          if ( me._authorized )
          {
            // user is authorized  - invoke the original enable method
            me.callSuper( arguments );
          }
          else
          {
            me._applyNotAuthorized();
          }
        },

        /**
         * This method sets the menu item to the desired (saved) state and restores the original icon
         */
        _applyAuthorized : function ()
        {
          var me = this;

          if ( me.destroyed )
          {
            // do nothing
          }
          else
          {
            // restore the standard menu item icon (if any)
            me.setIconCls( me._standardIconCls );

            // restore the standard menu item state (enabled/disabled)
            if ( me._standardStateEnabled )
            {
              me.superclass.enable.call( me );
            }
            else
            {
              me.superclass.disable.call( me );
            }
          }
        },

        /**
         * This method disables the menu item and makes the menu item use the configured "NAUTH" icon.
         */
        _applyNotAuthorized : function ()
        {
          var me = this;

          if ( me.destroyed )
          {
            // do nothing
          }
          else
          {
            // user is not authorized to use the menu item - disable the menu item and set the new icon CLS
            var newIconCls = me.qdNotAuthorizedIconCls ? me.qdNotAuthorizedIconCls : 'NAUTH';
            me.setIconCls( newIconCls );

            // disable the menu item if not disabled already
            if ( !me.isDisabled() )
            {
              me.superclass.disable.call( me );
            }
          }
        },

        _isAuthorized : function ( requiredAnyPermissions )
        {
          if ( requiredAnyPermissions == null )
          {
            // authentication / authorization not required
            return true;
          }
          else
          {
            if ( Ext.isEmpty( requiredAnyPermissions ) )
            {
              // required permissions are empty, the user must be at least authenticated
              return QDesk.CurrentUser.isAuthenticated();
            }
            else
            {
              return QDesk.CurrentUser.hasAnyPermissions( requiredAnyPermissions );
            }
          }
        }
      } );


      // SVG no longer available in ExtJS 5 and 6 charts. Override for the default sencha-charts SVG rasterizer URL.
      //Ext.override( Ext.draw.Container, {
      //  /*
      //   * Replaces the default ExtJS SVG export action URL "http://svg.sencha.io".
      //   */
      //  defaultDownloadServerUrl : QDesk.util.constant.ActionUrl.svgActionRasterize
      //} );


      /*
       * Forms are validated after rendering so that invalid fields are immediately visually
       * distinguished.
       *
       * Form validation also occurs upon form field change.
       */
      Ext.override( Ext.form.Panel, {
        initComponent : function ()
        {
          var me = this;

          me.callParent( arguments );

          me.on( 'afterrender', function ()
          {
            me.isValid();
          } );

          Ext.each( Ext.ComponentQuery.query( 'field', me ), function ( field )
          {
            field.on( 'change', function ()
            {
              me.isValid();
            } );
          } );
        }
      } );


      /*
       * Tooltip in label support for form fields.
       * Inspired by http://www.sencha.com/forum/showthread.php?19301
       */
      Ext.override( Ext.form.Field, {
        afterRender : function ()
        {
          var label = null;
          if ( this.fieldTooltip )
          {
            label = this._findTitle( this, 'label.x-form-item-label span.x-form-item-label-inner' );
            this._maybeAddToolTipToTitle( label, this.fieldTooltip );
          }

          if ( this.boxTooltip )
          {
            label = this._findTitle( this, 'label.x-form-cb-label' );
            this._maybeAddToolTipToTitle( label, this.boxTooltip );
          }

          this.callSuper();
        },

        _findTitle : function ( field, selector )
        {
          var matchedEls = field.getEl().select( selector );
          if ( matchedEls )
          {
            var matchedLabelEls = matchedEls.child( 'label' );
            // we get the first matched label (there should not be another labels, but anyway)
            return matchedLabelEls ? matchedLabelEls.first() : null;
          }
          return null;
        },

        _maybeAddToolTipToTitle : function ( label, tooltipText )
        {
          if ( label )
          {
            if ( this.fieldTooltipClosable )
            {
              label.createChild( {
                tag : 'img',
                src : 'images/icon/tooltip-non-auto-hide-10x10.png',
                style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
                'data-qtip' : Ext.htmlEncode( tooltipText ),
                'data-qtitle' : Ext.htmlEncode( QDesk.Msg.get( 'QDesk.tooltipNonAutoHideTitle' ) ),
                //'data-closable' : true,
                'data-hide' : 'user'  // disables automatic tooltip hiding when the user moves the cursor outside the tooltip icon
              } );
            }
            else
            {
              label.createChild( {
                tag : 'img',
                src : 'images/icon/tooltip-10x10.png',
                style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
                'data-qtip' : Ext.htmlEncode( tooltipText )
              } );
            }
          }
        }
      } );


      /*
       * Tooltip in label support for form field sets.
       * Inspired by http://www.sencha.com/forum/showthread.php?19301
       */
      Ext.override( Ext.form.FieldSet, {
        afterRender : function ()
        {
          var headerDiv = null;
          if ( this.tooltip )
          {
            headerDiv = this._findHeaderDiv( this, 'div.x-fieldset-header-text' );
            this._maybeAddToolTipToHeaderDiv( headerDiv, this.tooltip );
          }

          this.callSuper();
        },

        _findHeaderDiv : function ( fieldSet, selector )
        {
          var matchedEls = fieldSet.getEl().select( selector );
          if ( matchedEls )
          {
            var matchedDivEls = matchedEls.child( 'div' );
            // we need to get the first matched div - this is because a fieldset can contain a nested fieldset that also contains the div...
            return matchedDivEls ? matchedDivEls.first() : null;
          }
          return null;
        },

        _maybeAddToolTipToHeaderDiv : function ( headerDiv, tooltipText )
        {
          if ( headerDiv )
          {
            headerDiv.createChild( {
              tag : 'img',
              src : 'images/icon/tooltip-10x10.png',
              style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
              'data-qtip' : Ext.htmlEncode( tooltipText )
            } );
          }
        }
      } );


      /*
       * Tooltip in label support for form field containers.
       * Inspired by http://www.sencha.com/forum/showthread.php?19301
       */
      Ext.override( Ext.form.FieldContainer, {
        afterRender : function ()
        {
          var label = null;
          if ( this.fieldTooltip )
          {
            label = this._findTitle( this, 'label.x-form-item-label span.x-form-item-label-inner' );
            this._maybeAddToolTipToTitle( label, this.fieldTooltip );
          }

          this.callSuper();
        },

        _findTitle : function ( fieldContainer, selector )
        {
          var matchedEls = fieldContainer.getEl().select( selector );
          if ( matchedEls )
          {
            var matchedLabelEls = matchedEls.child( 'label' );
            // we need to get the first matched label - this is because a fieldcontainer can contain a nested fieldcontainer that also contains the label...
            return matchedLabelEls ? matchedLabelEls.first() : null;
          }
          return null;
        },

        _maybeAddToolTipToTitle : function ( label, tooltipText )
        {
          if ( label )
          {
            label.createChild( {
              tag : 'img',
              src : 'images/icon/tooltip-10x10.png',
              style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
              'data-qtip' : Ext.htmlEncode( tooltipText )
            } );
          }
        }
      } );


      /*
       * Tooltip in title support for grid panels.
       * Inspired by http://www.sencha.com/forum/showthread.php?19301
       */
      Ext.override( Ext.grid.Panel, {
        afterRender : function ()
        {
          var title = null;
          if ( this.tooltip )
          {
            title = this._findTitle( this, 'div.x-panel > .x-panel-header .x-panel-header-title div.x-title-text' );
            this._maybeAddToolTipToTitle( title, this.tooltip );
          }

          this.callSuper();
        },

        _findTitle : function ( panel, selector )
        {
          var matchedEls = panel.getEl().select( selector );
          if ( matchedEls )
          {
            var matchedLabelEls = matchedEls.child( 'div' );
            // we need to get the first matched div
            return matchedLabelEls ? matchedLabelEls.first() : null;
          }
          return null;
        },

        _maybeAddToolTipToTitle : function ( title, tooltipText )
        {
          if ( title )
          {
            title.createChild( {
              tag : 'img',
              src : 'images/icon/tooltip-10x10.png',
              style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
              'data-qtip' : Ext.htmlEncode( tooltipText )
            } );
          }
        }
      } );


      /**
       * (1) Custom readOnlyCls config property value.
       */
      Ext.override( Ext.form.field.Display, {
        readOnlyCls : '' /* This ensures that all displayfields will have opacity = 1 */
      } );


      /**
       * (1) New setCursorPosition and getCursorPosition methods for textfield, textarea etc.
       * http://www.sencha.com/forum/showthread.php?95486-Cursor-Position-in-TextField
       */
      Ext.override( Ext.form.field.Text, {
        setCaretPosition : function ( pos )
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            el.focus();
            el.setSelectionRange( pos, pos );
          }
          else if ( el.createTextRange )
          {
            var range = el.createTextRange();
            range.move( "character", pos );
            range.select();
          }
          else
          {
            //throw 'setCaretPosition() not supported';
          }
        },

        getCaretPosition : function ()
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            return el.selectionStart;
          }
          else if ( document.selection && el.createTextRange )
          {
            var range = document.selection.createRange();
            range.collapse( true );
            range.moveStart( "character", -el.value.length );
            return range.text.length;
          }
          else
          {
            //throw 'getCaretPosition() not supported';
          }
        }
      } );


      /**
       * Localization of Ext.ux.TreeIconCombo texts.
       */
      Ext.override( Ext.ux.form.TreeIconCombo, {
        emptyValueText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        valueNotFoundText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        loadFailedText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadFailedText' ),
        loadingText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadingText' )
      } );


      /**
       * Default datetime format of the datetime value used by the DateTime component.
       */
      Ext.override( Ext.ux.form.field.DateTime, {
        dateTimeFormat : QDesk.util.constant.DateFormat.timestampFormatLong
      } );


      /**
       * Localization of the Chart Preview window title.
       */
      Ext.override( Ext.draw.ContainerBase, {
        preview : function ()
        {
          var image = this.getImage();
          new Ext.window.Window( {
            title : QDesk.Msg.get( 'Ext.draw.ContainerBase.previewWindowTitle' ),
            closeable : true,
            renderTo : Ext.getBody(),
            autoShow : true,
            maximizeable : true,
            maximized : true,
            border : true,
            layout : {
              type : 'hbox',
              pack : 'center',
              align : 'middle'
            },
            items : {
              xtype : 'container',
              items : {
                xtype : 'image',
                mode : 'img',
                cls : Ext.baseCSSPrefix + 'chart-image',
                src : image.data,
                listeners : {
                  afterrender : function ()
                  {
                    var me = this,
                        img = me.imgEl.dom,
                        ratio = image.type === 'svg' ? 1 : (window[ 'devicePixelRatio' ] || 1),
                        size;
                    if ( !img.naturalWidth || !img.naturalHeight )
                    {
                      img.onload = function ()
                      {
                        var width = img.naturalWidth,
                            height = img.naturalHeight;
                        me.setWidth( Math.floor( width / ratio ) );
                        me.setHeight( Math.floor( height / ratio ) );
                      };
                    }
                    else
                    {
                      size = me.getSize();
                      me.setWidth( Math.floor( size.width / ratio ) );
                      me.setHeight( Math.floor( size.height / ratio ) );
                    }
                  }
                }
              }
            }
          } );
        }
      } );


      /**
       * Added support for auto HTML-encoding of tooltip text in grid columns.
       */
      Ext.override( Ext.grid.column.Column, {
        constructor : function ( config )
        {
          var me = this;
          if ( config && config.tooltip )
          {
            Ext.apply( config, {
              tooltip : Ext.htmlEncode( config.tooltip )
            } );
          }

          return this.callSuper( [ config ] );
        }
      } );


      /* Start of Temporary fixes */

      /*
       * This fixes the problem with the time field ignoring the seconds and milliseconds by
       * setValue( Date ).
       *
       * Sencha does not seem to consider this a bug, but we do. This problem is reported at
       * http://www.sencha.com/forum/showthread.php?275972-TimeField-don-t-consider-the-seconds-and-milliseconds
       */

      Ext.override( Ext.form.field.Time, {
        getInitDate : function ( hours, minutes, seconds, milliseconds )
        {
          var parts = this.initDateParts;
          return new Date( parts[ 0 ], parts[ 1 ], parts[ 2 ], hours || 0, minutes || 0, seconds || 0,
              milliseconds || 0 );
        },

        setValue : function ( v )
        {
          this.getPicker();
          if ( Ext.isDate( v ) )
          {
            v = this.getInitDate( v.getHours(), v.getMinutes(), v.getSeconds(), v.getMilliseconds() );
          }
          return this.callSuper( [ v ] );
        }
      } );


      Ext.override( Ext.ux.IFrame, {
        cleanupListeners : function ( destroying )
        {
          var doc, prop;
          if ( this.rendered )
          {
            try
            {
              doc = this.getDoc();
              if ( doc )
              {
                // QuartzDesk: we need to check if doc.id is string, because some pages
                // (e.g. www.quartzdesk.com that uses mootools) set doc.id to a function.
                if ( Ext.isString( doc.id ) )
                {
                  Ext.get( doc ).un( this._docListeners );
                  if ( destroying )
                  {
                    for ( prop in doc )
                    {
                      if ( doc.hasOwnProperty && doc.hasOwnProperty( prop ) )
                      {
                        delete doc[ prop ];
                      }
                    }
                  }
                }
              }
            }
            catch ( e )
            {
            }
          }
        },

        onLoad : function ()
        {
          var me = this;
          var doc = me.getDoc();
          var fn = me.onRelayedEvent;

          if ( doc )
          {
            // QuartzDesk: we need to check if doc.id is string, because some pages
            // (e.g. www.quartzdesk.com that uses mootools) set doc.id to a function.
            if ( Ext.isString( doc.id ) )
            {
              try
              {
                // These events need to be relayed from the inner document (where they stop
                // bubbling) up to the outer document. This has to be done at the DOM level so
                // the event reaches listeners on elements like the document body. The effected
                // mechanisms that depend on this bubbling behavior are listed to the right
                // of the event.
                Ext.get( doc ).on( me._docListeners = {
                  mousedown : fn,
                  // menu dismisal (MenuManager) and Window onMouseDown (toFront)
                  mousemove : fn,
                  // window resize drag detection
                  mouseup : fn,
                  // window resize termination
                  click : fn,
                  // not sure, but just to be safe
                  dblclick : fn,
                  // not sure again
                  scope : me
                } );
              }
              catch ( e )
              {
              }
              // cannot do this xss
              // We need to be sure we remove all our events from the iframe on unload or we're going to LEAK!
              Ext.get( me.getWin() ).on( 'beforeunload', me.cleanupListeners, me );
            }

            me.el.unmask();
            me.fireEvent( 'load', me );
          }
          else if ( me.src )
          {
            me.el.unmask();
            me.fireEvent( 'error', me );
          }
        }
      } );

      /* End of Temporary fixes */

      /* End of ExtJS overrides */

      /**
       * Create the viewport.
       */
      this._createViewport();
    },


    /**
     * Invokes the heartbeat action to obtain user and license details, creates a new viewport
     * and fires the viewportCreated event.
     */
    _createViewport : function ()
    {
      var me = this;

      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.heartbeatAction,
        callback : function ( opts, success, resp )
        {
          // Remove the "Loading..." message box before the viewport gets shown.
          var loadingMsgBox = Ext.get( 'loadingMsgBox' );
          if ( loadingMsgBox )
          {
            loadingMsgBox.destroy();
          }

          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              /*
               * Important to update the current user's data.
               */
              QDesk.CurrentUser.update( json.sessionToken, json.user, true );

              var viewport = Ext.create( 'QDesk.view.Viewport', {
                qdesk : {
                  product : json.product,
                  license : json.license,
                  user : json.user
                }
              } );

              me.fireEvent( QDesk.util.constant.EventName.VIEWPORT_CREATED, {
                viewport : viewport
              } );

              return;
            }
          }

          // there is some error invoking the heartbeat action, show a warning dialog with a Retry button
          Ext.MessageBox.show( {
            title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleWarning' ),
            msg : QDesk.Msg.get( 'QDesk.errorStartingGuiMsg' ),
            buttons : Ext.MessageBox.OK,
            buttonText : {
              ok : QDesk.Msg.get( 'QDesk.buttonLabelRetry' )
            },
            icon : Ext.MessageBox.WARNING,
            closable : false,
            fn : function ()
            {
              window.location.reload();
            }
          } );
        }
      } );
    }
  } );
})();

