/*jslint sloppy: true */
/*global Ext: false */

/**
 * @class Ext.ux.form.field.DateTime
 * @extends Ext.form.FieldContainer
 * @version 0.4 (November 5th, 2012)
 * @author Ext Example Class: Ext.calendar.form.field.DateRange
 * @author [atian25](http://www.sencha.com/forum/member.php?51682-atian25)
 * @author [ontho](http://www.sencha.com/forum/member.php?285806-ontho)
 * @author [jakob.ketterl](http://www.sencha.com/forum/member.php?25102-jakob.ketterl)
 * [Forum Thread](http://www.sencha.com/forum/showthread.php?134345-Ext.ux.form.field.DateTime)
 */
Ext.define( 'Ext.ux.form.field.DateTime', {
  extend : 'Ext.form.FieldContainer',

  alias : 'widget.xdatetime',

  requires : [
    'Ext.form.field.Date',
    'Ext.form.field.Time'
  ],

  mixins : {
    field : 'Ext.form.field.Field'
  },

  //configurables
  combineErrors : false,
  msgTarget : 'under',
  layout : 'hbox',
  readOnly : false,
  allowBlank : true,

  /*
   * Initial value.
   */
  value : null,

  /**
   * @private
   * @type time period object
   */
  _oldValue : null,

  /**
   * @cfg {String} dateFormat
   * Convenience config for specifying the format of the date portion.
   * This value is overridden if format is specified in the dateConfig.
   * The default is 'Y-m-d'
   */
  dateFormat : 'Y-m-d',

  /**
   * @cfg {String} timeFormat
   * Convenience config for specifying the format of the time portion.
   * This value is overridden if format is specified in the timeConfig.
   * The default is 'H:i:s'
   */
  timeFormat : 'H:i:s',

  /**
   * @cfg {Object} dateConfig
   * Additional config options for the date field.
   */
  dateConfig : {},

  /**
   * @cfg {Object} timeConfig
   * Additional config options for the time field.
   */
  timeConfig : {},

  /**
   * @property dateField
   * @type Ext.form.field.Date
   */
  dateField : null,

  /**
   * @property timeField
   * @type Ext.form.field.Time
   */
  timeField : null,

  /**
   * @cfg {String} readOnlyCls
   * The CSS class applied to the component's main element when it is {@link #readOnly}.
   */
  readOnlyCls : Ext.baseCSSPrefix + 'form-readonly',

  /*
   * Important to set to '' to prevent ExtJS from applying x-item-disabled (opacity : 0.3) to this component when it
   * gets disabled. We want the x-item-disabled class to be applied only to the lowest-level components (individual
   * date and time fields), not to this upper-level container.
   */
  disabledCls : '',


  initComponent : function ()
  {
    var me = this;

    Ext.apply( me, {
      isFormField : true, //so it will be included in form field query's
      items : [
        Ext.apply( {
          xtype : 'datefield',
          format : me.dateFormat,
          altFormats : null,
          //flex : 1,
          isFormField : false, //exclude from field query's
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          },
          submitValue : false,
          allowBlank : me.allowBlank
        }, me.dateConfig ),

        Ext.apply( {
          xtype : 'timefield',
          format : me.timeFormat,
          altFormats : null,
          //flex : 1,
          isFormField : false, //exclude from field query's
          listeners : {
            change : function ()
            {
              me.onFieldChange();
            },
            scope : me
          },
          typeAhead : false,
          submitValue : false,
          allowBlank : me.allowBlank
        }, me.timeConfig )
      ]
    } );

    me.callParent();

    me.dateField = me.down( 'datefield' );
    me.timeField = me.down( 'timefield' );

    me.initField();
  },


  // private
  beforeDestroy : function ()
  {
    Ext.destroy( this.fieldCt );
    this.callParent( arguments );
  },


  // private
  delegateFn : function ( fn )
  {
    this.items.each( function ( item )
    {
      if ( item[ fn ] )
      {
        item[ fn ]();
      }
    } );
  },


  // @inheritdoc
  getErrors : function ()
  {
    return [].concat( this.dateField.getErrors() ).concat( this.timeField.getErrors() );
  },


  // @inheritdoc
  isValid : function ()
  {
    /*
     * Intentionally not using "this.dateField.isValid() && this.timeField.isValid()" because it would not invoke
     * this.timeField.isValid() is this.dateField.isValid() returned false. We need to validate both fields so that
     * their validation status (error highlighting) gets updated.
     */
    var dateValid = this.dateField.isValid();
    var timeValid = this.timeField.isValid();

    return dateValid && timeValid;
  },


  // @inheritdoc
  getSubmitValue : function ()
  {
    var format = this.dateTimeFormat || this.getFormat();
    var value = this.getValue();

    return value ? Ext.Date.format( value, format ) : null;
  },


  /**
   * Returns the value of this datetime field as a Date.
   * @return {Date}
   */
  getValue : function ()
  {
    var me = this;

    var date = me.dateField.getSubmitValue();
    var time = me.timeField.getSubmitValue();
    var value = null;

    if ( this.isValid() )
    {
      if ( date && time )
      {
        value = Ext.Date.parse( date + ' ' + time, me.getFormat() );
      }
    }

    return value;
  },


  // @inheritdoc
  setValue : function ( value )
  {
    var format;

    if ( Ext.isString( value ) )
    {
      format = this.dateTimeFormat || this.getFormat();
      value = Ext.Date.parse( value, format ); //this.dateTimeFormat
    }
    this.dateField.setValue( value );
    this.timeField.setValue( value );
  },


  // @inheritdoc
  isDirty : function ()
  {
    var dirty = false;
    if ( this.rendered && !this.disabled )
    {
      this.items.each( function ( item )
      {
        if ( item.isDirty() )
        {
          dirty = true;
          return false;
        }
      } );
    }
    return dirty;
  },


  // @inheritdoc
  reset : function ()
  {
    this.delegateFn( 'reset' );
  },


  // @inheritdoc
  resetOriginalValue : function ()
  {
    this.dateField.resetOriginalValue();
    this.timeField.resetOriginalValue();
  },


  // @inheritdoc
  markInvalid : function ( errors )
  {
    this.dateField.markInvalid( errors );
    this.timeField.markInvalid( errors );
  },


  // @inheritdoc
  clearInvalid : function ( errors )
  {
    this.dateField.clearInvalid( errors );
    this.timeField.clearInvalid( errors );
  },


  // @inheritdoc
  setReadOnly : function ( readOnly )
  {
    this.dateField.setReadOnly( readOnly );
    this.timeField.setReadOnly( readOnly );
    this[ readOnly ? 'addCls' : 'removeCls' ]( this.readOnlyCls );
    this.readOnly = readOnly;
  },


  // @inheritdoc
  setValidation : function ( error )
  {
    this.dateField.setValidation( error );
    this.timeField.setValidation( error );
  },


  // private
  getFormat : function ()
  {
    var df = this.dateField,
        tf = this.timeField;
    return ((df.submitFormat || df.format) + " " + (tf.submitFormat || tf.format));
  },


  // private
  onDisable : function ()
  {
    this.delegateFn( 'disable' );
  },


  // private
  onEnable : function ()
  {
    this.delegateFn( 'enable' );
  },


  // private
  onFieldChange : function ()
  {
    var newValue = this.getValue();
    this.fireEvent( 'change', this, newValue, this._oldValue );
    this._oldValue = newValue;
  }
} );