/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens the License window.
 */
Ext.define( 'QDesk.controller.command.ViewLicenseInfo', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',
    'QDesk.view.work.license.LicenseWindow'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.ViewLicenseInfo.maskLicenseDataLoad' ) );

    var licenseWindow = Ext.widget( 'LicenseWindow' );

    var generalInfoStore = licenseWindow.down( '#generalInfoTree' ).getStore();
    var featuresInfoStore = licenseWindow.down( '#featuresInfoTree' ).getStore();

    // when all stores are loaded, show the license window, or the error dialog..
    QDesk.util.StoreUtils.onAllStoresLoad( [ generalInfoStore, featuresInfoStore ],
        function ( successful, firstFailedOperation )
        {
          QDesk.util.UxUtils.unmaskBody();

          if ( successful )
          {
            licenseWindow.show();
          }
          else
          {
            var response = firstFailedOperation.getResponse();
            if ( response && response.responseText )
            {
              // application errors
              QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
            }
            else
            {
              QDesk.util.UxUtils.showErrorDialog(
                  QDesk.Msg.get( 'QDesk.controller.command.ViewLicenseInfo.errorLicenseDataLoad' ) );
            }
          }
        }, me );

    licenseWindow.down( '#closeButton' ).on( 'click', function ()
    {
      licenseWindow.close();
    } );
  }
} );
