/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified connection and refreshes (reloads) its parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.DeleteSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /*
   * Required ID of the connection to delete.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.connectionDelete' ) )
    {
      var connectionName = QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.confirmMsg',
              Ext.String.htmlEncode( connectionName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteConnection();
            }
          }
      );
    }
    else
    {
      me._deleteConnection();
    }
  },


  _deleteConnection : function ()
  {
    var me = this;

    var connectionName = QDesk.controller.command.connection.ConnectionUtils.getConnectionName( me.connectionId );

    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.maskMsg',
        Ext.String.htmlEncode( connectionName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionDeleteConnection,
      params : {
        connectionId : me.connectionId
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me.application.fireEvent( QDesk.util.constant.EventName.CONNECTION_DELETED, {
              connectionId : me.connectionId
            } );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.successMsg',
                    Ext.String.htmlEncode( connectionName ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.DeleteSchedulerConnection.errorMsg',
                  Ext.String.htmlEncode( connectionName ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }

        // reload the connection's parent folder in the connections tree view
        var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getConnectionParentFolderId(
            me.connectionId );
        if ( parentFolderId )
        {
          Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
            folderId : parentFolderId
          } ).execute();
        }
        else
        {
          Ext.Error.raise( 'Parent folder ID for connection ID: ' + me.connectionId + ' not found.' );
        }
      }
    } );
  }
} );
