/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the name of an existing folder in the connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.EditConnectionFolder', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.EventName',
    'QDesk.view.work.navigation.connections.EditConnectionFolderWindow'
  ],

  /*
   * Required ID of the folder that is edited.
   */
  folderId : undefined,

  /**
   * Required config parameter with the existing folder name.
   */
  folderName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    if ( !config.folderName )
      Ext.Error.raise( 'Required config parameter missing: folderName' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditFolderWindow', {
      qdesk : {
        mode : QDesk.view.work.navigation.connections.EditConnectionFolderFormMode.EDIT
      }
    } );

    var formPanel = window.down( '#form' );

    formPanel.on( 'afterrender', function ()
    {
      var folderNameField = formPanel.down( '#folderName' );
      folderNameField.setValue( me.folderName );
    } );

    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      var form = formPanel.getForm();
      var folderName = formPanel.down( '#folderName' ).getValue();

      form.doAction( 'ParamsSubmit', {
        url : QDesk.util.constant.ActionUrl.connectionManagementActionUpdateFolder,
        params : {
          folderId : me.folderId,
          folderName : folderName
        },
        waitMsg : QDesk.Msg.get( 'QDesk.controller.command.connection.EditConnectionFolder.waitMsg',
            Ext.String.htmlEncode( folderName ) ),
        success : function ( form, action )
        {
          var json = Ext.decode( action.response.responseText );
          if ( json.success )
          {
            window.close();

            me.application.fireEvent( QDesk.util.constant.EventName.CONNECTION_FLD_UPDATED, {
              folderId : me.folderId
            } );

            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.connection.EditConnectionFolder.successMsg',
                    Ext.String.htmlEncode( folderName ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );

            me._refreshTargetFolder();
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        },
        failure : function ( form, action )
        {
          QDesk.util.UxUtils.showExtJsActionError( action,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      } );
    } )
    ;

    window.show();
  },


  /**
   * Reloads the connection's parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    // reload the connection's parent folder in the connections tree view
    var parentFolderId = QDesk.controller.command.connection.ConnectionUtils.getFolderParentFolderId( me.folderId );
    if ( parentFolderId )
    {
      Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
        folderId : parentFolderId
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Parent folder ID for folder ID: ' + me.folderId + ' not found.' );
    }
  }
} );
