/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Tests a scheduler connection with the specified ID.
 */
Ext.define( 'QDesk.controller.command.connection.TestSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.gui.ServerErrorType',
    'QDesk.util.UxUtils',
    'QDesk.controller.command.connection.ConnectionUtils'
  ],

  /*
   * Required ID of the connection to move.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.model.scheduler.SchedulerType.
   */
  schedulerType : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.TestSchedulerConnection.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerActionTestConnection,
      params : {
        connectionId : me.connectionId

      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.controller.command.connection.ConnectionUtils.handleTestResponse( json, me.schedulerType );
          }
          else
          {
            // application errors are shown in a warning dialog
            if ( json.errorType && json.errorType == QDesk.model.gui.ServerErrorType.APPLICATION )
            {
              QDesk.util.UxUtils.showWarningDialog( json.errors[ 0 ].msg );
            }
            else
            {
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.TestSchedulerConnection.errorMsg' ) );
        }
      }
    } );
  }
} );
