/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Tests a scheduler connection.
 */
Ext.define( 'QDesk.controller.command.connection.TestSchedulerConnectionData', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.gui.ServerErrorType',
    'QDesk.util.UxUtils',
    'QDesk.controller.command.connection.ConnectionUtils'
  ],

  /*
   * Required config parameter with the SchedulerConnectionDataForm form panel
   * initiating the test.
   */
  form : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.model.scheduler.SchedulerType.
   */
  schedulerType : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.form )
      Ext.Error.raise( 'Required config parameter missing: form' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    Ext.apply( config, {
      maskEl : config.form.up( 'window' ) ? config.form.up( 'window' ).getEl() : config.form.getEl()
    } );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var useServiceParams = me.form.down( '#serviceParamsRadio' ).getValue();
    var useServiceUrl = me.form.down( '#serviceUrlRadio' ).getValue();

    var fieldValues = me.form.getForm().getFieldValues();

    var connection = Ext.create( 'QDesk.model.scheduler.SchedulerConnection',
        {
          //createdAt : fieldValues.createdAt,
          //updatedAt : fieldValues.updatedAt,
          name : fieldValues.name,
          remoteJmxService : {
            protocol : useServiceParams ? fieldValues.protocol : null,
            host : useServiceParams ? fieldValues.host : null,
            port : useServiceParams ? fieldValues.port : null,
            serviceUrl : useServiceUrl ? fieldValues.serviceUrl : null,
            authentication : {
              username : fieldValues.username ? fieldValues.username : null,
              password : fieldValues.password ? fieldValues.password : null
            },
            secure : fieldValues.secure == null ? false : fieldValues.secure
          },
          schedulerType : me.schedulerType,
          schedulerObjectName : fieldValues.schedulerObjectName,
          configEntry : []
        }
    );

    if ( fieldValues.username == '' && fieldValues.password == '' )
      connection.get( 'remoteJmxService' ).authentication = null;

    /*
     * Important to drop id property from the data to avoid JSON deserialization issues on the server-side.
     */
    var connectionData = connection.getData();
    delete connectionData.id;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.connection.TestSchedulerConnectionData.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.schedulerActionTestConnectionData,
      params : {
        connection : Ext.encode( connectionData )

      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.controller.command.connection.ConnectionUtils.handleTestResponse( json, me.schedulerType );
          }
          else
          {
            // application errors are shown in a warning dialog
            if ( json.errorType && json.errorType == QDesk.model.gui.ServerErrorType.APPLICATION )
            {
              QDesk.util.UxUtils.showWarningDialog( json.errors[ 0 ].msg );
            }
            else
            {
              QDesk.util.UxUtils.showJsonResponseErrors( json );
            }
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.TestSchedulerConnectionData.errorMsg' ) );
        }
      }
    } );
  }
} );
