/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Processes the specified heartbeat control commands.
 */
Ext.define( 'QDesk.controller.command.heartbeat.ProcessControlCommands', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.gui.ControlCommandType'
  ],


  /*
   * Required array containing heartbeat control commands to process.
   */
  commands : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.commands )
      Ext.Error.raise( 'Required config parameter missing: commands' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];
    var lockRequested = false;

    if ( viewport )
    {
      Ext.each( me.commands, function ( command )
      {
        // highest priority lock first
        if ( !lockRequested && command.type === QDesk.model.gui.ControlCommandType.LOCK_LICENSE_EXPIRED )
        {
          viewport.lock( me._getCommandParamValue( command, "message" ) );
          lockRequested = true;
        }
        else if ( !lockRequested && command.type === QDesk.model.gui.ControlCommandType.LOCK_UNDERLICENSED )
        {
          viewport.lock( me._getCommandParamValue( command, "message" ) );
          lockRequested = true;
        }
      } );

      /*
       * If no lock was requested, unlock the GUI.
       */
      if ( !lockRequested )
      {
        viewport.unlock();
      }
    }
  },


  /**
   * Returns the value of the specified command parameter name.
   * @param command a command object.
   * @param paramName a parameter name.
   * @returns the command parameter value or null if no such parameter exists.
   * @private
   */
  _getCommandParamValue : function ( command, paramName )
  {
    var value = null;

    if ( !Ext.isEmpty( command.param ) )
    {
      Ext.each( command.param, function ( param )
      {
        if ( param.name === paramName )
        {
          value = param.value;
          return false;
        }
      } );
    }

    return value;
  }
} );
