/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the specified message channel profile.
 */
Ext.define( 'QDesk.controller.command.message.channel.EditMessageChannelProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder',
    'QDesk.controller.command.message.channel.SendTestMessage',
    'QDesk.util.CommonUtils',
    'QDesk.util.MessageChannelProfileUtils',
    'QDesk.util.UxUtils'
  ],

  /*
   * Required ID of the message channel profile to edit.
   */
  profileId : undefined,

  /**
   * Required message channel value (QDesk.model.message.channel.MessageChannel).
   */
  messageChannel : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.profileId )
      Ext.Error.raise( 'Required config parameter missing: profileId' );

    if ( !config.messageChannel )
      Ext.Error.raise( 'Required config parameter missing: messageChannel' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    QDesk.util.MessageChannelProfileUtils.cleanupEditPanel();

    var formPanel = QDesk.util.MessageChannelProfileUtils.createEmptyProfilePanel( me.messageChannel );

    formPanel.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      // QDesk.util.UxUtils.focus( nameField, true );
    } );

    // Send Test Message button
    formPanel.down( '#sendTestMessageButton' ).on( 'click', function ()
    {
      me._sendTestMessage();
    } );

    // Apply button
    formPanel.down( '#saveButton' ).on( 'click', function ()
    {
      me._submitForm();
    } );

    /* Load the form data */
    me.mask( QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfile.maskLoadMsg' ) );

    formPanel.load( {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionGetProfile,
      scope : me,
      params : {
        profileId : me.profileId
      },
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          formPanel.populate.apply( formPanel, [ json.data ] );
          QDesk.util.MessageChannelProfileUtils.addAndShowPanel( formPanel );
          me.unmask();
        }
        else
        {
          me.unmask();
          QDesk.util.UxUtils.showJsonResponseErrors( json );
        }
      },
      failure : function ( form, action )
      {
        me.unmask();
        QDesk.util.UxUtils.showExtJsActionError( action );
      }
    } );
  },


  _sendTestMessage : function ()
  {
    var me = this;

    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();

    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    // model.MessageChannelProfile populated with message channel specific values
    var messageChannelProfile = formPanel.createMessageChannelProfile();

    var window = QDesk.util.MessageChannelProfileUtils.getProfilesWindow();

    Ext.create( 'QDesk.controller.command.message.channel.SendTestMessage', {
      maskEl : window.getEl(),
      profile : messageChannelProfile
    } ).execute();
  },


  _submitForm : function ()
  {
    var me = this;

    var editPanel = QDesk.util.MessageChannelProfileUtils.getEditPanel();

    var formPanel = editPanel.down( 'form' );
    var form = formPanel.getForm();

    // model.MessageChannelProfile populated with message channel specific values
    var messageChannelProfile = formPanel.createMessageChannelProfile();

    var name = messageChannelProfile.get( 'name' );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.messageChannelProfileActionUpdateProfile,
      params : {
        profile : QDesk.util.CommonUtils.encode( messageChannelProfile )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfile.waitMsg',
          Ext.String.htmlEncode( name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          me._refreshTargetFolder();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.message.channel.EditMessageChannelProfile.successMsg',
                  Ext.String.htmlEncode( name ) ),
              function ()
              {
                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  },


  /**
   * Refreshes the parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    var parentFolderId = QDesk.util.MessageChannelProfileUtils.getProfileParentFolderId( me.profileId );
    if ( parentFolderId )
    {
      Ext.create( 'QDesk.controller.command.message.channel.RefreshMessageChannelProfileFolder', {
        folderId : parentFolderId
      } ).execute();
    }
    else
    {
      Ext.Error.raise( 'Parent folder ID for profile ID: ' + me.profileId + ' not found.' );
    }
  }
} );
