/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Various Quartz-specific controller related utility methods.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils', {
  singleton : true,

  requires : [
    'QDesk.controller.command.connection.ConnectionUtils'
  ],


  /**
   * Returns the Jobs tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getJobsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var jobsTab = schedulerView.down( '#jobsTab' );
    if ( !jobsTab )
      Ext.Error.raise( 'Jobs tab for connection ID: ' + me.connectionId + ' not found.' );

    return jobsTab;
  },


  /**
   * Returns the Currently Executing Jobs tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getExecutingJobsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var executingJobsTab = schedulerView.down( '#executingJobsTab' );
    if ( !executingJobsTab )
      Ext.Error.raise( 'Executing jobs tab for connection ID: ' + me.connectionId + ' not found.' );

    return executingJobsTab;
  },


  /**
   * Returns the Planned Job Executions tab the specified connection ID.
   * @param connectionId a connection ID.
   */
  getPlannedJobExecutionsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    var plannedJobExecutionsTab = schedulerView.down( '#plannedJobExecutionsTab' );
    if ( !plannedJobExecutionsTab )
      Ext.Error.raise( 'Planned job executions tab for connection ID: ' + me.connectionId + ' not found.' );

    return plannedJobExecutionsTab;
  },


  /**
   * Returns the Triggers tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getTriggersTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#triggersTab' );
  },


  /**
   * Returns the Scheduler Misfired Triggers tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerMisfiredTriggersTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#schedulerMisfiredTriggersTab' );
  },


  /**
   * Returns the Info tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getInfoTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#infoTab' );
  },


  /**
   * Returns the Scheduler Execution History tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerExecHistoryTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#schedulerExecHistoryTab' );
  },


  /**
   * Returns the Scheduler Statistics tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerStatisticsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#schedulerStatisticsTab' );
  },


  /**
   * Returns the Scheduler Notifications tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerNotificationsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#schedulerNotificationsTab' );
  },


  /**
   * Returns the Scheduler Job Chains tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerJobChainsTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#schedulerJobChainsTab' );
  },


  /**
   * Returns the Scheduler Audit Trail tab for the specified connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerAuditTrailTab : function ( connectionId )
  {
    var me = this;

    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( connectionId );
    if ( !schedulerView )
      Ext.Error.raise( 'Scheduler view for connection ID: ' + connectionId + ' not found.' );

    return schedulerView.down( '#schedulerAuditTrailTab' );
  }
} );
