/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the configured job data map.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.model.scheduler.SchedulerStatus',
    'QDesk.model.scheduler.quartz.QuartzJobDataMapEntry',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.work.scheduler.quartz.JobDataMapWindow'
  ],

  /*
   * Required config property containing the job data map.
   */
  jobDataMap : undefined,

  /**
   * Optional config property containing the merged job data map.
   */
  mergedJobDataMap : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.jobDataMap )
      Ext.Error.raise( 'Required config parameter missing: jobDataMap' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'QuartzJobDataMapWindow',
        {
          title : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap.windowTitle' ),

          qdesk : {
            jobDataMap : me.jobDataMap,
            mergedJobDataMap : me.mergedJobDataMap
          }
        } );

    window.down( '#closeButton' ).on( 'click', function ()
    {
      window.close();
    } );

    window.down( '#viewMergedJobDataMapCheckbox' ).on( 'change', me._onViewMergedJobDataMapCheckboxChange, me );

    window.show();
  },


  /**
   * Invoked when the View Merged Job Data Map checkbox selection changes.
   * @param checkbox the View Merged Job Data Map checkbox.
   * @param checked a checkbox selection status.
   */
  _onViewMergedJobDataMapCheckboxChange : function ( checkbox, checked )
  {
    var me = this;

    var jobDataMapGrid = checkbox.up( 'window' ).down( '#jobDataMapGrid' );

    var jobDataMap = jobDataMapGrid.qdesk.jobDataMap;
    var mergedJobDataMap = jobDataMapGrid.qdesk.mergedJobDataMap;

    var records;

    if ( checked )
    {
      records = me._createModelRecords( mergedJobDataMap );
    }
    else
    {
      records = me._createModelRecords( jobDataMap );
    }

    jobDataMapGrid.getStore().loadData( records );
  },


  /**
   * Converts the specified array of JSON instances representing job data map entries
   * to an array of {@code QDesk.model.scheduler.quartz.QuartzJobDataMapEntry} instances.
   * @param rawData raw data.
   * @return the created model records.
   */
  _createModelRecords : function ( rawData )
  {
    var records = [];
    Ext.each( rawData, function ( entry )
    {
      records.push(
          Ext.create( 'QDesk.model.scheduler.quartz.QuartzJobDataMapEntry', {
            name : entry.name,
            value : entry.value
          } )
      );
    } );

    return records;
  }
} );
