/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the scheduler Audit Trail grid in the scheduler Audit Trail panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.audit.RefreshSchedulerAuditTrailGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;

    if ( licensedFeatures.AuditTrail )
    {
      var schedulerAuditTrailTab =
          QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getSchedulerAuditTrailTab( me.connectionId );

      var auditTrailGrid = schedulerAuditTrailTab.down( '#auditTrailGrid' );
      var auditTrailGridStore = auditTrailGrid.getStore();

      auditTrailGridStore.getProxy().url =
          QDesk.util.constant.ActionUrl.quartz.auditTrailActionListForScheduler;

      var extraStoreParams = {
        connectionId : me.connectionId
      };

      var fetchPeriod = auditTrailGrid.down( '#fetchPeriod' ).getValue();
      Ext.apply( extraStoreParams, {
        fetchPeriod : Ext.encode( fetchPeriod )
      } );

      var maxFetchSize = auditTrailGrid.down( '#maxFetchSizeSelect' ).getValue();
      if ( maxFetchSize != -1 )  // -1 == unlimited
      {
        Ext.apply( extraStoreParams, {
          maxFetchSize : maxFetchSize
        } );
      }

      // if audit trail index enabled, we add the filter query text param
      // TODO: check the index is enabled
      if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobChainsIndexEnabled(
              me.connectionId ) )
      {
        var filterQueryText = auditTrailGrid.down( '#filterQueryText' ).getValue();
        if ( !Ext.isEmpty( filterQueryText ) )
        {
          Ext.apply( extraStoreParams, {
            filterQuery : filterQueryText
          } );
        }
      }

      auditTrailGridStore.getProxy().extraParams = extraStoreParams;

      auditTrailGridStore.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
