/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Info tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.connection.EditSchedulerConnection',
    'QDesk.controller.command.connection.TestSchedulerConnection',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.model.scheduler.SchedulerType'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var infoTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getInfoTab( me.connectionId );

    me._bindEventHandlers( infoTab );

    infoTab.setInitialized( true );
  },


  _bindEventHandlers : function ( infoTab )
  {
    var me = this;

    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;

    var testConnectionButton = infoTab.down( '#connectionInfoView #testButton' );
    testConnectionButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.connection.TestSchedulerConnection', {
        connectionId : me.connectionId,
        schedulerType : QDesk.model.scheduler.SchedulerType.QUARTZ
      } ).execute();
    } );

    var editConnectionButton = infoTab.down( '#connectionInfoView #editButton' );
    editConnectionButton.on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.connection.EditSchedulerConnection', {
        connectionId : me.connectionId,
        schedulerType : QDesk.model.scheduler.SchedulerType.QUARTZ
      } ).execute();
    } );

    if ( licensedProductEdition == QDesk.model.license.ProductEdition.LITE )
    {
      // do nothing
    }
    else
    {
      var jobCountField = infoTab.down( '#schedulerInfoView #jobCount' );
      jobCountField.on( 'valueClick', function ( field )
      {
        var schedulerView = infoTab.up( 'QuartzSchedulerView' );
        var jobsTab = schedulerView.down( '#jobsTab' );
        schedulerView.setActiveTab( jobsTab );
      } );

      var triggerCountField = infoTab.down( '#schedulerInfoView #triggerCount' );
      triggerCountField.on( 'valueClick', function ( field )
      {
        var schedulerView = infoTab.up( 'QuartzSchedulerView' );
        var triggersTab = schedulerView.down( '#triggersTab' );
        schedulerView.setActiveTab( triggersTab );
      } );
    }
  }
} );
