/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a job editor and saves the data as a new job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.scheduler.quartz.QuartzJobDetail',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName',
    'QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode',
    'QDesk.view.work.scheduler.quartz.jobs.JobDetailWindow'
  ],

  /*
   * Required ID of the Quartz scheduler connection.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob.maskLoadMsg' ) );

    // empty job detail
    var jobDetail = Ext.create( 'QDesk.model.scheduler.quartz.QuartzJobDetail' );

    /* by default job details are durable */
    jobDetail.set( 'durability', true );

    var formWindow = Ext.widget( 'QuartzJobDetailWindow', {
      title : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob.windowTitle' ),

      qdesk : {
        connectionId : me.connectionId,
        jobDetailFormMode : QDesk.view.work.scheduler.quartz.jobs.JobDetailFormMode.ADD,
        jobDetail : jobDetail,
        callback : function ( jobDetail )
        {
          var jobGroupName = jobDetail.get( 'group' ).name;
          var jobName = jobDetail.get( 'name' );

          /*
           * Important to drop id property from the data to avoid JSOn deserialization issues on the server-side.
           */
          var jobDetailData = jobDetail.getData();
          delete jobDetailData.id;

          var form = formWindow.down( '#jobDetailForm' ).getForm();
          form.doAction( 'ParamsSubmit', {
            url : QDesk.util.constant.ActionUrl.quartz.jobActionAdd,
            params : {
              connectionId : me.connectionId,
              jobDetail : Ext.encode( jobDetailData )
            },
            waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob.waitMsg',
                Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ),
            success : function ( form, action )
            {
              formWindow.close();

              QDesk.util.UxUtils.showInfo(
                  QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob.successMsg',
                      Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ),
                  function ()
                  {
                    me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.JOB_ADDED, {
                      connectionId : me.connectionId,
                      jobDetail : jobDetail
                    } );

                    // invoke the success callback handler
                    if ( me.success && Ext.isFunction( me.success ) )
                    {
                      me.success.apply( me );
                    }
                  } );
            },
            failure : function ( form, action )
            {
              QDesk.util.UxUtils.showExtJsActionError( action,
                  function ()
                  {
                    // invoke the failure callback handler
                    if ( me.failure && Ext.isFunction( me.failure ) )
                    {
                      me.failure.apply( me );
                    }
                  } );
            }
          } );
        },
        scope : me
      }
    } );

    var formStores = formWindow.getAllStores();

    /* Load the form data */
    QDesk.util.StoreUtils.onAllStoresLoad( formStores,
        function ( successful, firstFailedOperation )
        {
          me.unmask();

          if ( successful )
          {
            formWindow.show();
          }
          else
          {
            var response = firstFailedOperation.getResponse();
            if ( response && response.responseText )
            {
              // application errors
              QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
            }
            else
            {
              // error loading stores
              QDesk.util.UxUtils.showErrorDialog(
                  QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.AddJob.errorLoadMsg' ) );
            }
          }
        } );

    QDesk.util.StoreUtils.loadStores( formStores );
  }
} );
