/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the grid in the Planned Job Executions tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshPlannedJobExecutionsGrid', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.PlannedJobExecutions &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId ) )
    {
      var plannedJobExecutionsTab =
          QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getPlannedJobExecutionsTab( me.connectionId );

      var plannedJobExecutionsGrid = plannedJobExecutionsTab.down( '#plannedJobExecutionsGrid' );

      var store = plannedJobExecutionsGrid.getStore();

      // set defaults for reload
      store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      store.getProxy().extraParams = {};

      if ( plannedJobExecutionsGrid.filtersValid() )
      {
        store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.jobActionListPlannedJobExecutions;

        var extraStoreParams = {
          connectionId : me.connectionId,
          maxResults : 1000  // TODO: make configurable
        };

        var timePeriod = plannedJobExecutionsGrid.down( '#timePeriod' ).getValue();
        Ext.apply( extraStoreParams, {
          timePeriod : Ext.encode( timePeriod )
        } );

        // if job chains index enabled, we add the filter query text param
        if ( QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobsIndexEnabled(
                me.connectionId ) )
        {
          var filterQueryText = plannedJobExecutionsGrid.down( '#filterQueryText' ).getValue();
          if ( !Ext.isEmpty( filterQueryText ) )
          {
            Ext.apply( extraStoreParams, {
              filterQuery : filterQueryText
            } );
          }
        }

        store.getProxy().extraParams = extraStoreParams;
      }

      store.loadPage( 1, {
        scope : me,
        callback : function ( records, operation, success )
        {
          if ( success )
          {
            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            // invoke the failure callback handler
            if ( me.failure && Ext.isFunction( me.failure ) )
            {
              me.failure.apply( me );
            }
          }
        }
      } );
    }
  }
} );
