/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds a new notification rule.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.scheduler.SchedulerType',
    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.util.constant.EventName',
    'QDesk.util.quartz.QuartzExecNotificationRuleUtils',
    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode',
    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleWindow'
  ],

  /*
   * Required ID of the scheduler connection.
   */
  connectionId : undefined,

  /**
   * Optional job group name.
   */
  jobGroupName : undefined,

  /**
   * Optional job name.
   */
  jobName : undefined,

  /**
   * Optional trigger group name.
   */
  triggerGroupName : undefined,

  /**
   * Optional trigger name.
   */
  triggerName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditExecNotificationRuleWindow', {
      qdesk : {
        connectionId : me.connectionId,
        mode : QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.ADD,
        schedulerType : QDesk.model.scheduler.SchedulerType.QUARTZ
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      QDesk.util.UxUtils.focus( nameField, true );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    formPanel.down( '#jobGroupName' ).setValue( me.jobGroupName );
    formPanel.down( '#jobName' ).setValue( me.jobName );

    formPanel.down( '#triggerGroupName' ).setValue( me.triggerGroupName );
    formPanel.down( '#triggerName' ).setValue( me.triggerName );

    window.show();
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();

    var notificationRule = QDesk.util.quartz.QuartzExecNotificationRuleUtils.createNotificationRule( fieldValues );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionAdd,
      params : {
        connectionId : me.connectionId,
        notificationRule : QDesk.util.CommonUtils.encode( notificationRule )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule.waitMsg',
          Ext.String.htmlEncode( fieldValues.name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule.successMsg',
                  Ext.String.htmlEncode( fieldValues.name ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.EXEC_NOTIF_RULE_ADDED, {
                  connectionId : me.connectionId,
                  notificationRule : notificationRule
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  }
} );
