/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified trigger.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /*
   * Required config property containing the connection ID.
   */
  connectionId : undefined,

  /**
   * Required config property containing the trigger to delete.
   */
  trigger : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.trigger )
      Ext.Error.raise( 'Required config parameter missing: trigger' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.triggerDelete' ) )
    {
      var triggerGroupName = me.trigger.get( 'group' ).name;
      var triggerName = me.trigger.get( 'name' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger.confirmMsg',
              Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteTrigger();
            }
          }
      );
    }
    else
    {
      me._deleteTrigger();
    }
  },


  _deleteTrigger : function ()
  {
    var me = this;

    var triggerGroupName = me.trigger.get( 'group' ).name;
    var triggerName = me.trigger.get( 'name' );

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger.maskMsg',
            Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.triggerActionDelete,
      params : {
        connectionId : me.connectionId,
        triggerGroupName : triggerGroupName,
        triggerName : triggerName
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger.successMsg',
                    Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }

                  me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.TRIGGER_DELETED, {
                    connectionId : me.connectionId,
                    triggerGroupName : triggerGroupName,
                    triggerName : triggerName
                  } );
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger.errorMsg',
                  Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
