/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Initializes the Triggers tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.PreviewChart',
    'QDesk.controller.command.SaveChart',
    'QDesk.controller.command.ViewError',
    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.SelectJob',
    'QDesk.controller.command.scheduler.quartz.ViewJobDataMap',

    'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory',
    'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog',
    'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryUserData',
    'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult',

    'QDesk.controller.command.scheduler.quartz.chain.AddJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EditJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain',
    'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail',

    'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule',
    'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail',

    'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.CloneTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.PauseTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshAuditTrailView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecHistoryView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecNotificationsView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshMisfiredTriggersView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshJobChainsView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView',
    'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger',
    'QDesk.controller.command.scheduler.quartz.triggers.ViewNextFireTimes',

    'QDesk.model.scheduler.quartz.QuartzTriggerGroupState',
    'QDesk.model.scheduler.quartz.QuartzTriggerState',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var triggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getTriggersTab( me.connectionId );

    me._bindEventHandlers( triggersTab );

    triggersTab.setInitialized( true );
  },


  /**
   * Binds events from the triggers tab to their respective handlers.
   */
  _bindEventHandlers : function ( triggersTab )
  {
    var me = this;
    me._bindEventHandlersTriggersGrid( triggersTab );

    // individual views
    me._bindEventHandlersHistoryView( triggersTab );
    me._bindEventHandlersMisfiredTriggersView( triggersTab );
    me._bindEventHandlersStatisticsView( triggersTab );
    me._bindEventHandlersNotificationsView( triggersTab );
    me._bindEventHandlersJobChainsView( triggersTab );
    me._bindEventHandlersAuditTrailView( triggersTab );
  },


  /**
   * Binds the triggers grid events to their handlers.
   */
  _bindEventHandlersTriggersGrid : function ( triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    // refresh button
    triggersGrid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshTriggersGrid( me.connectionId );
        } );

    var resumeTriggerButton = triggersGrid.down( '#resumeTriggerButton' );
    resumeTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._resumeTrigger( trigger );
    }, me );

    var pauseTriggerButton = triggersGrid.down( '#pauseTriggerButton' );
    pauseTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._pauseTrigger( trigger );
    }, me );

    var addTriggerButton = triggersGrid.down( '#addTriggerButton' );
    Ext.each( Ext.ComponentQuery.query( 'menuitem', addTriggerButton ),
        function ( menuitem )
        {
          menuitem.on( 'click', function ()
          {
            me._addTrigger( menuitem.qdesk.triggerType );
          }, me );
        } );

    var deleteTriggerButton = triggersGrid.down( '#deleteTriggerButton' );
    deleteTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._deleteTrigger( trigger );
    }, me );

    var editTriggerButton = triggersGrid.down( '#editTriggerButton' );
    editTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._editTrigger( trigger );
    }, me );

    var cloneTriggerButton = triggersGrid.down( '#cloneTriggerButton' );
    cloneTriggerButton.on( 'click', function ()
    {
      var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
      var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );
      me._cloneTrigger( trigger );
    }, me );

    var filterByTriggerGroupCombo = triggersGrid.down( '#filterByTriggerGroupCombo' );

    filterByTriggerGroupCombo.on( 'change',
        function ( combo, newValue )
        {
          if ( newValue == null )
          {
            resumeTriggerGroupButton.disable();
            pauseTriggerGroupButton.disable();
          }
          else
          {
            // based on the selected trigger's group state enable either the resume, or pause button
            var triggerGroup = filterByTriggerGroupCombo.getStore().findRecord( 'name', newValue );
            var triggerGroupState = triggerGroup.get( 'state' );

            switch ( triggerGroupState )
            {
              case QDesk.model.scheduler.quartz.QuartzTriggerGroupState.NORMAL:
                resumeTriggerGroupButton.disable();
                pauseTriggerGroupButton.enable();
                break;

              case QDesk.model.scheduler.quartz.QuartzTriggerGroupState.PAUSED:
                resumeTriggerGroupButton.enable();
                pauseTriggerGroupButton.disable();
                break;

              default:
                Ext.Error.raise( 'Unexpected trigger group state: ' + triggerGroupState );
            }
          }

          triggersGrid.getStore().clearFilter( true );

          if ( newValue != null )
          {
            triggersGrid.getStore().addFilter( [
              Ext.create( 'Ext.util.Filter', {
                property : 'trigger.group.name',
                value : newValue
              } )
            ], false );
          }

          // apply, or clear the filter and load the first page
          triggersGrid.getStore().loadPage( 1 );
        } );

    filterByTriggerGroupCombo.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          if ( successful )
          {
            // ok
          }
          else
          {
            var response = operation.getResponse();
            if ( response && response.responseText )
            {
              // application errors
              QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
            }
            else
            {
              QDesk.util.UxUtils.showErrorDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingTriggerGroupsMsg' ) );
            }
          }
        } );

    var resumeTriggerGroupButton = triggersGrid.down( '#resumeTriggerGroupButton' );
    resumeTriggerGroupButton.on( 'click', function ()
    {
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._resumeTriggerGroup( triggerGroupName );
    }, me );

    var pauseTriggerGroupButton = triggersGrid.down( '#pauseTriggerGroupButton' );
    pauseTriggerGroupButton.on( 'click', function ()
    {
      var triggerGroupName = filterByTriggerGroupCombo.getValue();
      me._pauseTriggerGroup( triggerGroupName );
    }, me );

    var filterQueryText = triggersGrid.down( '#filterQueryText' );

    var triggersIndexEnabled = false;
    if ( jvmAgentReady )
    {
      if ( publicApiReady )
      {
        triggersIndexEnabled = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzTriggersIndexEnabled(
            me.connectionId );

        if ( !triggersIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.triggersIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexPublicApiNotReadyEmptyText' ) );
      }
    }
    else
    {
      filterQueryText.setEmptyText(
          QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexAgentNotReadyEmptyText' ) );
    }

    // event fired by the actioncolumn in the TriggersGrid
    triggersGrid.down( '#triggerJobDataMapColumn' ).on( 'requestViewJobDataMap',
        function ( jobDataMap, mergedJobDataMap )
        {
          me._viewJobDataMap( jobDataMap, mergedJobDataMap );
        } );

    // event fired by the actioncolumn in the TriggersGrid
    triggersGrid.down( '#triggerNextFireTimeColumn' ).on( 'requestViewNextFireTimes',
        function ( triggerGroupName, triggerName )
        {
          me._viewNextFireTimesOfTrigger( me.connectionId, triggerGroupName, triggerName );
        } );

    triggersGrid.getStore().on( 'beforeload',
        function ( store, operation )
        {
          triggersGrid.down( '#refreshButton' ).disable();
          triggersGrid.down( '#addTriggerButton' ).disable();

          filterByTriggerGroupCombo.disable();
          resumeTriggerGroupButton.disable();
          pauseTriggerGroupButton.disable();

          filterQueryText.disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          triggersGrid.getSelectionModel().deselectAll();
        } );

    triggersGrid.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          triggersGrid.down( '#refreshButton' ).enable();

          // enable only if the triggers index is enabled in the agent
          if ( triggersIndexEnabled )
          {
            filterQueryText.enable();
          }

          if ( successful )
          {
            // enable only if the JVM Agent and public API are ready
            if ( jvmAgentReady && publicApiReady )
            {
              triggersGrid.down( '#addTriggerButton' ).enable();
            }

            filterByTriggerGroupCombo.enable();
            var triggerGroupValue = filterByTriggerGroupCombo.getValue();
            if ( triggerGroupValue )
            {
              // based on the selected trigger group's state enable either the resume, or pause button
              var triggerGroup = filterByTriggerGroupCombo.findRecord( 'name', triggerGroupValue );
              var triggerGroupState = triggerGroup.get( 'state' );

              switch ( triggerGroupState )
              {
                case QDesk.model.scheduler.quartz.QuartzTriggerGroupState.NORMAL:
                  resumeTriggerGroupButton.disable();
                  pauseTriggerGroupButton.enable();
                  break;

                case QDesk.model.scheduler.quartz.QuartzTriggerGroupState.PAUSED:
                  resumeTriggerGroupButton.enable();
                  pauseTriggerGroupButton.disable();
                  break;

                default:
                  Ext.Error.raise( 'Unexpected trigger group state: ' + triggerGroupState );
              }
            }
          }
          else
          {
            store.removeAll();

            var response = operation.getResponse();
            if ( response && response.responseText )
            {
              // application errors
              QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
            }
            else
            {
              QDesk.util.UxUtils.showErrorDialog(
                  QDesk.Msg.get(
                      'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingTriggersMsg' ) );
            }
          }
        } );

    triggersGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
    {
      if ( selectedRecords.length == 0 )
      {
        me._onTriggerUnselected( triggersTab );
      }
      else
      {
        me._onTriggerSelected( selectedRecords[ 0 ], triggersTab );
      }
    } );

    /* JobTriggersGrid context menu actions */
    triggersGrid.on( 'resumeTrigger', me._resumeTrigger, me );
    triggersGrid.on( 'pauseTrigger', me._pauseTrigger, me );
    triggersGrid.on( 'deleteTrigger', me._deleteTrigger, me );
    triggersGrid.on( 'editTrigger', me._editTrigger, me );
    triggersGrid.on( 'cloneTrigger', me._cloneTrigger, me );
  },


  /**
   * Binds the trigger execution history panel events to their handlers.
   */
  _bindEventHandlersHistoryView : function ( triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var historyView = triggersTab.down( '#historyView' );
    historyView.on( 'activate', function ()
    {
      me._refreshHistoryView( me.connectionId, true );
      me._refreshMisfiredTriggersView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
      me._refreshAuditTrailView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionHistory && jvmAgentReady )
    {
      var execHistoryGrid = triggersTab.down( '#execHistoryGrid' );
      // assert execHistoryGrid != null - the trigger execution history grid should exist

      // refresh button
      execHistoryGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // export CSV button
      execHistoryGrid.down( '#exportCsvButton' ).on( 'click',
          function ()
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];

            var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger',
                triggerWithJobDetail.get( 'trigger' ) );

            me._exportExecHistory( me.connectionId, trigger );
          } );

      // filters changed
      execHistoryGrid.on( 'changeFilters',
          function ()
          {
            me._refreshHistoryView( me.connectionId, true );
          } );

      // event fired by the actioncolumn in the ExecHistoryGrid
      execHistoryGrid.down( '#jobDataMapColumn' ).on( 'requestViewJobDataMap',
          function ( jobDataMap )
          {
            me._viewJobDataMap( jobDataMap );
          } );

      execHistoryGrid.down( '#errorColumn' ).on( 'requestViewException',
          function ( exception )
          {
            me._viewException( me.connectionId, exception );
          } );

      execHistoryGrid.down( '#resultColumn' ).on( 'requestViewResult',
          function ( execHistoryId )
          {
            me._viewResult( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#userDataColumn' ).on( 'requestViewUserData',
          function ( execHistoryId )
          {
            me._viewUserData( me.connectionId, execHistoryId );
          } );

      execHistoryGrid.down( '#logColumn' ).on( 'requestViewLog',
          function ( execHistoryId )
          {
            me._viewLog( me.connectionId, execHistoryId );
          } );

      var filterQueryText = execHistoryGrid.down( '#filterQueryText' );

      var execHistoryIndexEnabled = false;
      if ( jvmAgentReady )
      {
        execHistoryIndexEnabled =
            QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecHistoryIndexEnabled(
                me.connectionId );

        if ( !execHistoryIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.execHistoryIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexAgentNotReadyEmptyText' ) );
      }

      execHistoryGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            execHistoryGrid.down( '#refreshButton' ).disable();
            execHistoryGrid.down( '#fetchPeriod' ).disable();
            execHistoryGrid.down( '#maxFetchSizeSelect' ).disable();
            filterQueryText.disable();
            execHistoryGrid.down( '#exportCsvButton' ).disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            execHistoryGrid.getSelectionModel().deselectAll();
          } );

      execHistoryGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

            if ( triggerSelected )
            {
              execHistoryGrid.down( '#refreshButton' ).enable();
              execHistoryGrid.down( '#fetchPeriod' ).enable();
              execHistoryGrid.down( '#maxFetchSizeSelect' ).enable();

              // enable only if the exec history index is enabled in the agent
              if ( execHistoryIndexEnabled )
              {
                filterQueryText.enable();
              }

              if ( jvmAgentReady )
              {
                execHistoryGrid.down( '#exportCsvButton' ).enable();
              }

              if ( successful )
              {
                // currently no logic
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              var response = operation.getResponse();
              if ( response && response.responseText )
              {
                // application errors
                QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
              }
              else
              {
                QDesk.util.UxUtils.showErrorDialog(
                    QDesk.Msg.get(
                        'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingExecHistoryMsg' ) );
              }
            }
          } );
    }
  },


  /**
   * Binds the trigger misfired triggers panel events to their handlers.
   */
  _bindEventHandlersMisfiredTriggersView : function ( triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var misfiredTriggersView = triggersTab.down( '#misfiredTriggersView' );
    misfiredTriggersView.on( 'activate', function ()
    {
      me._refreshMisfiredTriggersView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
      me._refreshAuditTrailView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.MisfiredTriggers && jvmAgentReady )
    {
      var misfiredTriggersGrid = triggersTab.down( '#misfiredTriggersGrid' );
      // assert misfiredTriggersGrid != null - the trigger misfired triggers grid should exist

      // refresh button
      misfiredTriggersGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshMisfiredTriggersView( me.connectionId, true );
          } );

      // filters changed
      misfiredTriggersGrid.on( 'changeFilters',
          function ()
          {
            me._refreshMisfiredTriggersView( me.connectionId, true );
          } );

      var filterQueryText = misfiredTriggersGrid.down( '#filterQueryText' );

      var misfiredTriggersIndexEnabled = false;
      if ( jvmAgentReady )
      {
        misfiredTriggersIndexEnabled =
            QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzMisfiredTriggersIndexEnabled(
                me.connectionId );

        if ( !misfiredTriggersIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.misfiredTriggersIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexAgentNotReadyEmptyText' ) );
      }

      misfiredTriggersGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            misfiredTriggersGrid.down( '#refreshButton' ).disable();
            misfiredTriggersGrid.down( '#fetchPeriod' ).disable();
            misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).disable();
            filterQueryText.disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            misfiredTriggersGrid.getSelectionModel().deselectAll();
          } );

      misfiredTriggersGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

            if ( triggerSelected )
            {
              misfiredTriggersGrid.down( '#refreshButton' ).enable();
              misfiredTriggersGrid.down( '#fetchPeriod' ).enable();
              misfiredTriggersGrid.down( '#maxFetchSizeSelect' ).enable();

              // enable only if the index is enabled in the agent
              if ( misfiredTriggersIndexEnabled )
              {
                filterQueryText.enable();
              }

              if ( successful )
              {
                // currently no logic
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              var response = operation.getResponse();
              if ( response && response.responseText )
              {
                // application errors
                QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
              }
              else
              {
                QDesk.util.UxUtils.showErrorDialog(
                    QDesk.Msg.get(
                        'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingMisfiredTriggersMsg' ) );
              }
            }
          } );
    }
  },


  /**
   * Binds the trigger statistics panel events to their handlers.
   */
  _bindEventHandlersStatisticsView : function ( triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var statisticsView = triggersTab.down( '#statisticsView' );
    statisticsView.on( 'activate', function ()
    {
      me._refreshStatisticsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshMisfiredTriggersView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
      me._refreshAuditTrailView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics && jvmAgentReady )
    {
      var statisticsPanel = statisticsView.down( '#statisticsPanel' );

      // refresh button
      statisticsPanel.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );

      // preview chart button
      statisticsPanel.down( '#previewChartButton' ).on( 'click',
          function ()
          {
            me._previewStatisticsChart( me.connectionId );
          } );

      // save chart button
      //statisticsPanel.down( '#saveChartButton #saveChartAsJpeg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/jpeg' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsPng' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/png' );
      //    } );
      //
      //statisticsPanel.down( '#saveChartButton #saveChartAsSvg' ).on( 'click',
      //    function ()
      //    {
      //      me._saveStatisticsChart( me.connectionId, 'image/svg+xml' );
      //    } );


      statisticsPanel.on( 'statisticsCriteriaChange',
          function ( statisticsType, periodLength, periodStart )
          {
            me._refreshStatisticsView( me.connectionId, true );
          } );
    }
  },


  /**
   * Binds the trigger notifications panel events to their handlers.
   */
  _bindEventHandlersNotificationsView : function ( triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var notificationsView = triggersTab.down( '#notificationsView' );
    notificationsView.on( 'activate', function ()
    {
      me._refreshExecNotificationsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshMisfiredTriggersView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
      me._refreshAuditTrailView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionNotifications && jvmAgentReady )
    {
      var execNotificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );

      // refresh button
      execNotificationRulesGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshExecNotificationsView( me.connectionId, true );
          } );

      execNotificationRulesGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onNotificationRuleUnselected( triggersTab );
        }
        else
        {
          me._onNotificationRuleSelected( selectedRecords[ 0 ], triggersTab );
        }
      } );

      var filterQueryText = execNotificationRulesGrid.down( '#filterQueryText' );

      var notificationRulesIndexEnabled = false;
      if ( jvmAgentReady )
      {
        notificationRulesIndexEnabled =
            QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzExecNotificationRulesIndexEnabled(
                me.connectionId );

        if ( !notificationRulesIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.notificationRulesIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexAgentNotReadyEmptyText' ) );
      }

      execNotificationRulesGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            execNotificationRulesGrid.down( '#refreshButton' ).disable();
            execNotificationRulesGrid.down( '#addButton' ).disable();
            execNotificationRulesGrid.down( '#deleteButton' ).disable();
            execNotificationRulesGrid.down( '#editButton' ).disable();
            execNotificationRulesGrid.down( '#cloneButton' ).disable();
            execNotificationRulesGrid.down( '#viewAuditTrailButton' ).disable();
            filterQueryText.disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            execNotificationRulesGrid.getSelectionModel().deselectAll();
          } );

      execNotificationRulesGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

            if ( triggerSelected )
            {
              execNotificationRulesGrid.down( '#refreshButton' ).enable();

              if ( successful )
              {
                execNotificationRulesGrid.down( '#addButton' ).enable();
              }

              // enable only if the notification rules index is enabled in the agent
              if ( notificationRulesIndexEnabled )
              {
                filterQueryText.enable();
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              var response = operation.getResponse();
              if ( response && response.responseText )
              {
                // application errors
                QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
              }
              else
              {
                QDesk.util.UxUtils.showErrorDialog(
                    QDesk.Msg.get(
                        'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingNotifRulesMsg' ) );
              }
            }
          } );

      execNotificationRulesGrid.on( 'addRule',
          function ()
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
            var trigger = triggerWithJobDetail.get( 'trigger' );

            Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.AddExecNotificationRule', {
              connectionId : me.connectionId,
              triggerGroupName : trigger.group.name,
              triggerName : trigger.name,
              /*
               * Upon success refresh the notifications view.
               */
              success : function ()
              {
                me._refreshExecNotificationsView( me.connectionId, true );
              }
            } ).execute();
          } );

      execNotificationRulesGrid.on( 'enableRule',
          function ( execNotifRule )
          {
            me._enableExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'disableRule',
          function ( execNotifRule )
          {
            me._disableExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'deleteRule',
          function ( execNotifRule )
          {
            me._deleteNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'editRule',
          function ( execNotifRule )
          {
            me._editExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'cloneRule',
          function ( execNotifRule )
          {
            me._cloneExecNotificationRule( execNotifRule );
          } );

      execNotificationRulesGrid.on( 'viewAuditTrail',
          function ( execNotifRule )
          {
            me._viewExecNotificationRuleAuditTrail( execNotifRule );
          } );

      // opens a window with messages enqueued by the notification rule
      execNotificationRulesGrid.down( '#messagesColumn' ).on( 'requestViewMessages',
          function ( execNotificationRuleId )
          {
            Ext.create( 'QDesk.controller.command.scheduler.common.message.ViewMessagesForCreator', {
              connectionId : me.connectionId,
              creatorId : execNotificationRuleId,
              creatorType : QDesk.model.message.MessageCreatorType.QUARTZ_EXEC_NOTIF_RULE
            } ).execute();
          } );
    }
  },


  /**
   * Binds the trigger job chains panel events to their handlers.
   */
  _bindEventHandlersJobChainsView : function ( triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var jobChainsView = triggersTab.down( '#jobChainsView' );
    jobChainsView.on( 'activate', function ()
    {
      me._refreshJobChainsView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshMisfiredTriggersView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
      me._refreshAuditTrailView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining && jvmAgentReady )
    {
      var jobChainsGrid = triggersTab.down( '#jobChainsGrid' );

      // refresh button
      jobChainsGrid.down( '#refreshButton' ).on( 'click',
          function ()
          {
            me._refreshJobChainsView( me.connectionId, true );
          } );

      jobChainsGrid.on( 'selectionchange', function ( selectionModel, selectedRecords )
      {
        if ( selectedRecords.length == 0 )
        {
          me._onJobChainUnselected( triggersTab );
        }
        else
        {
          me._onJobChainSelected( selectedRecords[ 0 ], triggersTab );
        }
      } );

      var filterQueryText = jobChainsGrid.down( '#filterQueryText' );

      var jobChainsIndexEnabled = false;
      if ( jvmAgentReady )
      {
        jobChainsIndexEnabled =
            QDesk.controller.command.connection.ConnectionUtils.isJvmAgentQuartzJobChainsIndexEnabled(
                me.connectionId );

        if ( !jobChainsIndexEnabled )
        {
          filterQueryText.setEmptyText(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.jobChainsIndexDisabledEmptyText' ) );
        }
      }
      else
      {
        filterQueryText.setEmptyText(
            QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.indexAgentNotReadyEmptyText' ) );
      }

      jobChainsGrid.getStore().on( 'beforeload',
          function ( store, operation )
          {
            jobChainsGrid.down( '#refreshButton' ).disable();
            jobChainsGrid.down( '#addButton' ).disable();
            jobChainsGrid.down( '#deleteButton' ).disable();
            jobChainsGrid.down( '#editButton' ).disable();
            jobChainsGrid.down( '#cloneButton' ).disable();
            jobChainsGrid.down( '#viewAuditTrailButton' ).disable();
            filterQueryText.disable();

            /*
             * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
             * Therefore we need to force-fire this event by clearing the selection.
             */
            jobChainsGrid.getSelectionModel().deselectAll();
          } );

      jobChainsGrid.getStore().on( 'load',
          function ( store, records, successful, operation )
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

            if ( triggerSelected )
            {
              jobChainsGrid.down( '#refreshButton' ).enable();

              if ( successful )
              {
                jobChainsGrid.down( '#addButton' ).enable();
              }

              // enable only if the job chains index is enabled in the agent
              if ( jobChainsIndexEnabled )
              {
                filterQueryText.enable();
              }
            }

            if ( successful )
            {
              // currently no logic
            }
            else
            {
              store.removeAll();

              var response = operation.getResponse();
              if ( response && response.responseText )
              {
                // application errors
                QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
              }
              else
              {
                QDesk.util.UxUtils.showErrorDialog(
                    QDesk.Msg.get(
                        'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.errorLoadingJobChainsMsg' ) );
              }
            }
          } );

      jobChainsGrid.on( 'addJobChain',
          function ()
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerWithJobDetail = triggersGrid.getSelectionModel().getSelection()[ 0 ];
            var trigger = triggerWithJobDetail.get( 'trigger' );

            Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.AddJobChain', {
              connectionId : me.connectionId,
              triggerGroupName : trigger.group.name,
              triggerName : trigger.name,
              /*
               * Upon success refresh the job chains view.
               */
              success : function ()
              {
                me._refreshJobChainsView( me.connectionId, true );
              }
            } ).execute();
          } );

      jobChainsGrid.on( 'enableJobChain',
          function ( jobChain )
          {
            me._enableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'disableJobChain',
          function ( jobChain )
          {
            me._disableJobChain( jobChain );
          } );

      jobChainsGrid.on( 'deleteJobChain',
          function ( jobChain )
          {
            me._deleteJobChain( jobChain );
          } );

      jobChainsGrid.on( 'editJobChain',
          function ( jobChain )
          {
            me._editJobChain( jobChain );
          } );

      jobChainsGrid.on( 'cloneJobChain',
          function ( jobChain )
          {
            me._cloneJobChain( jobChain );
          } );

      jobChainsGrid.on( 'viewAuditTrail',
          function ( jobChain )
          {
            me._viewJobChainAuditTrail( jobChain );
          } );
    }
  },


  /**
   * Binds the trigger audit trail panel events to their handlers.
   */
  _bindEventHandlersAuditTrailView : function ( triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var viewPanel = triggersTab.down( '#auditTrailView' );
    viewPanel.on( 'activate', function ()
    {
      me._refreshAuditTrailView( me.connectionId, true );
      me._refreshHistoryView( me.connectionId, false );
      me._refreshMisfiredTriggersView( me.connectionId, false );
      me._refreshStatisticsView( me.connectionId, false );
      me._refreshExecNotificationsView( me.connectionId, false );
      me._refreshJobChainsView( me.connectionId, false );
    } );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.General;
    if ( licensedFeatures.AuditTrail )
    {
      var auditTrailGrid = triggersTab.down( '#auditTrailGrid' );

      // refresh button
      auditTrailGrid.on( 'refresh',
          function ()
          {
            me._refreshAuditTrailView( me.connectionId, true );
          } );

      // filters changed
      auditTrailGrid.on( 'changeFilters',
          function ()
          {
            me._refreshAuditTrailView( me.connectionId, true );
          } );

      // TODO: check audit trail index is enabled
      var auditTrailIndexEnabled = false;

      auditTrailGrid.on( 'loadFailed',
          function ( operation )
          {
            auditTrailGrid.down( '#refreshButton' ).enable();
          } );

      auditTrailGrid.on( 'loadSuccess',
          function ( store, records )
          {
            var triggersGrid = triggersTab.down( '#triggersGrid' );
            var triggerSelected = triggersGrid.getSelectionModel().getSelection().length > 0;

            if ( triggerSelected )
            {
              auditTrailGrid.down( '#refreshButton' ).enable();
              auditTrailGrid.down( '#fetchPeriod' ).enable();
              auditTrailGrid.down( '#maxFetchSizeSelect' ).enable();
              auditTrailGrid.down( '#fetchRelatedCheckbox' ).enable();

              // enable only if the index is enabled
              if ( auditTrailIndexEnabled )
              {
                auditTrailGrid.down( '#filterQueryText' ).enable();
              }
            }

            // update empty text if there are no records
            var view = auditTrailGrid.getView();
            if ( Ext.isEmpty( records ) )
            {
              if ( triggerSelected )
              {
                view.emptyText =
                    QDesk.Msg.get(
                        'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.auditTrailEmptyTextNoMatchingDataForTrigger' );
              }
              else
              {
                view.emptyText =
                    QDesk.Msg.get(
                        'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab.auditTrailEmptyTextNoTriggerSelected' );
              }

              view.refresh();
            }
          } );
    }
  },


  /**
   * Refreshes the triggers grid.
   * @param connectionId a connection ID.
   * @private
   */
  _refreshTriggersGrid : function ( connectionId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid', {
      connectionId : connectionId
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger execution history view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshHistoryView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecHistoryView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Opens a window for exporting trigger execution history data to a CSV file.
   * @param connectionId a connection ID.
   * @param trigger a trigger whose execution history data should be exported.
   * @private
   */
  _exportExecHistory : function ( connectionId, trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ExportExecHistory', {
      connectionId : connectionId,
      trigger : trigger
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger misfired triggers view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshMisfiredTriggersView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshMisfiredTriggersView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger statistics view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshStatisticsView : function ( connectionId, populate )
  {
    var me = this;

    var triggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getTriggersTab( connectionId );
    var statisticsView = triggersTab.down( '#statisticsView' );

    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView', {
      connectionId : connectionId,
      populate : populate,
      maskEl : statisticsView.getEl()
    } ).execute();
  },


  /**
   * Invoked to view the chart in a new modal window which allows the user to save the chart
   * through the browser context menu.
   * @param connectionId a connection ID.
   */
  _previewStatisticsChart : function ( connectionId )
  {
    var me = this;

    var triggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getTriggersTab( connectionId );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = triggersTab.down( '#statisticsView #statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.PreviewChart', {
      chart : chart
    } ).execute();
  },


  /**
   * Invoked to save the chart in the trigger statistics view.
   * @param connectionId a connection ID.
   * @param mimeType the chart image mime type.
   */
  _saveStatisticsChart : function ( connectionId, mimeType )
  {
    var triggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getTriggersTab( connectionId );

    /* charts are either cartesian or polar so we use a common superclass - draw */
    var chart = triggersTab.down( '#statisticsView #statisticsPanel #chartContainer draw' );

    Ext.create( 'QDesk.controller.command.SaveChart', {
      chart : chart,
      mimeType : mimeType
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger notifications view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshExecNotificationsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecNotificationsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger job chains view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshJobChainsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshJobChainsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger audit trail view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshAuditTrailView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshAuditTrailView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked when a trigger in the TriggersGrid in the TriggerTab is selected.
   * @param triggerWithJobDetail a QDesk.model.scheduler.quartz.QuartzTriggerWithJobDetail instance representing the selected trigger.
   * @param triggersTab a TriggersTab.
   */
  _onTriggerSelected : function ( triggerWithJobDetail, triggersTab )
  {
    var me = this;

    var jvmAgentReady = QDesk.controller.command.connection.ConnectionUtils.isJvmAgentReady( me.connectionId );
    var publicApiReady = QDesk.controller.command.connection.ConnectionUtils.isPublicApiReady( me.connectionId );

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    var triggerState = triggerWithJobDetail.get( 'trigger' ).state;
    switch ( triggerState )
    {
      case QDesk.model.scheduler.quartz.QuartzTriggerState.NONE:
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.NORMAL:
        triggersGrid.down( '#resumeTriggerButton' ).disable();
        triggersGrid.down( '#pauseTriggerButton' ).enable();
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.PAUSED:
        triggersGrid.down( '#resumeTriggerButton' ).enable();
        triggersGrid.down( '#pauseTriggerButton' ).disable();
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.COMPLETE:
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.ERROR:
        break;

      case QDesk.model.scheduler.quartz.QuartzTriggerState.BLOCKED:
        break;

      default:
        Ext.Error.raise( 'Unexpected trigger state value: ' + triggerState );
    }

    triggersGrid.down( '#deleteTriggerButton' ).enable();

    // enable only if the JVM Agent and public API are ready
    if ( jvmAgentReady && publicApiReady )
    {
      triggersGrid.down( '#editTriggerButton' ).enable();
      triggersGrid.down( '#cloneTriggerButton' ).enable();
    }

    var views = triggersTab.down( '#views' );

    var connectionId = triggersTab.getConnectionId();
    var activeView = views.getActiveTab();

    me._refreshHistoryView( connectionId, activeView.itemId == 'historyView' );
    me._refreshMisfiredTriggersView( connectionId, activeView.itemId == 'misfiredTriggersView' );
    me._refreshStatisticsView( connectionId, activeView.itemId == 'statisticsView' );
    me._refreshExecNotificationsView( connectionId, activeView.itemId == 'notificationsView' );
    me._refreshJobChainsView( connectionId, activeView.itemId == 'jobChainsView' );
    me._refreshAuditTrailView( connectionId, activeView.itemId == 'auditTrailView' );
  },


  /**
   * Invoked when a trigger in the TriggersGrid in the TriggersTab is unselected.
   * @param triggersTab a TriggersTab.
   */
  _onTriggerUnselected : function ( triggersTab )
  {
    var me = this;

    var triggersGrid = triggersTab.down( '#triggersGrid' );

    triggersGrid.down( '#resumeTriggerButton' ).disable();
    triggersGrid.down( '#pauseTriggerButton' ).disable();
    triggersGrid.down( '#deleteTriggerButton' ).disable();
    triggersGrid.down( '#editTriggerButton' ).disable();
    triggersGrid.down( '#cloneTriggerButton' ).disable();

    var connectionId = triggersTab.getConnectionId();

    me._refreshHistoryView( connectionId, false );
    me._refreshMisfiredTriggersView( connectionId, false );
    me._refreshStatisticsView( connectionId, false );
    me._refreshExecNotificationsView( connectionId, false );
    me._refreshJobChainsView( connectionId, false );
    me._refreshAuditTrailView( connectionId, false );
  },


  /**
   * Invoked when an execution notification rule record in the TriggerNotificationRulesGrid in the TriggerNotificationsView
   * is selected.
   * @param execNotifRule a QDesk.model.scheduler.quartz.QuartzExecNotificationRule instance representing the selected
   * notification rule record.
   * @param triggersTab the TriggersTab.
   */
  _onNotificationRuleSelected : function ( execNotifRule, triggersTab )
  {
    var me = this;

    var execNotificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );

    execNotificationRulesGrid.down( '#deleteButton' ).enable();
    execNotificationRulesGrid.down( '#editButton' ).enable();
    execNotificationRulesGrid.down( '#cloneButton' ).enable();
    execNotificationRulesGrid.down( '#viewAuditTrailButton' ).enable();
  },


  /**
   * Invoked when a notification rule record in the TriggerNotificationRulesGrid in the TriggerNotificationsView
   * is unselected.
   * @param triggersTab the TriggersTab.
   */
  _onNotificationRuleUnselected : function ( triggersTab )
  {
    var me = this;

    var execNotificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );

    execNotificationRulesGrid.down( '#deleteButton' ).disable();
    execNotificationRulesGrid.down( '#editButton' ).disable();
    execNotificationRulesGrid.down( '#cloneButton' ).disable();
    execNotificationRulesGrid.down( '#viewAuditTrailButton' ).disable();
  },


  /**
   * Invoked when a job chain record in the JobChainsGrid in the TriggerJobChainsView is selected.
   * @param jobChain a QDesk.model.scheduler.quartz.QuartzJobChain instance representing the selected
   * job chain.
   * @param triggersTab the TriggersTab.
   */
  _onJobChainSelected : function ( jobChain, triggersTab )
  {
    var me = this;

    var jobChainsGrid = triggersTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteButton' ).enable();
    jobChainsGrid.down( '#editButton' ).enable();
    jobChainsGrid.down( '#cloneButton' ).enable();
  },


  /**
   * Invoked when a job chain record in the JobChainsGrid in the TriggerJobChainsView is unselected.
   * @param triggersTab the TriggersTab.
   */
  _onJobChainUnselected : function ( triggersTab )
  {
    var me = this;

    var jobChainsGrid = triggersTab.down( '#jobChainsGrid' );

    jobChainsGrid.down( '#deleteButton' ).disable();
    jobChainsGrid.down( '#editButton' ).disable();
    jobChainsGrid.down( '#cloneButton' ).disable();
  },


  _resumeTriggerGroup : function ( triggerGroupName )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseTriggerGroup : function ( triggerGroupName )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTriggerGroup', {
      connectionId : me.connectionId,
      triggerGroupName : triggerGroupName,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _resumeTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.ResumeTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _pauseTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _addTrigger : function ( triggerType )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.SelectJob', {
      connectionId : me.connectionId,
      /*
       * Upon job selection, open the Add trigger window.
       */
      success : function ( jobDetail )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger', {
          connectionId : me.connectionId,
          triggerType : triggerType,
          jobGroupName : jobDetail.get( 'group' ).name,
          jobName : jobDetail.get( 'name' ),

          /*
           * Upon success refresh the triggers tab.
           */
          success : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
              connectionId : me.connectionId
            } ).execute();
          }
        } ).execute();
      }
    } ).execute();
  },


  _deleteTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.DeleteTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _editTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.EditTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _cloneTrigger : function ( trigger )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.CloneTrigger', {
      connectionId : me.connectionId,
      trigger : trigger,
      /*
       * Upon success refresh the triggers tab.
       */
      success : function ()
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
          connectionId : me.connectionId
        } ).execute();
      }
    } ).execute();
  },


  _enableExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EnableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _disableExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DisableExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _deleteNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.DeleteExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRule : execNotifRule,
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _editExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotifRule.get( 'id' ),
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _cloneExecNotificationRule : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
      connectionId : me.connectionId,
      notificationRuleId : execNotifRule.get( 'id' ),
      /*
       * Upon success refresh the notifications view.
       */
      success : function ()
      {
        me._refreshExecNotificationsView( me.connectionId, true );
      }
    } ).execute();
  },


  _viewExecNotificationRuleAuditTrail : function ( execNotifRule )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail', {
      connectionId : me.connectionId,
      notificationRuleId : execNotifRule.get( 'id' )
    } ).execute();
  },


  _enableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EnableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _disableJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DisableJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _deleteJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.DeleteJobChain', {
      connectionId : me.connectionId,
      jobChain : jobChain,
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _editJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.EditJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _cloneJobChain : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.CloneJobChain', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' ),
      /*
       * Upon success refresh the job chains view.
       */
      success : function ()
      {
        me._refreshJobChainsView( me.connectionId, true );
      }
    } ).execute();
  },


  _viewJobChainAuditTrail : function ( jobChain )
  {
    var me = this;

    Ext.create( 'QDesk.controller.command.scheduler.quartz.chain.ViewJobChainAuditTrail', {
      connectionId : me.connectionId,
      jobChainId : jobChain.get( 'id' )
    } ).execute();
  },


  /**
   * Opens a window showing the specified job data maps.
   * @param jobDataMap the job data map.
   * @param mergedJobDataMap the merged job data map.
   */
  _viewJobDataMap : function ( jobDataMap, mergedJobDataMap )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.ViewJobDataMap', {
      // exec history record can have null job data map !!!
      jobDataMap : jobDataMap == null ? [] : jobDataMap,
      mergedJobDataMap : mergedJobDataMap
    } ).execute();
  },


  /**
   * Opens a window showing the next fire times for the specified trigger.
   * @param connectionId a connection ID.
   * @param triggerGroupName a trigger group name.
   * @param triggerName a trigger name.
   */
  _viewNextFireTimesOfTrigger : function ( connectionId, triggerGroupName, triggerName )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.ViewNextFireTimes', {
      connectionId : connectionId,
      triggerGroupName : triggerGroupName,
      triggerName : triggerName
    } ).execute();
  },


  /**
   * Opens a window showing the job execution exception.
   * @param connectionId a connection ID.
   * @param exception the exception data.
   */
  _viewException : function ( connectionId, exception )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.ViewError', {
      errorText : exception
    } ).execute();
  },


  /**
   * Opens a window showing the job execution log.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewLog : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryLog', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution result.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewResult : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryResult', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  },


  /**
   * Opens a window showing the job execution user data.
   * @param connectionId a connection ID.
   * @param execHistoryId an execution history ID.
   */
  _viewUserData : function ( connectionId, execHistoryId )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.history.ViewExecHistoryUserData', {
      connectionId : connectionId,
      execHistoryId : execHistoryId
    } ).execute();
  }
} );
