/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Pauses the specified trigger group.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTriggerGroup', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /*
   * Required config property containing the connection ID.
   */
  connectionId : undefined,

  /**
   * Required config property containing the trigger group name.
   */
  triggerGroupName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.triggerGroupName )
      Ext.Error.raise( 'Required config parameter missing: triggerGroupName' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'quartz.confirm.triggerGroupPause' ) )
    {
      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTriggerGroup.confirmMsg',
              Ext.String.htmlEncode( me.triggerGroupName ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._pauseTriggerGroup();
            }
          }
      );
    }
    else
    {
      me._pauseTriggerGroup();
    }
  },


  _pauseTriggerGroup : function ()
  {
    var me = this;

    QDesk.util.UxUtils.maskBody(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTriggerGroup.maskMsg',
            Ext.String.htmlEncode( me.triggerGroupName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.triggerGroupActionPause,
      params : {
        connectionId : me.connectionId,
        triggerGroupName : me.triggerGroupName
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskBody();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTriggerGroup.successMsg',
                    Ext.String.htmlEncode( me.triggerGroupName ) ),
                function ()
                {
                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }

                  me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.TRIGGER_GROUP_PAUSED, {
                    connectionId : me.connectionId,
                    triggerGroupName : me.triggerGroupName
                  } );
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.PauseTriggerGroup.errorMsg',
                  Ext.String.htmlEncode( me.triggerGroupName ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
