/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Triggers tab in the scheduler view panel.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecHistoryView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecNotificationsView'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    // refreshes the triggers grid
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersGrid', {
      connectionId : me.connectionId,
      success : function ()
      {
        me._refreshNotificationsView( me.connectionId, true );
        me._refreshHistoryView( me.connectionId, true );
        me._refreshStatisticsView( me.connectionId, true );
      },
      failure : function ()
      {
        me._refreshNotificationsView( me.connectionId, false );
        me._refreshHistoryView( me.connectionId, false );
        me._refreshStatisticsView( me.connectionId, false );
      }
    } ).execute();


  },


  /**
   * Invoked to refresh the data in the trigger execution history view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshHistoryView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecHistoryView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger statistics view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshStatisticsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  },


  /**
   * Invoked to refresh the data in the trigger notifications view.
   * @param connectionId a connection ID.
   * @param populate a flag indicating whether the view should be populated
   * with data to reflect the currently selected trigger.
   */
  _refreshNotificationsView : function ( connectionId, populate )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshExecNotificationsView', {
      connectionId : connectionId,
      populate : populate
    } ).execute();
  }
} );
