/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Logs out the currently logged in user.
 */
Ext.define( 'QDesk.controller.command.security.Logout', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.model.common.Result',
    'QDesk.model.security.LogoutError',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.EventName'
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.Logout.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.securityLogoutAction,
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var result = json.result;

            // update the authentication status and profile of the current user
            QDesk.CurrentUser.update(
                result.sessionToken,
                {
                  authenticated : false,
                  userProfile : null
                },
                false );

            if ( result.result === QDesk.model.common.Result.SUCCESS )
            {
              me.application.fireEvent( QDesk.util.constant.EventName.LOGOUT_SUCCESS, {
                sessionToken : result.sessionToken,
                automatic : false
              } );
            }
            else
            {
              if ( result.error )
              {
                switch ( result.error )
                {
                  case QDesk.model.security.LogoutError.SESSION_NOT_FOUND:
                    break;
                }
              }

              me.application.fireEvent( QDesk.util.constant.EventName.LOGOUT_SUCCESS, {
                sessionToken : result.sessionToken,
                automatic : false
              } );

              //me.application.fireEvent( QDesk.util.constant.EventName.LOGOUT_FAILED, {
              //   sessionToken : result.sessionToken
              // } );
            }
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.controller.command.security.Logout.errorMsg' ) );
        }
      }
    } );
  }
} );
