/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified group.
 */
Ext.define( 'QDesk.controller.command.security.groups.DeleteGroup', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required config parameter with the GroupStub instance representing the group that should be deleted.
   */
  groupStub : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.groupStub )
      Ext.Error.raise( 'Required config parameter missing: groupStub' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.groupDelete' ) )
    {
      var name = me.groupStub.get( 'name' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.security.groups.DeleteGroup.confirmMsg',
              Ext.String.htmlEncode( name ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteGroup();
            }
          }
      );
    }
    else
    {
      me._deleteGroup();
    }
  },


  /**
   * Deletes the group.
   */
  _deleteGroup : function ()
  {
    var me = this;

    var groupId = me.groupStub.get( 'id' );
    var name = me.groupStub.get( 'name' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.groups.DeleteGroup.maskMsg',
        Ext.String.htmlEncode( name ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.groupActionDelete,
      params : {
        groupId : groupId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.security.groups.DeleteGroup.successMsg',
                    Ext.String.htmlEncode( name ) ),
                function ()
                {
                  me.application.fireEvent( QDesk.util.constant.EventName.GROUP_DELETED, {
                    groupId : groupId
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.security.groups.DeleteGroup.errorMsg',
                  Ext.String.htmlEncode( name ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
