/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Deletes the specified user.
 */
Ext.define( 'QDesk.controller.command.security.users.DeleteUser', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required config parameter with the UserStub instance representing the user whose account should be deleted.
   */
  userStub : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.userStub )
      Ext.Error.raise( 'Required config parameter missing: userStub' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.userDelete' ) )
    {
      var username = me.userStub.get( 'username' );

      QDesk.util.UxUtils.showConfirmDialog(
          QDesk.Msg.get( 'QDesk.controller.command.security.users.DeleteUser.confirmMsg',
              Ext.String.htmlEncode( username ) ),
          function ( btn )
          {
            if ( btn == 'yes' )
            {
              me._deleteUser();
            }
          }
      );
    }
    else
    {
      me._deleteUser();
    }
  },


  /**
   * Deletes the user.
   */
  _deleteUser : function ()
  {
    var me = this;

    var userId = me.userStub.get( 'id' );
    var username = me.userStub.get( 'username' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.users.DeleteUser.maskMsg',
        Ext.String.htmlEncode( username ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.userActionDelete,
      params : {
        userId : userId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            QDesk.util.UxUtils.showInfo(
                QDesk.Msg.get( 'QDesk.controller.command.security.users.DeleteUser.successMsg',
                    Ext.String.htmlEncode( username ) ),
                function ()
                {
                  me.application.fireEvent( QDesk.util.constant.EventName.USER_DELETED, {
                    userId : userId
                  } );

                  // invoke the success callback handler
                  if ( me.success && Ext.isFunction( me.success ) )
                  {
                    me.success.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.security.users.DeleteUser.errorMsg',
                  Ext.String.htmlEncode( username ) ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
