/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Saves the specified user.
 */
Ext.define( 'QDesk.controller.command.security.users.SaveUser', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.User',

    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],

  /**
   * Required config parameter with the EditUserForm panel.
   */
  formPanel : undefined,

  /**
   * Required config parameter with the User instance representing the user to insert/update.
   */
  user : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.formPanel )
      Ext.Error.raise( 'Required config parameter missing: formPanel' );

    if ( !config.user )
      Ext.Error.raise( 'Required config parameter missing: user' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var isInsert = me.user.get( 'id' ) == null;

    if ( isInsert )
    {
      me._insertUser( me.user );
    }
    else
    {
      me._updateUser( me.user );
    }
  },


  _insertUser : function ( user )
  {
    var me = this;

    var username = user.get( 'username' );

    var form = me.formPanel.getForm();

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.userActionAdd,
      params : {
        user : QDesk.util.CommonUtils.encode( user )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.security.users.SaveUser.addWaitMsg',
          Ext.String.htmlEncode( username ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.security.users.SaveUser.addSuccessMsg',
                  Ext.String.htmlEncode( username ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.USER_ADDED, {
                  user : me.user
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  },


  _updateUser : function ( user )
  {
    var me = this;

    var username = user.get( 'username' );

    var form = me.formPanel.getForm();

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.userActionUpdate,
      params : {
        user : QDesk.util.CommonUtils.encode( user )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.security.users.SaveUser.updateWaitMsg',
          Ext.String.htmlEncode( username ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.security.users.SaveUser.updateSuccessMsg',
                  Ext.String.htmlEncode( username ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.USER_UPDATED, {
                  user : me.user
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showJsonResponseErrors( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  }
} );
