/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window showing effective permissions of the specified user.
 */
Ext.define( 'QDesk.controller.command.security.users.ViewEffectiveUserPermissions', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.work.security.users.EffectiveUserPermissionsWindow'
  ],

  /**
   * Required config parameter with the ID of the user whose effective user permissions should be displayed.
   */
  userId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.userId )
      Ext.Error.raise( 'Required config parameter missing: userId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EffectiveUserPermissionsWindow' );

    //me._bindEventHandlers( window );

    var store = window.down( '#effectiveUserPermissionsGrid' ).getStore();
    store.getProxy().extraParams = {
      userId : me.userId
    };

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.security.users.ViewEffectiveUserPermissions.maskLoadMsg' ) );

    // initial load of data
    store.loadPage( 1, {
      callback : function ( records, operation, successful )
      {
        me.unmask();

        if ( successful )
        {
          window.show();

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          var response = operation.getResponse();
          if ( response && response.responseText )
          {
            // application errors
            QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ),
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
          else
          {
            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.security.users.ViewEffectiveUserPermissions.errorMsg' ),
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
      }
    } );
  }
} );
