/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits the profile of the currently logged in user.
 */
Ext.define( 'QDesk.controller.command.user.EditProfile', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.user.SaveProfile',
    'QDesk.controller.command.security.users.ViewEffectiveUserPermissions',

    'QDesk.model.security.User',

    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',

    'QDesk.util.security.UserUtils',

    'QDesk.view.work.user.EditProfileWindow'
  ],

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var userProfile = QDesk.CurrentUser.getUserProfile();

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.user.EditProfile.maskLoadMsg' ) );

    // load the User record
    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.userActionGet,
      params : {
        userId : userProfile.userId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var user = Ext.create( 'QDesk.model.security.User', json.data );
            me._editProfile( user );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json,
                function ()
                {
                  // invoke the failure callback handler
                  if ( me.failure && Ext.isFunction( me.failure ) )
                  {
                    me.failure.apply( me );
                  }
                } );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.user.EditProfile.errorLoadMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  },


  _editProfile : function ( user )
  {
    var me = this;

    var window = Ext.widget( 'EditProfileWindow' );

    var formPanel = window.down( '#form' );
    formPanel.loadRecord( user );

    formPanel.on( 'viewEffectiveUserPermissions',
        function ( user )
        {
          Ext.create( 'QDesk.controller.command.security.users.ViewEffectiveUserPermissions', {
            userId : user.get( 'id' ),
            maskEl : window.getEl()
          } ).execute();
        } );

    // Save/Update button
    window.on( 'saveProfile', function ( user )
    {
      Ext.create( 'QDesk.controller.command.user.SaveProfile', {
        formPanel : formPanel,
        user : user,
        success : function ()
        {
          window.close();
        }
      } ).execute();
    } );

    window.show();
  }
} );
