/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.navigation.SettingsController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.CurrentUser',
    'QDesk.Msg',

    'QDesk.controller.command.audit.ViewAuditTrail',
    'QDesk.controller.command.config.EditApplicationConfig',
    'QDesk.controller.command.message.channel.EditMessageChannelProfiles',
    'QDesk.controller.command.security.groups.EditGroups',
    'QDesk.controller.command.security.users.EditUsers',

    'QDesk.util.UxUtils',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.constant.EventName'
  ],


  init : function ()
  {
    var me = this;

    // key is a component query, value is a listener configuration
    me.control( {
      // tree node click
      '#settingsTreePanel' : {
        itemdblclick : function ( view, record )
        {
          var nodeId = record.get( 'id' );

          if ( QDesk.CurrentUser.isAuthenticated() )
          {
            var qdLicensed = record.get( 'qdLicensed' );
            var qdRequiresAnyPermissions = record.get( 'qdRequiresAnyPermissions' );

            if ( qdLicensed )
            {
              var authorized = qdRequiresAnyPermissions == null ||
                  QDesk.CurrentUser.hasAnyPermissions( qdRequiresAnyPermissions );

              if ( authorized )
              {
                switch ( nodeId )
                {
                  case 'users':
                    me._onUsers();
                    break;

                  case 'groups':
                    me._onGroups();
                    break;

                  case 'auditTrail':
                    me._onAuditTrail();
                    break;

                  case 'appSettings':
                    me._onAppSettings();
                    break;

                  case 'messageChannelProfiles':
                    me._onMessageChannelProfiles();
                    break;

                  default:
                    Ext.Error.raise( 'Unrecognized node ID: ' + nodeId );
                }
              }
            }
            else
            {
              // not licensed
              QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName ) );
            }
          }
          else
          {
            // not authenticated
            QDesk.util.UxUtils.showErrorDialog( QDesk.Msg.get( 'QDesk.userNotAuthenticatedMsg' ) );
          }
        }
      }
    } );

    var application = me.getApplication();
    application.on( QDesk.util.constant.EventName.WORK_PANEL_ACTIVATED, me._onWorkPanelActivated, me );
    // application.on( QDesk.util.constant.EventName.WORK_PANEL_DEACTIVATED, me._onWorkPanelDeactivated, me );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;
    me.callParent( arguments );
  },


  _onAuditTrail : function ()
  {
    Ext.create( 'QDesk.controller.command.audit.ViewAuditTrail', {} ).execute();
  },


  _onUsers : function ()
  {
    Ext.create( 'QDesk.controller.command.security.users.EditUsers', {} ).execute();
  },


  _onGroups : function ()
  {
    Ext.create( 'QDesk.controller.command.security.groups.EditGroups', {} ).execute();
  },


  _onAppSettings : function ()
  {
    Ext.create( 'QDesk.controller.command.config.EditApplicationConfig', {} ).execute();
  },


  _onMessageChannelProfiles : function ()
  {
    Ext.create( 'QDesk.controller.command.message.channel.EditMessageChannelProfiles', {} ).execute();
  },


  _onWorkPanelActivated : function ()
  {
    var me = this;

    var navigationPanel = Ext.ComponentQuery.query( '#navigationPanel' )[ 0 ];

    var settingsTreePanel = navigationPanel.down( '#settingsTreePanel' );
    if ( settingsTreePanel )
    {
      me._loadSettingsTreePanelStore( settingsTreePanel );
    }
    else
    {
      // wait for the NavigationPanel's loaded event that indicates that all of the subpanels have been loaded
      navigationPanel.on( 'loaded', function ()
      {
        var settingsTreePanel = navigationPanel.down( '#settingsTreePanel' );
        me._loadSettingsTreePanelStore( settingsTreePanel );
      } );
    }
  },


  _loadSettingsTreePanelStore : function ( settingsTreePanel )
  {
    // deselect all nodes
    settingsTreePanel.getSelectionModel().deselectAll();

    var settingsTreeStore = settingsTreePanel.getStore();

    var licensedFeaturesGeneral = QDesk.util.license.LicensedProductFeatures.featureSets.General;
    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;

    var children = [];

    children.push( {
      id : 'auditTrail',
      iconCls : 'auditTrail',
      leaf : true,

      /* SettingsTreeNode model attributes */
      name : QDesk.Msg.get( 'QDesk.view.work.navigation.settings.SettingsTreePanel.nodeAuditTrail' ),
      qdLicensed : licensedFeaturesGeneral.AuditTrail,
      qdRequiresAnyPermissions : [
        QDesk.model.security.Permission.AUDIT_TRAIL_VIEW
      ],
      qdNotAuthorizedIconCls : 'auditTrailNAUTH'
    } );

    children.push( {
      id : 'users',
      iconCls : 'users',
      leaf : true,

      /* SettingsTreeNode model attributes */
      name : QDesk.Msg.get( 'QDesk.view.work.navigation.settings.SettingsTreePanel.nodeUsers' ),
      qdLicensed : true,
      qdRequiresAnyPermissions : [
        QDesk.model.security.Permission.USER_ADD,
        QDesk.model.security.Permission.USER_UPDATE,
        QDesk.model.security.Permission.USER_DELETE,
        QDesk.model.security.Permission.USER_LOCK_UPDATE,
        QDesk.model.security.Permission.USER_PERMISSIONS_UPDATE
      ],
      qdNotAuthorizedIconCls : 'usersNAUTH'
    } );

    children.push( {
      id : 'groups',
      iconCls : 'groups',
      leaf : true,

      /* SettingsTreeNode model attributes */
      name : QDesk.Msg.get( 'QDesk.view.work.navigation.settings.SettingsTreePanel.nodeGroups' ),
      qdLicensed : true,
      qdRequiresAnyPermissions : [
        QDesk.model.security.Permission.GROUP_ADD,
        QDesk.model.security.Permission.GROUP_UPDATE,
        QDesk.model.security.Permission.GROUP_DELETE,
        QDesk.model.security.Permission.GROUP_PERMISSIONS_UPDATE,
        QDesk.model.security.Permission.GROUP_MEMBERS_UPDATE
      ],
      qdNotAuthorizedIconCls : 'groupsNAUTH'
    } );

    // Application Settings are always accessible, no permissions are required for read-only access
    children.push( {
      id : 'appSettings',
      iconCls : 'appSettings',
      leaf : true,

      /* SettingsTreeNode model attributes */
      name : QDesk.Msg.get( 'QDesk.view.work.navigation.settings.SettingsTreePanel.nodeAppSettings' ),
      qdLicensed : true
    } );

    children.push( {
      id : 'messageChannelProfiles',
      iconCls : 'messageChannelProfiles',
      leaf : true,

      /* SettingsTreeNode model attributes */
      name : QDesk.Msg.get( 'QDesk.view.work.navigation.settings.SettingsTreePanel.nodeMessageChannelProfiles' ),
      qdLicensed : licensedProductEdition === QDesk.model.license.ProductEdition.ENTERPRISE
    } );


    /*
     * Update children node cls and iconCls based on qdLicensed, qdRequiresAnyPermissions node properties.
     */
    Ext.each( children, function ( child )
    {
      var qdLicensed = child.qdLicensed;
      var qdRequiresAnyPermissions = child.qdRequiresAnyPermissions;

      if ( qdLicensed )
      {
        if ( qdRequiresAnyPermissions )
        {
          if ( QDesk.CurrentUser.hasAnyPermissions( qdRequiresAnyPermissions ) )
          {
            // current user has the required permissions
          }
          else
          {
            child.cls = 'notAuthorized';

            var qdNotAuthorizedIconCls = child.qdNotAuthorizedIconCls;
            if ( qdNotAuthorizedIconCls )
            {
              child.iconCls = qdNotAuthorizedIconCls;
            }
          }
        }
        else
        {
          // no permissions required
        }
      }
      else
      {
        child.cls = 'notLicensed';
      }
    } );

    settingsTreeStore.loadData( children );
  }
} );
