/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.security.SecurityController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],


  init : function ()
  {
    var me = this;

    var application = me.getApplication();

    application.addListener( QDesk.util.constant.EventName.LOGIN_SUCCESS, me._onLoginSuccess, me );
    application.addListener( QDesk.util.constant.EventName.LOGIN_FAILED, me._onLoginFailed, me );

    application.addListener( QDesk.util.constant.EventName.LOGOUT_SUCCESS, me._onLogoutSuccess, me );
    //application.addListener( QDesk.util.constant.EventName.LOGOUT_FAILED, me._onLogoutFailed, me );

    application.addListener( QDesk.util.constant.EventName.HEARTBEAT, me._onHeartbeat, me );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;
    me.callParent( arguments );
  },


  _onLoginSuccess : function ( params )
  {
    var me = this;

    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];
    viewport.activateWorkPanel();

    var userProfile = params.userProfile;

    QDesk.util.UxUtils.showInfoBubble(
        QDesk.Msg.get( 'QDesk.controller.security.SecurityController.loginSuccessMsg',
            Ext.String.htmlEncode( userProfile.username ) ) );
  },


  _onLoginFailed : function ( params )
  {
    //var me = this;
  },


  _onLogoutSuccess : function ( params )
  {
    var me = this;

    me._cleanupWorkPanel();

    /*
     * Activate the security login panel.
     */
    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];
    viewport.activateSecurityLoginPanel();

    var automatic = params.automatic;
    if ( automatic )
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get( 'QDesk.controller.security.SecurityController.userSessionExpired' ) );
    }
    else
    {
      // manual logout
      QDesk.util.UxUtils.showInfoBubble(
          QDesk.Msg.get( 'QDesk.controller.security.SecurityController.logoutSuccessMsg' ) );
    }
  },


  _onLogoutFailed : function ( params )
  {
    //var me = this;
  },


  _onHeartbeat : function ( params )
  {
    var me = this;
    var viewport = Ext.ComponentQuery.query( 'viewport' )[ 0 ];

    // update license warnings (yellow box) in the security panel
    var securityPanel = viewport.down( '#securityPanel' );
    securityPanel.updateLicenseWarnings( params.license );
  },


  _cleanupWorkPanel : function ()
  {
    var me = this;

    /*
     * Close all opened windows (floating components).
     */
    Ext.WindowManager.each(
        function ( comp )
        {
          if ( comp.close )
            comp.close();
        } );

    /*
     * Close all closable tabs in the MainTabPanel (direct children of MainTabPanel). The WelcomePanel is not closable
     * and as such is not closed.
     */
    Ext.each( Ext.ComponentQuery.query( '#mainTabPanel > panel' ),
        function ( panel )
        {
          if ( panel.closable )
            panel.close();
        } );
  }
} );
