/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * The user data model based on the com.quartzdesk.domain.model.security.User class.
 */
Ext.define( 'QDesk.model.security.User', {
  extend : 'Ext.data.Model',

  requires : [
    'QDesk.util.constant.DateFormat'
  ],

  fields : [
    { name : 'id', type : 'int', allowNull : true },

    { name : 'externalId', type : 'string', allowNull : true },

    { name : 'createdAt', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601 },
    { name : 'updatedAt', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601 },

    { name : 'username', type : 'string' },

    { name : 'system', type : 'boolean' },

    /*
     * See QDesk.model.security.UserAccountStatus.
     */
    { name : 'accountStatus', type : 'string' },

    /*
     * See QDesk.model.security.UserAccountLockReason.
     */
    { name : 'accountLockReason', type : 'string', allowNull : true },
    {
      name : 'accountLockedAt',
      type : 'date',
      dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601,
      allowNull : true
    },
    {
      name : 'accountLockedTill',
      type : 'date',
      dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601,
      allowNull : true
    },

    { name : 'password', type : 'string' },
    /*
     * See QDesk.enums.security.UserPasswordHashAlgorithm.
     */
    { name : 'passwordHashAlgorithm', type : 'string', allowNull : true },

    /*
     * See QDesk.enums.security.UserPasswordStatus.
     */
    { name : 'passwordStatus', type : 'string' },

    {
      name : 'lastSuccessfulLoginAt',
      type : 'date',
      dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601,
      allowNull : true
    },
    {
      name : 'lastFailedLoginAt',
      type : 'date',
      dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601,
      allowNull : true
    },

    { name : 'firstName', type : 'string' },
    { name : 'surname', type : 'string' },
    { name : 'email', type : 'string' },

    /*
     * See QDesk.model.common.PhoneNumber.
     */
    { name : 'workPhoneNumber', type : 'auto', allowNull : true },

    /*
     * See QDesk.model.common.PhoneNumber.
     */
    { name : 'mobilePhoneNumber', type : 'auto', allowNull : true },

    /*
     * See QDesk.model.security.UserAvatarType.
     */
    { name : 'avatarType', type : 'string' },
    { name : 'avatarUrl', type : 'string', allowNull : true },

    /*
     * The list of IDs of permissions associated with the user.
     */
    { name : 'permissionIds', type : 'auto', allowNull : true },

    /*
     * The list of IDs of groups associated with the user.
     */
    { name : 'groupIds', type : 'auto', allowNull : true }
  ]
} );