/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various audit trails-related utility methods.
 */
Ext.define( 'QDesk.util.AuditTrailUtils', {
  singleton : true,

  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.model.audit.AuditTrailEntryCategory',
    'QDesk.model.audit.AuditTrailEntryType'
  ],


  /**
   * Returns true if details should be shown for the audit trail object with the specified category and type,
   * false otherwise.
   * @param category a category.
   * @param type a type.
   * @return true if details should be shown, false otherwise.
   */
  showDetails : function ( category, type )
  {
    switch ( category )
    {
      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_AUTHENTICATION:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.LOGIN:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.LOGOUT:
            return true;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_USER:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.USER_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.USER_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.USER_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.USER_LOCK:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.USER_UNLOCK:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.USER_CHANGE_PERMISSIONS:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.USER_CHANGE_GROUPS:
            return true;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_GROUP:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.GROUP_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.GROUP_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.GROUP_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.GROUP_CHANGE_PERMISSIONS:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.GROUP_CHANGE_MEMBERS:
            return true;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_CONNECTION:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_MOVE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_MOVE:
            return true;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_MSG_CHANNEL_PROFILE:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_MOVE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_MOVE:
            return true;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_SCHEDULER:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_START:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_PAUSE:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_STOP:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_PAUSE_TRIGGERS:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_RESUME_TRIGGERS:
            return false;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_JOB_GROUP:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_GROUP_PAUSE:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_GROUP_RESUME:
            return false;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_JOB:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_MANUAL_TRIGGER:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_PAUSE:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_RESUME:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_INTERRUPT:
            return false;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_TRIGGER_GROUP:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_GROUP_PAUSE:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_GROUP_RESUME:
            return false;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_TRIGGER:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_DELETE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_PAUSE:
            return false;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_RESUME:
            return false;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_EXEC_NOTIF_RULE:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_EXEC_NOTIF_RULE_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_EXEC_NOTIF_RULE_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_EXEC_NOTIF_RULE_DELETE:
            return true;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_JOB_CHAIN:
        switch ( type )
        {
          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_CHAIN_ADD:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_CHAIN_UPDATE:
            return true;

          case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_CHAIN_DELETE:
            return true;

          default:
            QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' for category: ' + category );
            return false;
        }

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry category: ' + category );
        return false;
    }
  },


  /**
   * Creates and returns an HTML table for the specified audit trail value object.
   * This method is invoked from the audit trail entry grids to obtain a formatted
   * HTML code to display audit trail entry details.
   * @param entryValues an audit trail entry value object (not an AuditTrailEntry model instance!).
   * @returns {string}
   */
  createDetailsHtml : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;

    switch ( category )
    {
      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_AUTHENTICATION:
        return me._createDetailsGeneralAuthentication( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_USER:
        return me._createDetailsGeneralUser( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_GROUP:
        return me._createDetailsGeneralGroup( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_CONNECTION:
        return me._createDetailsGeneralConnection( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.GENERAL_MSG_CHANNEL_PROFILE:
        return me._createDetailsGeneralMsgChannelProfile( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_SCHEDULER:
        return me._createDetailsQuartzScheduler( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_JOB_GROUP:
        return me._createDetailsQuartzJobGroup( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_JOB:
        return me._createDetailsQuartzJob( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_TRIGGER_GROUP:
        return me._createDetailsQuartzTriggerGroup( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_TRIGGER:
        return me._createDetailsQuartzTrigger( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_EXEC_NOTIF_RULE:
        return me._createDetailsQuartzExecNotifRule( entryValues );

      case QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_JOB_CHAIN:
        return me._createDetailsQuartzJobChain( entryValues );

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry category: ' + category );
        return '';
    }
  },


  _createDetailsGeneralAuthentication : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.LOGIN :
        return '';

      case QDesk.model.audit.AuditTrailEntryType.LOGOUT :
        return '<b>' + QDesk.Msg.get( 'QDesk.type' ) + ':</b> ' + entryValues.afterDetails;

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsGeneralUser : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.USER_ADD:
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.USER_UPDATE:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.USER_DELETE:
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.USER_LOCK:
        return '<b>' + QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.lockReason' ) + ':</b> ' +
            QDesk.util.EnumUtils.getUserAccountLockReason( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.USER_UNLOCK:
        return '<b>' + QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.lockReason' ) + ':</b> ' +
            QDesk.util.EnumUtils.getUserAccountLockReason( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.USER_CHANGE_PERMISSIONS:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.USER_CHANGE_GROUPS:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsGeneralGroup : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.GROUP_ADD:
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.GROUP_UPDATE:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.GROUP_DELETE:
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.GROUP_CHANGE_PERMISSIONS:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.GROUP_CHANGE_MEMBERS:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsGeneralConnection : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_ADD:
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_UPDATE:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_DELETE:
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_MOVE:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.originalObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.beforeDetails ) + '</span>'
          ],
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.newObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.afterDetails ) + '</span>'
          ]
        ] );

      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_ADD:
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_UPDATE:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_DELETE:
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.CONNECTION_FLD_MOVE:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.originalObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.beforeDetails ) + '</span>'
          ],
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.newObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.afterDetails ) + '</span>'
          ]
        ] );

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsGeneralMsgChannelProfile : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_ADD:
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_UPDATE:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_DELETE:
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_MOVE:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.originalObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.beforeDetails ) + '</span>'
          ],
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.newObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.afterDetails ) + '</span>'
          ]
        ] );

      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_ADD:
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_UPDATE:
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_DELETE:
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.MSG_CHANNEL_PROFILE_FLD_MOVE:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.originalObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.beforeDetails ) + '</span>'
          ],
          [
            QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.newObjectPath' ),
            '<span class="pre">' + Ext.String.htmlEncode( entryValues.afterDetails ) + '</span>'
          ]
        ] );

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsQuartzScheduler : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_START:
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_PAUSE:
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_STOP:
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_PAUSE_TRIGGERS:
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_SCHEDULER_RESUME_TRIGGERS:
        return '';

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsQuartzJobGroup : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_GROUP_PAUSE:
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_GROUP_RESUME:
        return '';

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsQuartzJob : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_MANUAL_TRIGGER :
        return me._createOneColumnJobDataMapTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_ADD :
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_UPDATE :
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_DELETE :
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_PAUSE :
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_RESUME :
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_INTERRUPT :
        return '';

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsQuartzTriggerGroup : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_GROUP_PAUSE:
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_GROUP_RESUME:
        return '';

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsQuartzTrigger : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_ADD :
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_UPDATE :
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_DELETE :
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_PAUSE :
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_TRIGGER_RESUME :
        return '';

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsQuartzExecNotifRule : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_EXEC_NOTIF_RULE_ADD :
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_EXEC_NOTIF_RULE_UPDATE :
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_EXEC_NOTIF_RULE_DELETE :
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createDetailsQuartzJobChain : function ( entryValues )
  {
    var me = this;

    var category = entryValues.category;
    var type = entryValues.type;

    switch ( type )
    {
      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_CHAIN_ADD :
        return me._createOneColumnDataTable( entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_CHAIN_UPDATE :
        return me._createTwoColumnDataTable( entryValues.beforeDetails, entryValues.afterDetails );

      case QDesk.model.audit.AuditTrailEntryType.QUARTZ_JOB_CHAIN_DELETE :
        return me._createOneColumnDataTable( entryValues.beforeDetails );

      default:
        QDesk.Log.warn( 'Unexpected audit trail entry type: ' + type + ' in category: ' + category );
        return '';
    }
  },


  _createOneColumnJobDataMapTable : function ( columnData )
  {
    var html = '<table class="rowDetailsTable">' +
        '<thead>' +
        '<tr><th>' +
        QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.rowDetailsTable.columnNameJobDataMap' ) +
        '</th></tr>' +
        '</thead>' +
        '<tbody>';

    html += '<tr>';

    html += '<td><pre>' + (columnData == null ? '' :
        Ext.String.htmlEncode( columnData.replace( /\\n/g, '\n' ) )) + '</pre></td>';

    html += '</tr>';

    html += '</tbody>' +
        '</table>';

    return html;
  },


  _createOneColumnDataTable : function ( columnData )
  {
    var html = '<table class="rowDetailsTable">' +
        '<thead>' +
        '<tr><th>' +
        QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.rowDetailsTable.columnNameData' ) +
        '</th></tr>' +
        '</thead>' +
        '<tbody>';

    html += '<tr>';

    html += '<td><pre>' + (columnData == null ? '' :
        Ext.String.htmlEncode( columnData.replace( /\\n/g, '\n' ) )) + '</pre></td>';

    html += '</tr>';

    html += '</tbody>' +
        '</table>';

    return html;
  },


  _createTwoColumnDataTable : function ( firstColumnData, secondColumnData )
  {
    var html = '<table class="rowDetailsTable">' +
        '<thead>' +
        '<tr><th>' +
        QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.rowDetailsTable.columnNameOriginalData' ) +
        '</th><th>' +
        QDesk.Msg.get( 'QDesk.util.AuditTrailUtils.rowDetailsTable.columnNameUpdatedData' ) +
        '</th></tr>' +
        '</thead>' +
        '<tbody>';

    html += '<tr>';

    html += '<td><pre>' + (firstColumnData == null ? '' :
        Ext.String.htmlEncode( firstColumnData.replace( /\\n/g, '\n' ) )) + '</pre></td>';

    html += '<td><pre>' + (secondColumnData == null ? '' :
        Ext.String.htmlEncode( secondColumnData.replace( /\\n/g, '\n' ) )) + '</pre></td>';

    html += '</tr>';

    html += '</tbody>' +
        '</table>';

    return html;
  }
} );
