/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


Ext.define( 'QDesk.util.EnumUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg'
  ],

  singleton : true,

  /**
   * Returns a localized name for the specified QDesk.model.audit.AuditTrailEntryCategory value.
   * @param auditTrailEntryCategory a QDesk.model.audit.AuditTrailEntryCategory value.
   * @return the localized JavaScript audit trail entry category name.
   */
  getAuditTrailEntryCategoryName : function ( auditTrailEntryCategory )
  {
    return QDesk.Msg.get( 'QDesk.model.audit.AuditTrailEntryCategory.' + auditTrailEntryCategory );
  },


  /**
   * Returns a localized name for the specified QDesk.model.audit.AuditTrailEntryType value.
   * @param auditTrailEntryType a QDesk.model.audit.AuditTrailEntryType value.
   * @return the localized JavaScript audit trail entry type name.
   */
  getAuditTrailEntryTypeName : function ( auditTrailEntryType )
  {
    return QDesk.Msg.get( 'QDesk.model.audit.AuditTrailEntryType.' + auditTrailEntryType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.gui.JsLoggingLevel value.
   * @param jsLoggingLevel a QDesk.model.gui.JsLoggingLevel value.
   * @return the localized JavaScript logging level name.
   */
  getJsLoggingLevelName : function ( jsLoggingLevel )
  {
    return QDesk.Msg.get( 'QDesk.model.gui.JsLoggingLevel.' + jsLoggingLevel );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.MessageCreatorType value.
   * @param messageCreatorType a QDesk.enums.MessageCreatorType value.
   * @return the localized message creator type name.
   */
  getMessageCreatorTypeName : function ( messageCreatorType )
  {
    return QDesk.Msg.get( 'QDesk.enums.MessageCreatorType.' + messageCreatorType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.MessageStatus value.
   * @param messageStatus a QDesk.enums.MessageStatus value.
   * @return the localized message status name.
   */
  getMessageStatusName : function ( messageStatus )
  {
    return QDesk.Msg.get( 'QDesk.enums.MessageStatus.' + messageStatus );
  },


  /**
   * Returns a localized name for the specified QDesk.model.scheduler.NotificationSeverity value.
   * @param notificationSeverity a QDesk.model.scheduler.NotificationSeverity value.
   * @return the localized notification severity name.
   */
  getNotificationSeverityName : function ( notificationSeverity )
  {
    return QDesk.Msg.get( 'QDesk.model.scheduler.NotificationSeverity.' + notificationSeverity );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.SchedulerType value.
   * @param schedulerType a QDesk.enums.SchedulerType value.
   * @return the localized scheduler type.
   */
  getSchedulerTypeName : function ( schedulerType )
  {
    return QDesk.Msg.get( 'QDesk.enums.SchedulerType.' + schedulerType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.common.ExpressionLanguage value.
   * @param expressionLanguage a QDesk.model.common.ExpressionLanguage value.
   * @return the localized expression language name.
   */
  getExpressionLanguageName : function ( expressionLanguage )
  {
    return QDesk.Msg.get( 'QDesk.model.common.ExpressionLanguage.' + expressionLanguage );
  },


  /**
   * Returns a localized name for the specified QDesk.model.license.LicenseType value.
   * @param licenseType a QDesk.model.license.LicenseType value.
   * @return the localized license type name.
   */
  getLicenseTypeName : function ( licenseType )
  {
    return QDesk.Msg.get( 'QDesk.model.license.LicenseType.' + licenseType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.message.channel.web_service.MessageReceiverWSVersion value.
   * @param wsVersion a QDesk.model.message.channel.web_service.MessageReceiverWSVersion value.
   * @return the localized WS version name.
   */
  getMessageReceiverWSVersionName : function ( wsVersion )
  {
    return QDesk.Msg.get( 'QDesk.model.message.channel.web_service.MessageReceiverWSVersion.' + wsVersion );
  },


  /**
   * Returns a localized name for the specified QDesk.model.jmx.JmxProtocol value.
   * @param protocol a QDesk.model.jmx.JmxProtocol value.
   * @return the localized JMX protocol name.
   */
  getProtocolName : function ( protocol )
  {
    return QDesk.Msg.get( 'QDesk.model.jmx.JmxProtocol.' + protocol );
  },


  /**
   * Returns a localized name for the specified QDesk.model.message.channel.ProxyType value.
   * @param proxyType a QDesk.model.message.channel.ProxyType value.
   * @return the localized proxy type name.
   */
  getProxyTypeName : function ( proxyType )
  {
    return QDesk.Msg.get( 'QDesk.model.message.channel.ProxyType.' + proxyType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.message.channel.snmp.SnmpV3AuthenticationProtocol value.
   * @param authenticationProtocol a QDesk.model.message.channel.snmp.SnmpV3AuthenticationProtocol value.
   * @return the localized authentication protocol name.
   */
  getSnmpV3AuthenticationProtocolName : function ( authenticationProtocol )
  {
    return QDesk.Msg.get( 'QDesk.model.message.channel.snmp.SnmpV3AuthenticationProtocol.' + authenticationProtocol );
  },


  /**
   * Returns a localized name for the specified QDesk.model.message.channel.snmp.SnmpV3PrivacyProtocol value.
   * @param privacyProtocol a QDesk.model.message.channel.snmp.SnmpV3PrivacyProtocol value.
   * @return the localized privacy protocol name.
   */
  getSnmpV3PrivacyProtocolName : function ( privacyProtocol )
  {
    return QDesk.Msg.get( 'QDesk.model.message.channel.snmp.SnmpV3PrivacyProtocol.' + privacyProtocol );
  },


  /**
   * Returns a localized name for the specified QDesk.model.message.channel.snmp.SnmpV3SecurityLevel value.
   * @param securityLevel a QDesk.model.message.channel.snmp.SnmpV3SecurityLevel value.
   * @return the localized security level name.
   */
  getSnmpV3SecurityLevelName : function ( securityLevel )
  {
    return QDesk.Msg.get( 'QDesk.model.message.channel.snmp.SnmpV3SecurityLevel.' + securityLevel );
  },


  /**
   * Returns a localized name for the specified QDesk.model.common.TimePeriodLength value.
   * @param periodLength a QDesk.model.common.TimePeriodLength value.
   * @return the localized period length name.
   */
  getTimePeriodLengthName : function ( periodLength )
  {
    return QDesk.Msg.get( 'QDesk.model.common.TimePeriodLength.' + periodLength );
  },


  /**
   * Returns a localized name for the specified QDesk.model.security.UserSessionOrigin value.
   * @param userSessionOrigin a QDesk.model.security.UserSessionOrigin value.
   * @return the localized user session origin name.
   */
  getUserSessionOriginName : function ( userSessionOrigin )
  {
    return QDesk.Msg.get( 'QDesk.model.security.UserSessionOrigin.' + userSessionOrigin );
  },


  /**
   * Returns a localized name for the specified QDesk.model.common.Weekday value.
   * @param weekday a QDesk.model.common.Weekday value.
   * @return the localized weekday name.
   */
  getWeekdayName : function ( weekday )
  {
    return QDesk.Msg.get( 'QDesk.model.common.Weekday.' + weekday );
  },


  /**
   * Returns a localized short name for the specified QDesk.model.common.Weekday value.
   * @param weekday a QDesk.model.common.Weekday value.
   * @return the localized short weekday name.
   */
  getWeekdayShortName : function ( weekday )
  {
    return QDesk.Msg.get( 'QDesk.model.common.Weekday.shortName.' + weekday );
  },


  /**
   * Returns a localized name for the specified QDesk.model.message.channel.web_service.WSAuthenticationType value.
   * @param wsAuthenticationType a QDesk.model.message.channel.web_service.WSAuthenticationType value.
   * @return the localized WS authentication type name.
   */
  getWSAuthenticationTypeName : function ( wsAuthenticationType )
  {
    return QDesk.Msg.get( 'QDesk.model.message.channel.web_service.WSAuthenticationType.' + wsAuthenticationType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.security.UserAccountStatus value.
   * @param userAccountStatus a QDesk.model.security.UserAccountStatus value.
   * @return the localized user account status name.
   */
  getUserAccountStatusName : function ( userAccountStatus )
  {
    return QDesk.Msg.get( 'QDesk.model.security.UserAccountStatus.' + userAccountStatus );
  },


  /**
   * Returns a localized name for the specified QDesk.model.security.Permission value.
   * @param permission a QDesk.model.security.Permission value.
   * @return the localized permission name.
   */
  getPermissionName : function ( permission )
  {
    // currently permission names are not localized
    return permission;

    //return QDesk.Msg.get( 'QDesk.model.security.Permission.' + permission );
  },


  /**
   * Returns a localized description for the specified QDesk.model.security.Permission value.
   * @param permission a QDesk.model.security.Permission value.
   * @return the localized permission description.
   */
  getPermissionDescription : function ( permission )
  {
    return QDesk.Msg.get( 'QDesk.model.security.Permission.description.' + permission );
  },


  /**
   * Returns a localized name for the specified QDesk.model.security.EffectiveUserPermissionSource value.
   * @param source a QDesk.model.security.EffectiveUserPermissionSource value.
   * @return the localized source name.
   */
  getEffectiveUserPermissionSourceName : function ( source )
  {
    return QDesk.Msg.get( 'QDesk.model.security.EffectiveUserPermissionSource.' + source );
  },


  /**
   * Returns a localized name for the specified QDesk.model.common.Result value.
   * @param result a QDesk.model.common.Result value.
   * @return the localized result name.
   */
  getResultName : function ( result )
  {
    return QDesk.Msg.get( 'QDesk.model.common.Result.' + result );
  },


  /**
   * Returns a localized name for the specified QDesk.model.common.Result value.
   * @param format a QDesk.model.common.Result value.
   * @return the localized format name.
   */
  getTabularDataExportFormatName : function ( format )
  {
    return QDesk.Msg.get( 'QDesk.model.common.TabularDataExportFormat.' + format );
  },


  /**
   * Returns a localized name for the specified QDesk.model.message.channel.MessageChannel value.
   * @param messageChannel a QDesk.model.message.channel.MessageChannel value.
   * @return the localized message channel name.
   */
  getMessageChannelName : function ( messageChannel )
  {
    return QDesk.Msg.get( 'QDesk.model.message.channel.MessageChannel.' + messageChannel );
  },


  /**
   * Returns a localized name for the specified QDesk.model.scheduler.ExecNotificationRuleConditionType value.
   * @param notificationConditionType a QDesk.model.scheduler.ExecNotificationRuleConditionType value.
   * @return the localized notification condition type name.
   */
  getExecNotificationRuleConditionTypeName : function ( notificationConditionType )
  {
    return QDesk.Msg.get( 'QDesk.model.scheduler.ExecNotificationRuleConditionType.' + notificationConditionType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.scheduler.ExecStatus value.
   * @param execStatus a QDesk.model.scheduler.ExecStatus value.
   * @return the localized execution status name.
   */
  getExecStatusName : function ( execStatus )
  {
    return QDesk.Msg.get( 'QDesk.model.scheduler.ExecStatus.' + execStatus );
  },


  /**
   * Returns a localized name for the specified QDesk.model.scheduler.ExecType value.
   * @param execType a QDesk.model.scheduler.ExecType value.
   * @return the localized execution type name.
   */
  getExecTypeName : function ( execType )
  {
    return QDesk.Msg.get( 'QDesk.model.scheduler.ExecType.' + execType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.scheduler.JobChainConditionType value.
   * @param jobChainConditionType a QDesk.model.scheduler.JobChainConditionType value.
   * @return the localized job chain condition type name.
   */
  getJobChainConditionTypeName : function ( jobChainConditionType )
  {
    return QDesk.Msg.get( 'QDesk.model.scheduler.JobChainConditionType.' + jobChainConditionType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.scheduler.JobChainTargetExecType value.
   * @param jobChainTargetExecType a QDesk.model.scheduler.JobChainTargetExecType value.
   * @return the localized job chain target exec type name.
   */
  getJobChainTargetExecTypeName : function ( jobChainTargetExecType )
  {
    return QDesk.Msg.get( 'QDesk.model.scheduler.JobChainTargetExecType.' + jobChainTargetExecType );
  },


  /**
   * Returns a localized name for the specified QDesk.model.scheduler.JobChainTargetType value.
   * @param jobChainTargetType a QDesk.model.scheduler.JobChainTargetType value.
   * @return the localized job chain target type name.
   */
  getJobChainTargetTypeName : function ( jobChainTargetType )
  {
    return QDesk.Msg.get( 'QDesk.model.scheduler.JobChainTargetType.' + jobChainTargetType );
  },


  /**
   * Returns a localized name for the specified QDesk.domain.common.jvm.ThreadState value.
   * @param threadState a QDesk.domain.common.jvm.ThreadState value.
   * @return the localized thread state name.
   */
  getThreadStateName : function ( threadState )
  {
    return QDesk.Msg.get( 'QDesk.domain.common.jvm.ThreadState.' + threadState );
  },


  /**
   * Returns a localized name for the specified QDesk.model.security.UserAccountLockReason value.
   * @param reason a QDesk.model.security.UserAccountLockReason value.
   * @return the localized user account lock reason.
   */
  getUserAccountLockReason : function ( reason )
  {
    return QDesk.Msg.get( 'QDesk.model.security.UserAccountLockReason.' + reason );
  }
} );
