/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * QDesk configuration singleton that encapsulates action URLs.
 */
Ext.define( 'QDesk.util.constant.EventName', {

  singleton : true,

  /**
   * Heartbeat event.
   * Arguments:
   * {
   *   heartbeat data
   * }
   */
  HEARTBEAT : 'heartbeat',

  /**
   * Application configuration changed.
   * Arguments:
   * {
   *   changes : an object containing the configuration changes
   * }
   */
  APP_CONFIG_CHANGED : 'applicationConfigChanged',


  /**
   * Application's viewport has been created.
   * Arguments:
   * {
   *   viewport : viewport
   * }
   */
  VIEWPORT_CREATED : 'viewportCreated',

  /**
   * Work panel has been activated.
   * Arguments:
   * {
   *   workPanel : work panel
   * }
   */
  WORK_PANEL_ACTIVATED : 'workPanelActivated',

  /**
   * Work panel has been deactivated.
   * Arguments:
   * {
   *   workPanel : work panel
   * }
   */
  WORK_PANEL_DEACTIVATED : 'workPanelDeactivated',


  /**
   * User logged in successfully.
   * Arguments:
   * {
   *   sessionToken : the session token
   *   userProfile : the user profile
   * }
   */
  LOGIN_SUCCESS : 'loginSuccess',

  /**
   * User login failed.
   * Arguments:
   * {
   *   sessionToken : the session token
   *   username : the user name
   * }
   */
  LOGIN_FAILED : 'loginFailed',

  /**
   * User logged out successfully.
   * Arguments:
   * {
   *   sessionToken : the session token
   *   automatic : a flag indicating if the logout was automatic after the user's session expired (true)
   *               or manual (false)
   * }
   */
  LOGOUT_SUCCESS : 'logoutSuccess',


  /**
   * The current user's authentication status changed.
   * Arguments:
   * {
   *   sessionToken : the session token
   *   oldStatus : the old authentication status
   *   newStatus : the new authentication status
   *   isFromHeartbeat : a flag indicating if this event was fired by the heartbeat
   * }
   */
  CURRENT_USER_AUTH_STATUS_CHANGED : 'currentUserAuthStatusChanged',

  /**
   * The current user's permissions changed.
   * Arguments:
   * {
   *   sessionToken : the session token
   *   oldEffectivePermissions : the old effective permissions
   *   newEffectivePermissions : the new effective permissions
   *   isFromHeartbeat : a flag indicating if this event was fired by the heartbeat
   * }
   */
  CURRENT_USER_PERMISSIONS_CHANGED : 'currentUserPermissionsChanged',


  /**
   * A user has been added.
   * Arguments:
   * {
   *   user : the user
   * }
   */
  USER_ADDED : 'userAdded',

  /**
   * A user has been updated.
   * Arguments:
   * {
   *   user : the user
   * }
   */
  USER_UPDATED : 'userUpdated',

  /**
   * A user has been deleted.
   * Arguments:
   * {
   *   userId : the user ID
   * }
   */
  USER_DELETED : 'userDeleted',

  /**
   * A user has been locked.
   * Arguments:
   * {
   *   userId : the user ID
   * }
   */
  USER_LOCKED : 'userLocked',

  /**
   * A user has been unlocked.
   * Arguments:
   * {
   *   userId : the user ID
   * }
   */
  USER_UNLOCKED : 'userUnlocked',


  /**
   * A group has been added.
   * Arguments:
   * {
   *   group : the group
   * }
   */
  GROUP_ADDED : 'groupAdded',

  /**
   * A group has been updated.
   * Arguments:
   * {
   *   group : the group
   * }
   */
  GROUP_UPDATED : 'groupUpdated',

  /**
   * A group has been deleted.
   * Arguments:
   * {
   *   groupId : the group ID
   * }
   */
  GROUP_DELETED : 'groupDeleted',


  /**
   * Fired after a connection folder has been added.
   * Arguments:
   * {
   *   folderId : the ID of the newly added folder
   * }
   */
  CONNECTION_FLD_ADDED : 'connectionFolderAdded',

  /**
   * Fired after a connection folder has been updated.
   * Arguments:
   * {
   *   folderId : the ID of the updated folder
   * }
   */
  CONNECTION_FLD_UPDATED : 'connectionFolderUpdated',

  /**
   * Fired after a connection folder has been deleted.
   * Arguments:
   * {
   *   folderId : the ID of the deleted folder
   *   folderPath : the deleted folder's path
   * }
   */
  CONNECTION_FLD_DELETED : 'connectionFolderDeleted',

  /**
   * Fired after a connection folder has been updated.
   * Arguments:
   * {
   *   folderId : the ID of the moved folder
   *   folderPath : the moved folder's path
   *   targetFolderId : the ID of the target folder
   *   targetFolderPath : the path of the target folder
   * }
   */
  CONNECTION_FLD_MOVED : 'connectionFolderMoved',


  /**
   * Fired after a new connection has been added.
   * Arguments:
   * {
   *   connectionId : the ID of the new connection
   * }
   */
  CONNECTION_ADDED : 'schedulerConnectionAdded',

  /**
   * Fired after a connection has been updated.
   * Arguments:
   * {
   *   connectionId : the ID of the new connection
   * }
   */
  CONNECTION_UPDATED : 'schedulerConnectionUpdated',

  /**
   * Fired after a connection has been deleted.
   * Arguments:
   * {
   *   connectionId : the ID of the deleted connection
   * }
   */
  CONNECTION_DELETED : 'schedulerConnectionDeleted',

  /**
   * Fired after a connection has been deleted.
   * Arguments:
   * {
   *   connectionId : the ID of the moved connection
   *   targetFolderId : the ID of the target folder
   *   targetFolderPath : the path of the target folder
   * }
   */
  CONNECTION_MOVED : 'schedulerConnectionMoved',


  /**
   * Fired after a scheduler has been started (resumed).
   * Arguments:
   * {
   *   connectionId : the ID of the scheduler's connection
   * }
   */
  SCHEDULER_STARTED : 'schedulerStarted',

  /**
   * Fired after a scheduler has been paused.
   * Arguments:
   * {
   *   connectionId : the ID of the scheduler's connection
   * }
   */
  SCHEDULER_PAUSED : 'schedulerPaused',

  /**
   * Fired after a scheduler has been stopped.
   * Arguments:
   * {
   *   connectionId : the ID of the scheduler's connection
   * }
   */
  SCHEDULER_STOPPED : 'schedulerStopped',


  QUARTZ : {
    /**
     * Fired after a new Quartz job has been added.
     * Arguments:
     * {
        *   connectionId : the ID of the scheduler's connection
        *   jobDetail : the job detail
        * }
     */
    JOB_ADDED : 'quartzJobAdded',

    /**
     * Fired after a Quartz job has been updated.
     * Arguments:
     * {
        *   connectionId : the ID of the scheduler's connection
        *   jobDetail : the job detail
        * }
     */
    JOB_UPDATED : 'quartzJobUpdated',

    /**
     * Fired after a Quartz job has been deleted.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   jobGroupName : the job group name
       *   jobName : the job name
       * }
     */
    JOB_DELETED : 'quartzJobDeleted',

    /**
     * Fired after a Quartz job has been interrupted.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   executingJob : the executing job data
       * }
     */
    JOB_INTERRUPTED : 'quartzJobInterrupted',

    /**
     * Fired after a Quartz job has been manually triggered.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   jobDetail : the job detail
       * }
     */
    JOB_TRIGGERED_MANUALLY : 'quartzJobTriggeredManually',


    /**
     * Fired after a Quartz job group has been paused.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   jobGroupName : the job group name
       * }
     */
    JOB_GROUP_PAUSED : 'quartzJobGroupPaused',

    /**
     * Fired after a Quartz job group has been resumed.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   jobGroupName : the job group name
       * }
     */
    JOB_GROUP_RESUMED : 'quartzJobGroupResumed',


    /**
     * Fired after a new Quartz trigger has been added.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   trigger : the trigger
       * }
     */
    TRIGGER_ADDED : 'quartzTriggerAdded',

    /**
     * Fired after a Quartz trigger has been updated.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   trigger : the trigger
       * }
     */
    TRIGGER_UPDATED : 'quartzTriggerUpdated',

    /**
     * Fired after a Quartz trigger has been deleted.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   triggerGroupName : the trigger group name
       *   triggerName : the trigger name
       * }
     */
    TRIGGER_DELETED : 'quartzTriggerDeleted',

    /**
     * Fired after a Quartz trigger has been paused.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   triggerGroupName : the trigger group name
       *   triggerName : the trigger name
       * }
     */
    TRIGGER_PAUSED : 'quartzTriggerPaused',

    /**
     * Fired after a Quartz trigger has been resumed.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   triggerGroupName : the trigger group name
       *   triggerName : the trigger name
       * }
     */
    TRIGGER_RESUMED : 'quartzTriggerResumed',


    /**
     * Fired after a Quartz trigger group has been paused.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   triggerGroupName : the trigger group name
       * }
     */
    TRIGGER_GROUP_PAUSED : 'quartzTriggerGroupPaused',

    /**
     * Fired after a Quartz trigger group has been resumed.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   triggerGroupName : the trigger group name
       * }
     */
    TRIGGER_GROUP_RESUMED : 'quartzTriggerGroupResumed',


    /**
     * Fired after a new Quartz job chain has been added.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   notificationRule : the execution notification rule
       * }
     */
    EXEC_NOTIF_RULE_ADDED : 'quartzExecNotifRuleAdded',

    /**
     * Fired after a Quartz job chain has been updated.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   notificationRule : the execution notification rule
       * }
     */
    EXEC_NOTIF_RULE_UPDATED : 'quartzExecNotifRuleUpdated',

    /**
     * Fired after a Quartz job chain has been deleted.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   notificationRule : the execution notification rule
       * }
     */
    EXEC_NOTIF_RULE_DELETED : 'quartzExecNotifRuleDeleted',

    /**
     * Fired after a Quartz job chain has been enabled.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   notificationRule : the execution notification rule
       * }
     */
    EXEC_NOTIF_RULE_ENABLED : 'quartzExecNotifRuleEnabled',

    /**
     * Fired after a Quartz job chain has been disabled.
     * Arguments:
     * {
       *   connectionId : the ID of the scheduler's connection
       *   notificationRule : the execution notification rule
       * }
     */
    EXEC_NOTIF_RULE_DISABLED : 'quartzExecNotifRuleDisabled',


    /**
     * Fired after a new Quartz job chain has been added.
     * Arguments:
     * {
     *   connectionId : the ID of the scheduler's connection
     *   jobChain : the job chain
     * }
     */
    JOB_CHAIN_ADDED : 'quartzJobChainAdded',

    /**
     * Fired after a Quartz job chain has been updated.
     * Arguments:
     * {
     *   connectionId : the ID of the scheduler's connection
     *   jobChain : the job chain
     * }
     */
    JOB_CHAIN_UPDATED : 'quartzJobChainUpdated',

    /**
     * Fired after a Quartz job chain has been deleted.
     * Arguments:
     * {
     *   connectionId : the ID of the scheduler's connection
     *   jobChain : the job chain
     * }
     */
    JOB_CHAIN_DELETED : 'quartzJobChainDeleted',

    /**
     * Fired after a Quartz job chain has been enabled.
     * Arguments:
     * {
     *   connectionId : the ID of the scheduler's connection
     *   jobChain : the job chain
     * }
     */
    JOB_CHAIN_ENABLED : 'quartzJobChainEnabled',

    /**
     * Fired after a Quartz job chain has been disabled.
     * Arguments:
     * {
     *   connectionId : the ID of the scheduler's connection
     *   jobChain : the job chain
     * }
     */
    JOB_CHAIN_DISABLED : 'quartzJobChainDisabled'
  }
} );
