/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various notifications related utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzExecNotificationRuleUtils', {
  singleton : true,

  requires : [
    'QDesk.Properties',
    'QDesk.model.common.Weekday',
    'QDesk.model.message.channel.MessageChannel',
    'QDesk.model.scheduler.ExecNotificationRuleConditionType',

    'QDesk.util.MessageChannelProfileUtils'
  ],


  /**
   * Creates a QDesk.model.scheduler.quartz.QuartzExecNotificationRule model instance and populates it with
   * the specified EditExecNotificationRuleForm field values.
   * @param fieldValues a EditExecNotificationRuleForm field values.
   * @return the created QDesk.model.scheduler.quartz.QuartzExecNotificationRule model instance.
   */
  createNotificationRule : function ( fieldValues )
  {
    var me = this;

    var notificationRule = Ext.create( 'QDesk.model.scheduler.quartz.QuartzExecNotificationRule' );

    if ( !Ext.isEmpty( fieldValues.id ) )
      notificationRule.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.schedulerObjectName ) )
      notificationRule.set( 'schedulerObjectName', fieldValues.schedulerObjectName );

    if ( !Ext.isEmpty( fieldValues.createdAt ) )
      notificationRule.set( 'createdAt', fieldValues.createdAt );

    if ( !Ext.isEmpty( fieldValues.updatedAt ) )
      notificationRule.set( 'updatedAt', fieldValues.updatedAt );

    /* Start of Quartz specific fields */
    if ( !Ext.isEmpty( fieldValues.jobGroupName ) )
      notificationRule.set( 'jobGroupName', fieldValues.jobGroupName );

    if ( !Ext.isEmpty( fieldValues.jobName ) )
      notificationRule.set( 'jobName', fieldValues.jobName );

    if ( !Ext.isEmpty( fieldValues.triggerGroupName ) )
      notificationRule.set( 'triggerGroupName', fieldValues.triggerGroupName );

    if ( !Ext.isEmpty( fieldValues.triggerName ) )
      notificationRule.set( 'triggerName', fieldValues.triggerName );
    /* End of Quartz specific fields */

    if ( !Ext.isEmpty( fieldValues.name ) )
      notificationRule.set( 'name', fieldValues.name );

    if ( !Ext.isEmpty( fieldValues.description ) )
      notificationRule.set( 'description', fieldValues.description );

    if ( !Ext.isEmpty( fieldValues.severity ) )
      notificationRule.set( 'severity', fieldValues.severity );

    notificationRule.set( 'enabled', fieldValues.enabled );

    if ( !Ext.isEmpty( fieldValues.enabledFromTime ) )
      notificationRule.set( 'enabledFromTime', QDesk.util.FormatUtils.formatDayTime( fieldValues.enabledFromTime ) );

    if ( !Ext.isEmpty( fieldValues.enabledToTime ) )
      notificationRule.set( 'enabledToTime', QDesk.util.FormatUtils.formatDayTime( fieldValues.enabledToTime ) );

    notificationRule.set( 'enabledOnWeekday', me._createEnabledOnWeekdaysArray( fieldValues ) );

    if ( !Ext.isEmpty( fieldValues.conditionType ) )
      notificationRule.set( 'conditionType', fieldValues.conditionType );

    if ( fieldValues.conditionType === QDesk.model.scheduler.ExecNotificationRuleConditionType.EXEC_STATUS )
      notificationRule.set( 'conditionExecStatus', fieldValues.conditionExecStatus );

    if ( fieldValues.conditionType === QDesk.model.scheduler.ExecNotificationRuleConditionType.MAX_DURATION )
      notificationRule.set( 'conditionMaxDuration', fieldValues.conditionMaxDuration );

    if ( fieldValues.conditionType === QDesk.model.scheduler.ExecNotificationRuleConditionType.EXPRESSION )
      notificationRule.set( 'conditionExpression', fieldValues.conditionExpression );

    if ( !Ext.isEmpty( fieldValues.messageChannelProfileId ) )
      notificationRule.set( 'messageChannelProfileId', fieldValues.messageChannelProfileId );

    if ( !Ext.isEmpty( fieldValues.messageToAddr ) )
      notificationRule.set( 'messageToAddr', fieldValues.messageToAddr );

    if ( !Ext.isEmpty( fieldValues.messageSubjectTemplate ) )
      notificationRule.set( 'messageSubjectTemplate', fieldValues.messageSubjectTemplate );

    if ( !Ext.isEmpty( fieldValues.messageBodyTemplate ) )
      notificationRule.set( 'messageBodyTemplate', fieldValues.messageBodyTemplate );

    // attachments
    if ( QDesk.util.MessageChannelProfileUtils.messageChannelSupportsAttachments(
            fieldValues.messageChannelProfileChannel ) )
    {
      notificationRule.set( 'messageAttachLog', fieldValues.messageAttachLog );
      notificationRule.set( 'messageAttachError', fieldValues.messageAttachError );
      notificationRule.set( 'messageAttachResult', fieldValues.messageAttachResult );
      notificationRule.set( 'messageAttachUserData', fieldValues.messageAttachUserData );
    }
    else
    {
      notificationRule.set( 'messageAttachLog', false );
      notificationRule.set( 'messageAttachError', false );
      notificationRule.set( 'messageAttachResult', false );
      notificationRule.set( 'messageAttachUserData', false );
    }

    return notificationRule;
  },


  /**
   * Returns an array with weekdays the notification rule is enabled on.
   * @param fieldValues form field values.
   * @return the weekday array.
   */
  _createEnabledOnWeekdaysArray : function ( fieldValues )
  {
    var enabledOnWeekdays = [];

    if ( fieldValues.enabledOnMonday )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.MONDAY );

    if ( fieldValues.enabledOnTuesday )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.TUESDAY );

    if ( fieldValues.enabledOnWednesday )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.WEDNESDAY );

    if ( fieldValues.enabledOnThursday )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.THURSDAY );

    if ( fieldValues.enabledOnFriday )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.FRIDAY );

    if ( fieldValues.enabledOnSaturday )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.SATURDAY );

    if ( fieldValues.enabledOnSunday )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.SUNDAY );

    return enabledOnWeekdays;
  }
} );
