/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.lock.LockPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.About'
  ],

  alias : 'widget.LockPanel',

  cls : 'lockPanel',

  border : false,

  bodyPadding : '0 30 0 30',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    // if ( !config.qdesk )
    //   Ext.Error.raise( 'Required config parameter missing: qdesk' );
    //
    // if ( !config.qdesk.msg )
    //   Ext.Error.raise( 'Required config parameter missing: qdesk.msg' );

    Ext.apply( config, {
      items : [
        {
          cls : 'title',
          html : QDesk.Msg.get( 'QDesk.view.lock.LockPanel.title' ),
          border : false,
          flex : 1
        },

        {
          cls : 'message',
          itemId : 'message',
          html : config.qdesk && config.qdesk.msg ? config.qdesk.msg : '',
          hidden : !(config.qdesk && config.qdesk.msg),
          border : false,
          flex : 1
        },

        {
          xtype : 'panel',
          border : false,
          itemId : 'bottomLinks',
          cls : 'bottomLinks',
          layout : {
            type : 'hbox',
            align : 'stretch',
            pack : 'center'
          },
          items : [
            {
              xtype : 'displayfield',
              value : '&copy;',
              margin : '0 5 0 0'
            },
            {
              xtype : 'displayfield',
              cls : 'link',
              itemId : 'linkCopyright',
              value : QDesk.Msg.get( 'QDesk.view.lock.LockPanel.bottomLinkCopyright' )

            },
            {
              xtype : 'displayfield',
              cls : 'divider',
              value : '|'
            },
            {
              xtype : 'displayfield',
              cls : 'link',
              itemId : 'linkTermsAndConditions',
              value : QDesk.Msg.get( 'QDesk.view.lock.LockPanel.bottomLinkTermsAndConditions' )
            },
            {
              xtype : 'displayfield',
              cls : 'divider',
              value : '|'
            },
            {
              xtype : 'displayfield',
              cls : 'link',
              itemId : 'linkAbout',
              value : QDesk.Msg.get( 'QDesk.view.lock.LockPanel.bottomLinkAbout' )
            }
          ]
        }
      ]
    } );

    delete config.qdesk;

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Binds event handlers.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    /*
     * Add click events to all links in the bottomLinks panel. These are not HTML links and as such they
     * do not fire click events when clicked.
     */
    Ext.each( Ext.ComponentQuery.query( '#bottomLinks > displayfield[cls=link]', me ),
        function ( link )
        {
          link.on( 'afterrender', function ()
          {
            link.getEl().on( 'click', function ()
            {
              link.fireEvent( 'click' );
            } );
          } );
        } );

    me.down( '#linkCopyright' ).on( 'click', function ()
    {
      window.open( QDesk.Msg.get( 'QDesk.link.productHome' ), '_blank' );
    } );

    me.down( '#linkTermsAndConditions' ).on( 'click', function ()
    {
      var url = null;
      if ( QDesk.util.license.LicensedProduct.productEdition === QDesk.model.license.ProductEdition.LITE )
      {
        url = QDesk.Msg.get( 'QDesk.link.productTermsAndConditionsFree' );
      }
      else
      {
        url = QDesk.Msg.get( 'QDesk.link.productTermsAndConditionsCommercial' );
      }
      window.open( url, '_blank' );
    } );

    me.down( '#linkAbout' ).on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.About', {} ).execute();
    } );
  }
} );
