/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Panel with a form that allows a user to reset their password.
 */
Ext.define( 'QDesk.view.security.ForgotPasswordPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.util.license.LicensedProduct',
    'QDesk.view.security.ForgotPasswordForm',

    'QDesk.controller.command.security.ResetPasswordInit'
  ],

  alias : 'widget.ForgotPasswordPanel',

  border : false,

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'center'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'ForgotPasswordForm',
          flex : 1
        },
        {
          xtype : 'panel',
          border : false,
          itemId : 'navigationLinks',
          layout : {
            type : 'hbox',
            align : 'stretch',
            pack : 'center'
          },
          items : [
            // {
            //   xtype : 'displayfield',
            //   cls : 'link',
            //   itemId : 'forgotUsernameLink',
            //   value : QDesk.Msg.get( 'QDesk.view.security.ForgotPasswordPanel.forgotUsernameLink' ),
            //   margin : '0 10 0 10'
            // },
            // {
            //   xtype : 'displayfield',
            //   cls : 'link',
            //   itemId : 'loginLink',
            //   value : QDesk.Msg.get( 'QDesk.view.security.ForgotPasswordPanel.loginLink' ),
            //   margin : '0 10 0 10'
            // }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Binds event handlers.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    me.on( 'activate', function ()
    {
      var usernameField = me.down( '#username' );
      QDesk.util.UxUtils.focus( usernameField, true );

      return true;
    } );


    /*
     * Add click events to all links in the navigationLinks panel. These are not HTML links and as such they
     * do not fire click events when clicked.
     */
    Ext.each( Ext.ComponentQuery.query( '#navigationLinks > displayfield[cls=link]', me ),
        function ( link )
        {
          link.on( 'afterrender', function ()
          {
            link.getEl().on( 'click', function ()
            {
              link.fireEvent( 'click' );
            } );
          } );
        } );


    me.down( '#resetPasswordButton' ).on( 'click', function ()
    {
      var username = me.down( '#username' ).getValue();

      Ext.create( 'QDesk.controller.command.security.ResetPasswordInit', {
        username : username
      } ).execute();
    } );


    // me.down( '#forgotUsernameLink' ).on( 'click', function ()
    // {
    //   var securityPanel = me.up( '#securityPanel' );
    //   securityPanel.activateForgotUsernamePanel();
    // } );
    //
    // me.down( '#loginLink' ).on( 'click', function ()
    // {
    //   var securityPanel = me.up( '#securityPanel' );
    //   securityPanel.activateLoginPanel();
    // } );
  }
} );
