/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Action column that extends the standard ExtJS Action column class and allows for displaying the
 * textual column value. The column value can be displayed on the left or on the right (default) from
 * the configured action icons.
 */
Ext.define( 'QDesk.view.widget.WActionColumn', {
  extend : 'Ext.grid.column.Action',

  alias : 'widget.WActionColumn',

  hideItemsIfValueNull : false,

  constructor : function ( config )
  {
    var me = this;

    // if menuText not explicitely defined, we use the text value (column header value)
    if ( config.text && !config.menuText )
    {
      Ext.apply( config, {
        menuText : config.text
      } )
    }

    me.callParent( [ config ] );
    me.innerCls = null;
  },


  // Renderer closure iterates through items creating an <img> element for each and tagging with an identifying
  // class name x-action-col-{n}
  defaultRenderer : function ( v, cellValues, record, rowIdx, colIdx, store, view )
  {
    var me = this,
        scope = me.origScope || me,
        items = me.items,
        len = items.length,
        i, item, ret, disabled, tooltip, altText, icon;

    // If value is null, we can hide all items (action icons) alongside with the value.
    // The default behavior is not to hide action icons if the value is null.
    if ( !v && me.hideItemsIfValueNull )
    {
      return null;
    }

    var renderValue;
    if ( Ext.isFunction( me.origRenderer ) )
    {
      renderValue = me.origRenderer.apply( scope, arguments )
    }

    ret = '';

    cellValues.tdCls += ' ' + Ext.baseCSSPrefix + 'action-col-cell';

    for ( i = 0; i < len; i++ )
    {
      item = items[ i ];
      icon = item.icon;

      disabled = item.disabled ||
          (item.isActionDisabled ?
              item.isActionDisabled.call( item.scope || scope, view, rowIdx, colIdx, item, record ) : false);

      tooltip = disabled ? null :
          (item.tooltip || (item.getTip ? item.getTip.apply( item.scope || scope, arguments ) : null));

      altText = item.getAltText ? item.getAltText.apply( item.scope || scope, arguments ) : item.altText || me.altText;

      // Only process the item action setup once.
      if ( !item.hasActionConfiguration )
      {
        // Apply our documented default to all items
        item.stopSelection = me.stopSelection;
        item.disable = Ext.Function.bind( me.disableAction, me, [ i ], 0 );
        item.enable = Ext.Function.bind( me.enableAction, me, [ i ], 0 );
        item.hasActionConfiguration = true;
      }

      ret += '<' + (icon ? 'img' : 'div') + ' tabIndex="0" role="button"' +
          (icon ? (' align="top" alt="' + altText + '" src="' + item.icon + '"') : '') +
          ' class="' + me.actionIconCls + ' ' + Ext.baseCSSPrefix + 'action-col-' + String( i ) + ' ' +
          (disabled ? me.disabledCls + ' ' : ' ') +
          (Ext.isFunction( item.getClass ) ? item.getClass.apply( item.scope || scope, arguments ) :
              (item.iconCls || me.iconCls || '')) + '"' +
          (tooltip ? ' data-qtip="' + tooltip + '"' : '') + (icon ? '/>' : '></div>');
    }

    if ( renderValue )
    {
      if ( Ext.isEmpty( items ) )
      {
        ret += '<span>' + renderValue + '</span>';
      }
      else
      {
        ret += '<span style="padding-left: 5px">' + renderValue + '</span>';
      }
    }

    return ret;
  }
} );
