/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.WClearableField', {
  extend : 'Ext.form.field.Text',

  alias : 'widget.WClearableField',


  constructor : function ( config )
  {
    var me = this;

    var triggers = {};

    Ext.apply( triggers, {
      clear : {
        weight : 0,
        cls : Ext.baseCSSPrefix + 'form-clear-trigger',
        handler : '_onClearClick',
        scope : 'this',
        isFieldEnabled : function ()
        {
          // trigger is enabled if the field is enabled and not empty
          return !me.isDisabled() && !Ext.isEmpty( me.getValue() );
        }
      }
    } );

    Ext.apply( config, {
      triggers : triggers,
      listeners : {
        change : {
          fn : me._updateTriggers
        },
        afterrender : {
          // set initial state of the clear trigger based on the initial text field value
          fn : me._updateTriggers
        }
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent( arguments );

    me.on( 'specialkey', function ( f, e )
    {
      if ( e.getKey() == e.ENTER )
      {
        me.fireEvent( 'enter', me );
      }
    } );
  },


  _onClearClick : function ()
  {
    var me = this;
    var value = me.getValue();

    if ( Ext.isEmpty( value ) )
    {
      // no value entered
    }
    else
    {
      me.setValue( '' );

      var reloadStore = me.fireEvent( 'clear', me );

      if ( me.store && me.store.isStore )
      {
        var extraParams = me.store.getProxy().extraParams;
        if ( extraParams && extraParams[ me.paramName ] )
        {
          delete extraParams[ me.paramName ];
          me.store.getProxy().extraParams = extraParams;

          if ( reloadStore )
          {
            me.store.load();
          }
        }
      }
    }
  },


  _updateTriggers : function ()
  {
    var me = this;
    var clearTrigger = me.getTrigger( 'clear' );
    var clearTriggerEl = clearTrigger.getEl();

    if ( Ext.isEmpty( me.getValue() ) )
    {
      // disable the clear trigger
      clearTriggerEl.mask();
    }
    else
    {
      // enable the clear trigger
      clearTriggerEl.unmask();
    }
  }
} );