/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.WSearchField', {
  extend : 'Ext.form.field.Text',

  alias : 'widget.WSearchField',


  constructor : function ( config )
  {
    var me = this;

    var triggers = {};

    Ext.apply( triggers, {
      search : {
        weight : 0,
        cls : Ext.baseCSSPrefix + 'form-search-trigger',
        handler : '_onSearchClick',
        scope : 'this',
        isFieldEnabled : function ()
        {
          // search trigger is enabled if the field is enabled and not empty
          return !me.isDisabled() && !Ext.isEmpty( me.getValue() );
        }
      }
    } );

    Ext.apply( triggers, {
      clear : {
        weight : 1,
        cls : Ext.baseCSSPrefix + 'form-clear-trigger',
        handler : '_onClearClick',
        scope : 'this',
        isFieldEnabled : function ()
        {
          // clear trigger is enabled if the field is enabled and not empty
          return !me.isDisabled() && !Ext.isEmpty( me.getValue() );
        }
      }
    } );

    Ext.apply( config, {
      triggers : triggers
    } );

    me.callParent( [ config ] );

    me.on( 'change', me._updateTriggers );

    // set initial state of the triggers based on the initial text field value
    me.on( 'afterrender', me._updateTriggers );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent( arguments );

    me.on( 'specialkey', function ( f, e )
    {
      if ( e.getKey() == e.ENTER )
      {
        me._onSearchClick();
      }
    } );

    if ( !me.store || !me.store.isStore )
    {
      me.store = Ext.data.StoreManager.lookup( me.store );
    }
  },


  _onSearchClick : function ()
  {
    var me = this;
    var value = me.getValue();

    if ( Ext.isEmpty( value ) )
    {
      // no value entered
    }
    else
    {
      var reloadStore = me.fireEvent( 'search', me, value );

      if ( me.store && me.store.isStore )
      {
        var extraParams = me.store.getProxy().extraParams;
        extraParams = extraParams || {};
        extraParams[ me.paramName ] = value;

        me.store.getProxy().extraParams = extraParams;

        if ( reloadStore )
        {
          me.store.load();
        }
      }
    }
  },


  _onClearClick : function ()
  {
    var me = this;
    var value = me.getValue();

    if ( Ext.isEmpty( value ) )
    {
      // no value entered
    }
    else
    {
      me.setValue( '' );

      var reloadStore = me.fireEvent( 'clear', me );

      if ( me.store && me.store.isStore )
      {
        var extraParams = me.store.getProxy().extraParams;
        if ( extraParams && extraParams[ me.paramName ] )
        {
          delete extraParams[ me.paramName ];
          me.store.getProxy().extraParams = extraParams;

          if ( reloadStore )
          {
            me.store.load();
          }
        }
      }
    }
  },


  _updateTriggers : function ()
  {
    var me = this;

    var searchTrigger = me.getTrigger( 'search' );
    var searchTriggerEl = searchTrigger.getEl();

    var clearTrigger = me.getTrigger( 'clear' );
    var clearTriggerEl = clearTrigger.getEl();

    if ( Ext.isEmpty( me.getValue() ) )
    {
      // disable both triggers
      searchTriggerEl.mask();
      clearTriggerEl.mask();
    }
    else
    {
      // enable both triggers
      searchTriggerEl.unmask();
      clearTriggerEl.unmask();
    }
  }
} );