/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.expression.JSScriptingObjectsPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.view.widget.message.InfoMessagePanel'
  ],

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  border : false,

  padding : 5,

  alias : 'widget.expression.JSScriptingObjectsPanel',

  cls : 'jsScriptingObjectsPanel',

  // required config param holding the tree store with the data types data
  store : null,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.store )
      Ext.Error.raise( 'Required config parameter missing: store' );

    Ext.apply( config, {
      items : [
        {
          xtype : 'InfoMessagePanel',
          margin : '0 0 5 0',
          qdesk : {
            msg : QDesk.Msg.get( 'QDesk.view.widget.expression.JSScriptingObjectsPanel.description' )
          }
        },
        {
          xtype : 'treepanel',
          itemId : 'objectsTreePanel',
          border : true,
          layout : 'fit',
          flex : 1,
          cls : 'objectsTreePanel',
          scrollable : true,
          collapsible : false,
          useArrows : true,
          rootVisible : false,
          multiSelect : false,
          singleExpand : false,
          displayField : 'propertyName',
          store : config.store,
          columns : [
            {
              xtype : 'treecolumn',
              text : QDesk.Msg.get( 'QDesk.view.widget.expression.JSScriptingObjectsPanel.textPropertyName' ),
              flex : 2.7,
              dataIndex : 'propertyName',
              sortable : false,
              renderer : function ( value, metaData, record )
              {
                var propertyType = record.get( 'propertyType' );
                if ( record.isLeaf() && propertyType === 'Object' || propertyType === 'Function' )
                {
                  metaData.style = 'text-decoration: underline';
                }
                return value;
              }
            },
            {
              text : QDesk.Msg.get( 'QDesk.view.widget.expression.JSScriptingObjectsPanel.textPropertyType' ),
              flex : 1,
              dataIndex : 'propertyType',
              sortable : false,
              renderer : function ( value )
              {
                return '<span class="pre">' + value + '</span>';
              }
            },
            {
              text : QDesk.Msg.get( 'QDesk.view.widget.expression.JSScriptingObjectsPanel.textDescription' ),
              flex : 6,
              dataIndex : 'description',
              sortable : false
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );
