/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.widget.expression.JSScriptingObjectsWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.widget.expression.JSScriptingObjectsForm'
  ],

  alias : 'widget.expression.JSScriptingObjectsWindow',

  cls : 'jsScriptingObjectsWindow',
  iconCls : 'windowIcon',

  modal : true,
  maximizable : true,
  resizable : true,

  width : 850,
  height : 500,

  layout : 'fit',

  // required config parameter with the objects tree store
  objectsStore : null,

  // required config parameter with the data types tree store
  dataTypesStore : null,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.objectsStore )
      Ext.Error.raise( 'Required config parameter missing: objectsStore' );

    if ( !config.dataTypesStore )
      Ext.Error.raise( 'Required config parameter missing: dataTypesStore' );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.widget.expression.JSScriptingObjectsWindow.windowTitle' ),
      items : [
        {
          xtype : 'expression.JSScriptingObjectsForm',
          itemId : 'form',
          objectsStore : config.objectsStore,
          dataTypesStore : config.dataTypesStore
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    me.down( '#closeButton' ).on( 'click', function ()
    {
      me.close();
    } );

    me.down( '#objectsPanel #objectsTreePanel' ).on( 'itemdblclick',
        function ( treePanel, record, item, index, event )
        {
          var propertyType = record.get( 'propertyType' );
          if ( record.isLeaf() && propertyType === 'Object' || propertyType === 'Function' )
          {
            var propertyName = record.get( 'propertyName' );
            var value = record.getPath( 'propertyName', '.' ).substring( 2 );
            me.fireEvent( 'valueSelected', value );
            me.close();
          }
        }, me );

    me.down( '#dataTypesPanel #dataTypesTreePanel' ).on( 'itemdblclick',
        function ( treePanel, record, item, index, event )
        {
          if ( record.isLeaf() )
          {
            var propertyName = record.get( 'propertyName' );

            // remove '..' at the start
            var value = record.getPath( 'propertyName', '.' ).substring( 2 );

            var i = value.indexOf( '.' );
            if ( i > 0 )
            {
              // remove the data type name from the selected value
              value = value.substring( i );
            }
            me.fireEvent( 'valueSelected', value );
            me.close();
          }
        }, me );
  }
} );
