/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.FolderPanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.model.security.Permission'
  ],

  alias : 'widget.FolderPanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '10px 10px 5px 10px',
          items : [
            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.FolderPanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.FolderPanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.FolderPanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            }
          ]
        }
      ],

      buttons : [
//    {
//      itemId: 'resetButton',
//      text: QDesk.Msg.get( 'QDesk.buttonLabelReset' )
//    },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_FLD_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_FLD_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified folder object.
   * @param folder a folder object.
   */
  populate : function ( folder )
  {
    var me = this;

    var values = {};

    Ext.apply( values, {
      id : folder.folderId,  // use folderId (number) rather then id (textual node ID)
      name : folder.name
    } );

    me.getForm().setValues( values );

    // if root node
    if ( folder.folderId == 1 )
    {
      me.down( '#name' ).setDisabled( true );
//      me.down( '#resetButton' ).hide();
      me.down( '#saveButton' ).hide();
    }
  }
} );
