/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.ProxyPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.store.enums.ProxyTypeStore'
  ],

  alias : 'widget.ProxyPanel',

  border : false,

  layout : 'anchor',

  defaults : {
    anchor : '100%',
    labelWidth : 85
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.tabLabelProxy' ),

      items : [
        {
          xtype : 'combo',
          itemId : 'proxyType',
          name : 'proxyType',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldLabelProxyType' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldTooltipProxyType' ),
          value : '',
          allowBlank : false,
          editable : false,
          forceSelection : true,
          queryMode : 'local',
          store : 'enums.ProxyTypeStore',
          valueField : 'value',
          displayField : 'valueLabel'
        },

        {
          xtype : 'textfield',
          itemId : 'proxyHost',
          name : 'proxyHost',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldLabelProxyHost' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldTooltipProxyHost' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldEmptyTxtProxyHost' ),
          allowBlank : true,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'numberfield',
          itemId : 'proxyPort',
          name : 'proxyPort',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldLabelProxyPort' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldTooltipProxyPort' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldEmptyTxtProxyPort' ),
          minValue : 1,
          maxValue : 65535,
          anchor : '60%',
          allowBlank : true,
          qdesk : {
            required : true
          }
        },

        {
          xtype : 'textfield',
          itemId : 'proxyUsername',
          name : 'proxyUsername',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldLabelProxyUsername' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldTooltipProxyUsername' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldEmptyTxtProxyUsername' ),
          allowBlank : true,
          qdesk : {
            required : false
          }
        },
        {
          xtype : 'textfield',
          inputType : 'password',
          itemId : 'proxyPassword',
          name : 'proxyPassword',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldLabelProxyPassword' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldTooltipProxyPassword' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.ProxyPanel.fieldEmptyTxtProxyPassword' ),
          allowBlank : true,
          qdesk : {
            required : false
          }
        }
      ]
    } );

    me.callParent( [ config ] );
  },

  /**
   * Updates the allowBlank flags of the proxy host and port form fields based on the
   * specified proxy configuration object.
   * @param proxy a proxy configuration object.
   */
  updateProxyFieldsAllowsBlank : function ( proxy )
  {
    var me = this;

    var proxyIsSet = !Ext.isEmpty( proxy ) && !Ext.isEmpty( proxy.type );

    var proxyHostField = me.down( '#proxyHost' );
    proxyHostField.allowBlank = !proxyIsSet;
    proxyHostField.validate();
    proxyHostField.clearInvalid();

    var proxyPortField = me.down( '#proxyPort' );
    proxyPortField.allowBlank = !proxyIsSet;
    proxyPortField.validate();
    proxyPortField.clearInvalid();
  },


  /**
   * Extracts proxy configuration values from the specified proxy configuration
   * object.
   *
   * @param proxy proxy configuration object.
   * @return the form field values.
   */
  proxyConfig2FieldValues : function ( proxy )
  {
    var fieldValues = {};

    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( fieldValues, {
        proxyType : proxy.type,
        proxyHost : proxy.host,
        proxyPort : proxy.port,
        proxyUsername : proxy.username,
        proxyPassword : proxy.password
      } );
    }

    return fieldValues;
  },


  /**
   * Extracts proxy related data from the specified form field values object
   * and creates a proxy configuration object. If the proxy type is set to
   * an empty value (Proxy type = "None"), then this method returns null.
   *
   * @param fieldValues form field values.
   */
  fieldValues2ProxyConfig : function ( fieldValues )
  {
    var me = this;

    // if proxy set to "None"
    if ( Ext.isEmpty( fieldValues.proxyType ) )
    {
      return null;
    }
    else
    {
      var proxy = {};

      Ext.apply( proxy, {
        type : fieldValues.proxyType,
        host : fieldValues.proxyHost,
        port : fieldValues.proxyPort
      } );

      if ( !Ext.isEmpty( fieldValues.proxyUsername ) )
      {
        Ext.apply( proxy, {
          username : fieldValues.proxyUsername
        } )
      }

      if ( !Ext.isEmpty( fieldValues.proxyPassword ) )
      {
        Ext.apply( proxy, {
          password : fieldValues.proxyPassword
        } )
      }

      return proxy;
    }
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var proxyTypeField = me.down( '#proxyType' );

    /*
     * Before the form is rendered and whenever the proxy type combo value changes, enable/disable
     * and adjust value of allowBlank attributes of the host, port, username and password fields.
     */
    proxyTypeField.on( 'beforerender', me._updateProxyFields, me );
    proxyTypeField.on( 'change', me._updateProxyFields, me );
  },


  /**
   * Enables/disables proxy fields on the current proxy type selection.
   * @private
   */
  _updateProxyFields : function ()
  {
    var me = this;

    var proxyTypeField = me.down( '#proxyType' );
    var proxyHostField = me.down( '#proxyHost' );
    var proxyPortField = me.down( '#proxyPort' );
    var proxyUsernameField = me.down( '#proxyUsername' );
    var proxyPasswordField = me.down( '#proxyPassword' );

    if ( Ext.isEmpty( proxyTypeField.getValue() ) )
    {
      // None proxy type selected
      proxyHostField.setValue( null );
      proxyHostField.setDisabled( true );
      proxyHostField.allowBlank = true;

      proxyPortField.setValue( null );
      proxyPortField.setDisabled( true );
      proxyPortField.allowBlank = true;

      proxyUsernameField.setValue( null );
      proxyUsernameField.setDisabled( true );
      //usernameField.allowBlank = true;  // username is always optional

      proxyPasswordField.setValue( null );
      proxyPasswordField.setDisabled( true );
      //passwordField.allowBlank = true;  // password is always optional
    }
    else
    {
      proxyHostField.setDisabled( false );
      proxyHostField.allowBlank = false;

      proxyPortField.setDisabled( false );
      proxyPortField.allowBlank = false;

      proxyUsernameField.setDisabled( false );
      //usernameField.allowBlank = false;  // username is always optional

      proxyPasswordField.setDisabled( false );
      //passwordField.allowBlank = false;  // password is always optional
    }

    proxyHostField.validate();
    proxyPortField.validate();
    proxyUsernameField.validate();
    proxyPasswordField.validate();
  }
} )
;
