/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.email.EmailProfilePanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.model.security.Permission',
    'QDesk.model.message.channel.MessageChannel'
  ],

  alias : 'widget.EmailProfilePanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'EMAIL',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.work.message.channel.email.EmailProfilePanel.title' )
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            /* Basic settings panel */
            {
              xtype : 'panel',
              border : false,
              title : QDesk.Msg.get( 'QDesk.view.work.message.channel.email.EmailProfilePanel.tabLabelBasic' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%',
                labelWidth : 80
              },
              items : [
                /* Sender address */
                {
                  xtype : 'textfield',
                  itemId : 'sender',
                  name : 'sender',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldLabelSmtpFromAddr' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldTooltipSmtpFromAddr' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldEmptyTxtSmtpFromAddr' ),
                  allowBlank : false,
                  qdesk : {
                    required : true
                  }
                },

                /* SMTP host and port */
                {
                  xtype : 'textfield',
                  itemId : 'smtpHost',
                  name : 'smtpHost',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldLabelSmtpHost' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldTooltipSmtpHost' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldEmptyTxtSmtpHost' ),
                  allowBlank : false,
                  qdesk : {
                    required : true
                  }
                },
                {
                  xtype : 'numberfield',
                  itemId : 'smtpPort',
                  name : 'smtpPort',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldLabelSmtpPort' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldTooltipSmtpPort' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldEmptyTxtSmtpPort' ),
                  value : 25, /* default SMTP port */
                  minValue : 1,
                  maxValue : 65535,
                  anchor : '60%',
                  allowBlank : false,
                  qdesk : {
                    required : true
                  }
                },

                {
                  xtype : 'checkboxfield',
                  itemId : 'smtpTlsEnabled',
                  name : 'smtpTlsEnabled',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldLabelSmtpTlsEnabled' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldTooltipSmtpTlsEnabled' ),
                  value : false
                },

                /* SMTP username & password */
                {
                  xtype : 'textfield',
                  itemId : 'smtpUsername',
                  name : 'smtpUsername',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldLabelSmtpUsername' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldTooltipSmtpUsername' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldEmptyTxtSmtpUsername' ),
                  allowBlank : true,
                  qdesk : {
                    required : false
                  }
                },
                {
                  xtype : 'textfield',
                  inputType : 'password',
                  itemId : 'smtpPassword',
                  name : 'smtpPassword',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldLabelSmtpPassword' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldTooltipSmtpPassword' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.message.channel.email.EmailProfilePanel.fieldEmptyTxtSmtpPassword' ),
                  allowBlank : true,
                  qdesk : {
                    required : false
                  }
                }
              ]
            }
            /* End of Basic setting panel */
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessageEllipsis' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },

  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    me.getForm().setValues( {
      id : profile.id,
      name : profile.name,
      sender : profile.channelConfig.sender,
      smtpHost : profile.channelConfig.smtpHost,
      smtpPort : profile.channelConfig.smtpPort,
      smtpTlsEnabled : profile.channelConfig.smtpTlsEnabled,
      smtpUsername : profile.channelConfig.smtpUsername,
      smtpPassword : profile.channelConfig.smtpPassword
    } );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.model.message.channel.MessageChannel.EMAIL );

    var channelConfig = {};

    if ( !Ext.isEmpty( fieldValues.sender ) )
    {
      Ext.apply( channelConfig, {
        sender : fieldValues.sender
      } );
    }

    if ( !Ext.isEmpty( fieldValues.smtpHost ) )
    {
      Ext.apply( channelConfig, {
        smtpHost : fieldValues.smtpHost
      } );
    }

    if ( !Ext.isEmpty( fieldValues.smtpPort ) )
    {
      Ext.apply( channelConfig, {
        smtpPort : fieldValues.smtpPort
      } );
    }

    Ext.apply( channelConfig, {
      smtpTlsEnabled : fieldValues.smtpTlsEnabled
    } );

    if ( !Ext.isEmpty( fieldValues.smtpUsername ) )
    {
      Ext.apply( channelConfig, {
        smtpUsername : fieldValues.smtpUsername
      } );
    }

    if ( !Ext.isEmpty( fieldValues.smtpPassword ) )
    {
      Ext.apply( channelConfig, {
        smtpPassword : fieldValues.smtpPassword
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
