/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.model.security.Permission',
    'QDesk.view.work.message.channel.ProxyPanel',
    'QDesk.view.work.message.channel.im.gtalk.GTalkAdvancedPanel',
    'QDesk.view.work.message.channel.im.gtalk.GTalkBasicPanel'
  ],

  alias : 'widget.ImGTalkProfilePanel',

  defaultButton : 'saveButton',

  layout : {
    type : 'vbox',
    align : 'stretch',
    pack : 'start'
  },

  bodyPadding : 0,

  border : false,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },

        {
          xtype : 'panel',
          border : false,
          layout : 'anchor',
          defaults : {
            anchor : '100%',
            labelWidth : 50
          },
          bodyPadding : '3px 10px 5px 10px',
          items : [
            {
              xtype : 'panel',
              bodyCls : 'IM_GTALK',
              border : false,
              width : 32,
              height : 32,
              html : QDesk.Msg.get( 'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel.title' )
            },

            {
              xtype : 'displayfield',
              value : QDesk.Msg.get( 'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel.disclaimerMsg' ),
              fieldCls : 'disclaimerMessageChannel'
            },

            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel.fieldLabelName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.message.channel.im.gtalk.GTalkProfilePanel.fieldEmptyTxtName' ),
              value : QDesk.Msg.get( 'QDesk.unnamed' ),
              allowBlank : false,
              qdesk : {
                required : true
              }
            }
          ]
        },

        {
          xtype : 'tabpanel',
          border : false,
          bodyPadding : 10,
          items : [
            {
              xtype : 'ImGTalkBasicPanel'
            },
            {
              xtype : 'ImGTalkAdvancedPanel'
            },
            {
              xtype : 'ProxyPanel'
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'sendTestMessageButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSendTestMessageEllipsis' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        },
        {
          itemId : 'saveButton',
          reference : 'saveButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelSave' ),
          formBind : true,
          qdRequiresAnyPermissions : [
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_ADD,
            QDesk.model.security.Permission.MSG_CHANNEL_PROFILE_UPDATE
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  /**
   * Populates this form panel from the specified profile object.
   * @param profile a profile object.
   */
  populate : function ( profile )
  {
    var me = this;

    var values = {};

    var usernameParts;
    if ( profile.channelConfig.username )
    {
      usernameParts = profile.channelConfig.username.split( '@', 2 );
    }

    Ext.apply( values, {
      id : profile.id,
      name : profile.name,
      password : profile.channelConfig.password
    } );

    if ( usernameParts && usernameParts.length >= 1 && usernameParts[ 0 ] )
    {
      Ext.apply( values, {
        username : usernameParts[ 0 ]
      } );
    }

    if ( usernameParts && usernameParts.length >= 2 && usernameParts[ 1 ] )
    {
      Ext.apply( values, {
        usernameDomain : usernameParts[ 1 ]
      } );
    }

    if ( profile.channelConfig.serviceName )
    {
      Ext.apply( values, {
        serviceName : profile.channelConfig.serviceName
      } );
    }

    if ( profile.channelConfig.host )
    {
      Ext.apply( values, {
        host : profile.channelConfig.host
      } );
    }

    if ( profile.channelConfig.port )
    {
      Ext.apply( values, {
        port : profile.channelConfig.port
      } );
    }

    var proxyPanel = me.down( 'ProxyPanel' );
    Ext.apply( values, proxyPanel.proxyConfig2FieldValues( profile.channelConfig.proxy ) );

    // based on the selected proxy type make the proxy host & port form fields required
    proxyPanel.updateProxyFieldsAllowsBlank( profile.channelConfig.proxy );

    me.getForm().setValues( values );
  },


  /**
   * Creates a new {@link MessageChannelProfile} model instance populated with
   * form data.
   */
  createMessageChannelProfile : function ()
  {
    var me = this;

    var profile = Ext.create( 'QDesk.model.message.MessageChannelProfile' );

    var fieldValues = me.getForm().getFieldValues();

    if ( !Ext.isEmpty( fieldValues.id ) )
      profile.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.name ) )
      profile.set( 'name', fieldValues.name );

    profile.set( 'channel', QDesk.model.message.channel.MessageChannel.IM_GTALK );

    var channelConfig = {};

    Ext.apply( channelConfig, {
      username : fieldValues.username + '@' + fieldValues.usernameDomain,
      password : fieldValues.password,
      serviceName : fieldValues.serviceName,
      host : fieldValues.host,
      port : fieldValues.port
    } );

    var proxyPanel = me.down( 'ProxyPanel' );
    var proxy = proxyPanel.fieldValues2ProxyConfig( fieldValues );
    if ( !Ext.isEmpty( proxy ) )
    {
      Ext.apply( channelConfig, {
        proxy : proxy
      } );
    }

    profile.set( 'channelConfig', channelConfig );

    return profile;
  }
} );
