/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Base class for all scheduler views.
 */
Ext.define( 'QDesk.view.work.scheduler.BaseSchedulerView', {
  extend : 'QDesk.view.widget.tab.XTabPanel',

  closable : 'true',

  qdesk : {
    /*
     * Required config parameter with the ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Required config parameter with the node path of the scheduler connection.
     */
    connectionNodePath : undefined,

    /*
     * Required config parameter with the QDesk.SchedulerInfoAccessor instance.
     */
    schedulerInfo : undefined,

    /*
     * Required config parameter with the QDesk.JvmAgentInfoAccessor instance.
     */
    jvmAgentInfo : undefined
  },


  /**
   * Returns the connection ID this tab has been created for.
   * @return the connection ID.
   */
  getConnectionId : function ()
  {
    return this.qdesk.connectionId;
  },


  /**
   * Returns the node path of the connection this scheduler view has been opened for.
   * @returns the connection node path.
   */
  getConnectionNodePath : function ()
  {
    return this.qdesk.connectionNodePath;
  },


  /**
   * Sets the node path of the connection this scheduler view has been created for.
   * @param connectionNodePath a connection node path.
   */
  setConnectionNodePath : function ( connectionNodePath )
  {
    this.qdesk.connectionNodePath = connectionNodePath;
  },


  /**
   * Returns the QDesk.SchedulerInfoAccessor instance providing access to the scheduler information.
   * @return the QDesk.SchedulerInfoAccessor instance.
   */
  getSchedulerInfo : function ()
  {
    return this.qdesk.schedulerInfo;
  },


  /**
   * Returns the QDesk.JvmAgentInfoAccessor instance providing access to information on the installed
   * QuartzDesk JVM Agent.
   * @return the QDesk.JvmAgentInfoAccessor instance.
   */
  getJvmAgentInfo : function ()
  {
    return this.qdesk.jvmAgentInfo;
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.connectionNodePath )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionNodePath' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    me.callParent( [ config ] );
  }
} );
