/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.info.JvmAgentInfoView', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.common.info.JvmAgentInfoForm'
  ],

  alias : 'widget.JvmAgentInfoView',

  border : false,

  iconCls : 'jvmAgentInfoViewIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.common.info.JvmAgentInfoView.tooltip' )
    } );

    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;
    if ( licensedProductEdition == QDesk.model.license.ProductEdition.LITE )
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }
    else
    {
      var items = [];

      items.push(
          {
            xtype : 'JvmAgentInfoForm',
            itemId : 'jvmAgentInfoForm',
            flex : 1
          }
      );

      /*
       * Intentionally over-nested a bit so that we use identical layout to other tabs that
       * have a license restrictions related message panel above the grid.
       */
      Ext.apply( config, {
        items : [
          {
            xtype : 'panel',
            border : false,
            layout : 'fit',
            flex : 1,
            items : [
              {
                xtype : 'panel',
                border : false,
                layout : {
                  type : 'vbox',
                  align : 'stretch'
                },
                items : items
              }
            ]
          }
        ]
      } );
    }

    me.callParent( [ config ] );
  },


  _getTitle : function ()
  {
    var licensedProductEdition = QDesk.util.license.LicensedProduct.productEdition;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.common.info.JvmAgentInfoView.title' );
    if ( licensedProductEdition == QDesk.model.license.ProductEdition.LITE )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );