/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleForm',
    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode',
    'QDesk.Msg'
  ],

  alias : 'widget.EditExecNotificationRuleWindow',

  cls : 'editExecNotifRuleWindow',
  iconCls : 'windowIcon',

  modal : true,
  resizable : false,

  minWidth : 600,
  width : 600,

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditExecNotificationRuleFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.model.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      items : [
        {
          xtype : 'EditExecNotificationRuleForm',
          itemId : 'form',

          qdesk : {
            connectionId : config.qdesk.connectionId,
            mode : config.qdesk.mode,
            schedulerType : config.qdesk.schedulerType
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode value.
   * @return {string} the title.
   * @private
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.ADD:
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleWindow.windowTitleAdd' );

      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.EDIT:
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleWindow.windowTitleEdit' );

      case QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.CLONE:
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleWindow.windowTitleClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
