/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.SchedulerView', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerView',

  requires : [
    'QDesk.model.security.Permission',

    'QDesk.view.work.scheduler.quartz.audit.SchedulerAuditTrailTab',
    'QDesk.view.work.scheduler.quartz.chain.SchedulerJobChainsTab',
    'QDesk.view.work.scheduler.quartz.info.InfoTab',
    'QDesk.view.work.scheduler.quartz.jobs.ExecutingJobsTab',
    'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsTab',
    'QDesk.view.work.scheduler.quartz.jobs.JobsTab',
    'QDesk.view.work.scheduler.quartz.misfire.SchedulerMisfiredTriggersTab',
    'QDesk.view.work.scheduler.quartz.triggers.TriggersTab',
    'QDesk.view.work.scheduler.quartz.history.SchedulerExecHistoryTab',
    'QDesk.view.work.scheduler.quartz.notif.exec.SchedulerExecNotificationsTab',
    'QDesk.view.work.scheduler.quartz.stats.SchedulerStatisticsTab'
  ],

  alias : 'widget.QuartzSchedulerView',

  iconCls : 'quartzSchedulerViewIcon',

  bodyStyle : {
    borderTop : 0
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.connectionNodePath )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionNodePath' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    Ext.apply( config, {

      items : [
        {
          xtype : 'QuartzJobsTab',
          itemId : 'jobsTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzExecutingJobsTab',
          itemId : 'executingJobsTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzPlannedJobExecutionsTab',
          itemId : 'plannedJobExecutionsTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzTriggersTab',
          itemId : 'triggersTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzMisfiredTriggersTab',
          itemId : 'schedulerMisfiredTriggersTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzSchedulerExecHistoryTab',
          itemId : 'schedulerExecHistoryTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzSchedulerStatisticsTab',
          itemId : 'schedulerStatisticsTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzSchedulerExecNotificationsTab',
          itemId : 'schedulerNotificationsTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzSchedulerJobChainsTab',
          itemId : 'schedulerJobChainsTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzSchedulerAuditTrailTab',
          itemId : 'schedulerAuditTrailTab',
          qdesk : config.qdesk
        },

        {
          xtype : 'QuartzInfoTab',
          itemId : 'infoTab',
          qdesk : config.qdesk
        }
      ],

      /*
       * Due to a bug in ExtJS it is not possible to move the dockedItems config option
       * to the class definition as it seems to mess up the sub-tabs.
       */
      dockedItems : [
        {
          xtype : 'toolbar',
          itemId : 'schedulerViewToolBar',
          dock : 'bottom',
          items : [
            {
              xtype : 'button',
              itemId : 'refreshButton',
              scale : 'small',
              iconCls : 'refresh',
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SchedulerView.tooltipRefreshButton' ),
              disabled : true,
              qdRequiresAnyPermissions : [],
              qdNotAuthorizedIconCls : 'refreshNAUTH'
            },
            '-',
            {
              xtype : 'button',
              itemId : 'startSchedulerButton',
              scale : 'small',
              iconCls : 'startScheduler',
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SchedulerView.tooltipStartSchedulerButton' ),
              disabled : true,
              qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_START ],
              qdNotAuthorizedIconCls : 'startSchedulerNAUTH'
            },
            {
              xtype : 'button',
              itemId : 'pauseSchedulerButton',
              scale : 'small',
              iconCls : 'pauseScheduler',
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SchedulerView.tooltipPauseSchedulerButton' ),
              disabled : true,
              qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_PAUSE ],
              qdNotAuthorizedIconCls : 'pauseSchedulerNAUTH'
            },
            {
              xtype : 'button',
              itemId : 'stopSchedulerButton',
              scale : 'small',
              iconCls : 'stopScheduler',
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SchedulerView.tooltipStopSchedulerButton' ),
              disabled : true,
              qdRequiresAnyPermissions : [ QDesk.model.security.Permission.SCHEDULER_STOP ],
              qdNotAuthorizedIconCls : 'stopSchedulerNAUTH'
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );
