/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.ViewNextFireTimesWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.quartz.ViewNextFireTimesGrid'
  ],

  alias : 'widget.QuartzViewNextFireTimesWindow',

  cls : 'quartzViewNextFireTimesWindow',
  iconCls : 'windowIcon',

  scrollable : true,

  modal : true,
  resizable : true,
  maximizable : true,

  width : 500,
  height : 500,

  minWidth : 300,
  minHeight : 300,

  layout : 'fit',

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Optional job name.
     */
    jobName : undefined,

    /*
     * Optional trigger name.
     */
    triggerName : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var title = null;

    if ( config.qdesk.jobName )
    {
      title =
          QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.ViewNextFireTimesWindow.titleWithJobName',
              config.qdesk.jobName );
    }
    else if ( config.qdesk.triggerName )
    {
      title =
          QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.ViewNextFireTimesWindow.titleWithTriggerName',
              config.qdesk.triggerName );
    }
    else
    {
      title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.ViewNextFireTimesWindow.titleNoName' );
    }

    Ext.apply( config, {
      title : title,
      items : [
        {
          xtype : 'QuartzViewNextFireTimesGrid',
          itemId : 'quartzViewNextFireTimesGrid',
          qdesk : config.qdesk
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'closeButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelClose' )
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me.down( '#closeButton' ).on( 'click', function ()
    {
      me.close();
    } );
  }
} );