/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.common.ExpressionLanguage',
    'QDesk.model.common.ExpressionType',
    'QDesk.model.common.Weekday',

    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.scheduler.JobChainConditionType',

    'QDesk.model.security.Permission',

    'QDesk.store.enums.ExecStatusStore',
    'QDesk.store.enums.ExpressionLanguageStore',
    'QDesk.store.enums.scheduler.JobChainConditionTypeStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel',

    'QDesk.view.work.scheduler.quartz.chain.JobChainTargetsGrid',

    'QDesk.view.work.scheduler.common.chain.EditJobChainTargetFormMode',
    'QDesk.view.work.scheduler.quartz.chain.EditJobChainTargetWindow',

    'QDesk.view.widget.expression.WExpressionEditor'
  ],

  alias : 'widget.QuartzEditJobChainForm',

  defaultButton : 'actionButton',

  fieldDefaults : {
    labelWidth : 80
  },

  qdesk : {
    /*
     * Required ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditJobChainFormMode).
     */
    mode : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    var enabledOnWeekdaysPanel =
        {
          xtype : 'panel',
          border : false,
          height : 50,
          layout : {
            type : 'hbox',
            pack : 'start',
            align : 'stretch'
          },
          items : [
            {
              xtype : 'checkboxgroup',
              style : {
                paddingLeft : '10px',
                backgroundColor : '#E6F7E6'
              },
              flex : 2,

              vertical : true,
              columns : 3,
              items : [
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.MONDAY ),
                  name : 'enabledOnMonday',
                  itemId : 'enabledOnMonday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.MONDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.TUESDAY ),
                  name : 'enabledOnTuesday',
                  itemId : 'enabledOnTuesday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.TUESDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.WEDNESDAY ),
                  name : 'enabledOnWednesday',
                  itemId : 'enabledOnWednesday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.WEDNESDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.THURSDAY ),
                  name : 'enabledOnThursday',
                  itemId : 'enabledOnThursday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.THURSDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.FRIDAY ),
                  name : 'enabledOnFriday',
                  itemId : 'enabledOnFriday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.FRIDAY
                  }
                }
              ]
            },
            {
              xtype : 'checkboxgroup',
              style : {
                paddingLeft : '10px',
                backgroundColor : '#F7E6E6'
              },
              flex : 1,
              vertical : true,
              columns : 1,
              items : [
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.SATURDAY ),
                  name : 'enabledOnSaturday',
                  itemId : 'enabledOnSaturday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.SATURDAY
                  }
                },
                {
                  boxLabel : QDesk.util.EnumUtils.getWeekdayName( QDesk.model.common.Weekday.SUNDAY ),
                  name : 'enabledOnSunday',
                  itemId : 'enabledOnSunday',
                  qdesk : {
                    value : QDesk.model.common.Weekday.SUNDAY
                  }
                }
              ]
            }
          ]
        };


    var enabledFromToPanel =
        {
          xtype : 'panel',
          border : false,
          layout : 'column',
          style : {
            marginBottom : '5px'
          },
          items : [
            {
              xtype : 'timefield',
              itemId : 'enabledFromTime',
              name : 'enabledFromTime',
              labelWidth : 95,
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelEnabledFromTime' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipEnabledFromTime' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtEnabledFromTime' ),
              formatText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldFormatTxtEnabledFromTime' ),
              format : QDesk.util.constant.DateFormat.dayTimeFormat,
              increment : 30
            },
            {
              xtype : 'timefield',
              itemId : 'enabledToTime',
              name : 'enabledToTime',
              labelWidth : 82,
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelEnabledToTime' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipEnabledToTime' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtEnabledToTime' ),
              formatText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldFormatTxtEnabledToTime' ),
              format : QDesk.util.constant.DateFormat.dayTimeFormat,
              increment : 30,
              margin : '0 0 0 15'
            }
          ]
        };

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          itemId : 'jobChainId',
          name : 'id'
        },

        {
          xtype : 'hiddenfield',
          itemId : 'schedulerObjectName',
          name : 'schedulerObjectName'
        },

        /* Start of Quartz specific fields */
        {
          xtype : 'hiddenfield',
          itemId : 'jobGroupName',
          name : 'jobGroupName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'jobName',
          name : 'jobName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'triggerGroupName',
          name : 'triggerGroupName'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'triggerName',
          name : 'triggerName'
        },
        /* End of Quartz specific fields */

        {
          xtype : 'hiddenfield',
          itemId : 'createdAt',
          name : 'createdAt'
        },
        {
          xtype : 'hiddenfield',
          itemId : 'updatedAt',
          name : 'updatedAt'
        },

        {
          xtype : 'textfield',
          itemId : 'name',
          name : 'name',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelName' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipName' ),
          emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtName' ),
          value : QDesk.Msg.get( 'QDesk.unnamed' ),
          allowBlank : false
        },
        {
          xtype : 'textareafield',
          itemId : 'description',
          name : 'description',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelDescription' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipDescription' ),
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtDescription' ),
          labelAlign : 'top',
          allowBlank : true
        },

        {
          xtype : 'checkboxfield',
          itemId : 'enabled',
          name : 'enabled',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelEnabled' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipEnabled' ),
          allowBlank : true
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldsetLabelEnabledOnWeekdays' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldsetTooltipEnabledOnWeekdays' ),
          itemId : 'enabledOnWeekdaysFieldSet',
          defaults : {
            labelWidth : 100,
            anchor : '100%'
          },
          padding : '0 10px 7px 10px',
          items : [
            enabledOnWeekdaysPanel
          ]
        },

        enabledFromToPanel,

        {
          xtype : 'tabpanel',
          items : [
            {
              xtype : 'panel',
              padding : '10 10 5 10',
              border : false,
              iconCls : 'quartzJobChainConditionTabIcon',
              title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.tabLabelCondition' ),
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.tabTooltipCondition' ),
              layout : 'anchor',
              defaults : {
                labelWidth : 120,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'combo',
                  itemId : 'conditionType',
                  name : 'conditionType',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelConditionType' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipConditionType' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.scheduler.JobChainConditionTypeStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },

                /* Condition exec status */
                {
                  xtype : 'combo',
                  itemId : 'conditionExecStatus',
                  name : 'conditionExecStatus',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelConditionExecStatus' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipConditionExecStatus' ),
                  allowBlank : false,
                  editable : false,
                  forceSelection : true,
                  queryMode : 'local',
                  store : 'enums.ExecStatusStore',
                  valueField : 'value',
                  displayField : 'valueLabel'
                },

                /* Condition max duration */
                {
                  xtype : 'numberfield',
                  itemId : 'conditionMaxDuration',
                  name : 'conditionMaxDuration',
                  fieldLabel : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldLabelMaxDuration' ),
                  fieldTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipMaxDuration' ),
                  emptyText : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtMaxDuration' ),
                  allowBlank : false,
                  minValue : 1
                },

                /* Condition expression */
                {
                  xtype : 'expression.WExpressionEditor',
                  itemId : 'conditionExpression',
                  name : 'conditionExpression',
                  allowBlank : false,
                  value : {
                    language : QDesk.model.common.ExpressionLanguage.JAVASCRIPT,
                    text : null
                  },
                  expressionType : QDesk.model.common.ExpressionType.QUARTZ_JOB_CHAIN_CONDITION,
                  textEmptyText : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldEmptyTxtConditionExpressionText' ),
                  textTooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.fieldTooltipConditionExpressionText' ),
                  objectsStore : me._createObjectsTreeStore(),
                  dataTypesStore : me._createDataTypesTreeStore(),
                  extraValidateParamFn : function ()
                  {
                    return {
                      connectionId : me.qdesk.connectionId
                    };
                  }
                }
              ]
            },

            {
              xtype : 'panel',
              itemId : 'targetsTab',
              border : false,
              iconCls : 'quartzJobChainTargetTabIcon',
              title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.tabLabelTargets' ),
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.chain.EditJobChainForm.tabTooltipTargets' ),
              layout : 'anchor',
              defaults : {
                labelWidth : 105,
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'QuartzJobChainTargetsGrid',
                  border : false,
                  itemId : 'jobChainTargetsGrid',
                  height : 150,
                  qdesk : {
                    connectionId : config.qdesk.connectionId
                  }
                }
              ]
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true,
          qdRequiresAnyPermissions : me._getRequiredPermissions( config.qdesk.mode )
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    /* Defaults */
    me.down( '#enabled' ).setValue( true );
    me.down( '#enabledOnMonday' ).setValue( true );
    me.down( '#enabledOnTuesday' ).setValue( true );
    me.down( '#enabledOnWednesday' ).setValue( true );
    me.down( '#enabledOnThursday' ).setValue( true );
    me.down( '#enabledOnFriday' ).setValue( true );
    me.down( '#enabledOnSaturday' ).setValue( true );
    me.down( '#enabledOnSunday' ).setValue( true );

    me.down( '#conditionType' ).setValue( QDesk.model.scheduler.JobChainConditionType.EXEC_STATUS );
    me.down( '#conditionExecStatus' ).setValue( QDesk.model.scheduler.ExecStatus.SUCCESS );
  },


  /**
   * Invoked from the load-success handler to set special form field values (e.g. the enabledOnWeekdays
   * checkboxes etc.).
   * @param jsonData JSON data.
   */
  loadSpecialFields : function ( jsonData )
  {
    var me = this;

    // set enabledOnWeekdays checkboxes
    Ext.each( Ext.ComponentQuery.query( '#enabledOnWeekdaysFieldSet checkbox', me ),
        function ( checkbox )
        {
          checkbox.setValue( Ext.Array.contains( jsonData.enabledOnWeekday, checkbox.qdesk.value ) );
        } );

    // set enabledFromTime
    if ( jsonData.enabledFromTime )
    {
      var enabledFromDate = QDesk.util.FormatUtils.parseDayTime( jsonData.enabledFromTime );
      me.down( '#enabledFromTime' ).setValue( enabledFromDate );
    }

    // set enabledToTime fields
    if ( jsonData.enabledToTime )
    {
      var enabledToDate = QDesk.util.FormatUtils.parseDayTime( jsonData.enabledToTime );
      me.down( '#enabledToTime' ).setValue( enabledToDate );
    }

    if ( !Ext.isEmpty( jsonData.target ) )
    {
      var jobChainTargetsStore = me.down( '#jobChainTargetsGrid' ).getStore();
      jobChainTargetsStore.loadData( jsonData.target );
    }
  },


  /**
   * Invoked to obtain the array of QDesk.model.scheduler.quartz.QuartzJobChainTarget instances representing
   * current job chain targets.
   */
  getJobChainTargetsData : function ()
  {
    var me = this;

    var result = [];

    Ext.each( me.down( '#jobChainTargetsGrid' ).getStore().getRange(), function ( jobChainTarget )
    {
      result.push( jobChainTarget.getData() );
    } );

    return result;
  },


  /**
   * Binds event handlers to all form widgets.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    // job chain condition type change
    me.down( '#conditionType' ).on( 'change', function ( combo, newValue, oldValue )
    {
      Ext.each( me.down( '#conditionExecStatus' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.JobChainConditionType.EXEC_STATUS );
        field.setDisabled( newValue != QDesk.model.scheduler.JobChainConditionType.EXEC_STATUS );
        field.validate();
      } );

      Ext.each( me.down( '#conditionMaxDuration' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.JobChainConditionType.MAX_DURATION );
        field.setDisabled( newValue != QDesk.model.scheduler.JobChainConditionType.MAX_DURATION );
        field.validate();
      } );

      Ext.each( me.down( '#conditionExpression' ), function ( field )
      {
        field.setVisible( newValue == QDesk.model.scheduler.JobChainConditionType.EXPRESSION );
        field.setDisabled( newValue != QDesk.model.scheduler.JobChainConditionType.EXPRESSION );
        field.validate();
      } );
    } );
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.work.scheduler.common.chain.EditJobChainFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.scheduler.common.chain.EditJobChainFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.work.scheduler.common.chain.EditJobChainFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  _getRequiredPermissions : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.work.scheduler.common.chain.EditJobChainFormMode.ADD:
        return [ QDesk.model.security.Permission.QUARTZ_JOB_CHAIN_ADD ];

      case QDesk.view.work.scheduler.common.chain.EditJobChainFormMode.EDIT:
        return [ QDesk.model.security.Permission.QUARTZ_JOB_CHAIN_UPDATE ];

      case QDesk.view.work.scheduler.common.chain.EditJobChainFormMode.CLONE:
        return [ QDesk.model.security.Permission.QUARTZ_JOB_CHAIN_ADD ];

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  _createObjectsTreeStore : function ()
  {
    return Ext.StoreManager.lookup( 'scheduler.quartz.QuartzJSScriptingObjectsTreeStore' );
  },


  _createDataTypesTreeStore : function ()
  {
    return Ext.StoreManager.lookup( 'scheduler.quartz.QuartzJSScriptingDataTypesTreeStore' );
  }
} );
