/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.store.enums.scheduler.quartz.QuartzExecHistoryExportColumnStore',

    'QDesk.view.base.BaseFormPanel',
    // 'QDesk.view.widget.message.InfoMessagePanel',

    'QDesk.util.constant.DateFormat',

    'Ext.ux.form.ItemSelector'
  ],

  alias : 'widget.QuartzExportExecHistoryForm',

  defaultButton : 'exportButton',

  fieldDefaults : {
    labelWidth : 90
  },

  /* QDesk specific data */
  qdesk : {
    /**
     * Required config parameter with the ID of the Quartz scheduler connection.
     */
    connectionId : undefined,

    /**
     * Optional config parameter containing the job detail whose exec history should be exported and saved.
     */
    jobDetail : undefined,

    /**
     * Optional config parameter containing the trigger whose exec history should be exported and saved.
     */
    trigger : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    //var exportColumnsStore = Ext.StoreManager.lookup( 'enums.scheduler.quartz.QuartzExecHistoryExportColumnStore' );

    Ext.apply( config, {
      items : [
        // {
        //   xtype : 'InfoMessagePanel',
        //   margin : '0 0 5 0',
        //   qdesk : {
        //     msg : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.description' )
        //   }
        // },
        {
          xtype : 'itemselectorfield',
          itemId : 'selectedColumns',
          name : 'selectedColumns',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldLabelSelectedColumns' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldTooltipSelectedColumns' ),
          fromTitle : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldTitleSelectedColumnsFrom' ),
          toTitle : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldTitleSelectedColumnsTo' ),
          labelAlign : 'top',
          allowBlank : false,
          displayField : 'valueLabel',
          valueField : 'value',
          //minSelections : 1,
          buttons : [ 'add', 'remove' ],
          store : 'enums.scheduler.quartz.QuartzExecHistoryExportColumnStore',
          height : 400
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldsetLabelTimePeriod' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldsetTooltipTimePeriod' ),
          items : [
            {
              xtype : 'WDateTimePeriod',
              itemId : 'exportPeriod',
              allowBlank : false
            }
          ]
        },

        {
          xtype : 'WClearableField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.history.ExportExecHistoryForm.emptyTextFieldFilterQueryText' ),
          labelWidth : 50,
          disabled : false
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'exportButton',
          reference : 'exportButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelExport' ),
          formBind : true
        }
      ]
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var now = new Date();

    var todayMidnight = new Date();
    todayMidnight.setTime( now.getTime() );
    todayMidnight.setHours( 0, 0, 0, 0 );

    var tomorrowMidnight = new Date();
    tomorrowMidnight.setTime( now.getTime() );
    tomorrowMidnight.setDate( tomorrowMidnight.getDate() + 1 );
    tomorrowMidnight.setHours( 0, 0, 0, 0 );

    var exportPeriod = me.down( '#exportPeriod' );
    exportPeriod.setValue( {
      from : todayMidnight,
      to : tomorrowMidnight
    } );
  }
} );
