/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.info.InfoTab', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.view.work.scheduler.quartz.info.ConnectionInfoView',
    'QDesk.view.work.scheduler.common.info.JvmAgentInfoView',
    'QDesk.view.work.scheduler.quartz.info.SchedulerInfoView',

    'QDesk.view.widget.tab.XTabPanel'
  ],

  alias : 'widget.QuartzInfoTab',

  bodyCls : 'quartzInfoSubTabBody',
  iconCls : 'quartzInfoSubTabIcon',

  // border : false,
  /* The top Info tab should not be scrollable, but sub-tabs should be! */
  scrollable : false,

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var items = [];

    items.push(
        {
          xtype : 'QuartzConnectionInfoView',
          itemId : 'connectionInfoView',
          qdesk : config.qdesk
        } );

    items.push(
        {
          xtype : 'QuartzSchedulerInfoView',
          itemId : 'schedulerInfoView',
          qdesk : config.qdesk
        } );

    items.push(
        {
          xtype : 'JvmAgentInfoView',
          itemId : 'jvmAgentInfoView',
          qdesk : config.qdesk
        } );


    Ext.apply( config, {
      items : [
        {
          xtype : 'XTabPanel',
          border : false,
          flex : 1,  // for the parent vbox/stretch layout
          items : items
        }
      ]
    } );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.info.InfoTab.title' ),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.info.InfoTab.tooltip' )
    } );

    me.callParent( [ config ] );
  }
} );
