/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.JvmAgentInfoAccessor',
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.store.scheduler.quartz.QuartzJobDetailStore',
    'QDesk.store.scheduler.quartz.QuartzJobGroupStore',

    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.widget.WExecHealthIndicator',
    'QDesk.view.widget.WSearchField',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzJobsGrid',

  cls : 'quartzJobsGrid',

  viewConfig : {
    stripeRows : true
  },

  border : false,
  scrollable : true,

  listeners : {
    itemcontextmenu : function ( grid, record, item, index, event )
    {
      var me = this;
      event.stopEvent();
      var rowCtxMenu = me._createRowCtxMenu( record );
      rowCtxMenu.showAt( event.getXY() );
    }
  },


  plugins : [
    {
      // TODO: expanding/collapsing all grid rows does not work reliably - produces "Uncaught TypeError: Cannot read property 'hasCls' of null"
      /*
       ptype : 'EnhancedRowExpander',
       pluginId: 'rowExpander',
       */
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
        '<tpl if="description">{[Ext.String.htmlEncode(values.description)]}</tpl>' +
        '<tpl if="!description">{[Ext.String.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
        '</p>' +
        '<p style="white-space: nowrap">' +
        '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
        '{[this.getMonitoringLink( values.monitoringUrl )]}' +
        '</p>',
        {
          getMonitoringLink : function ( monitoringUrl )
          {
            var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
            var tooltip;
            if ( licensedFeatures.PollMonitoring )
            {
              tooltip =
                  Ext.String.htmlEncode(
                      QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipMonitoringUrl' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
            }
            else
            {
              tooltip =
                  Ext.String.htmlEncode( QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipMonitoringUrlNotLicensed' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                  tooltip + '">' + monitoringUrl + '</a>';
            }
          }
        }
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobDetailStore' );

    store.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var schedulerVersion = config.qdesk.schedulerInfo.getSchedulerType();
    var jvmAgentReady = config.qdesk.jvmAgentInfo.isReady();
    var publicApiReady = config.qdesk.schedulerInfo.isPublicApiReady();

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'triggerJobButton',
          scale : 'small',
          iconCls : 'quartzTriggerJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipTriggerJobButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_TRIGGER_MANUALLY ],
          qdNotAuthorizedIconCls : 'quartzTriggerJobNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'addJobButton',
          scale : 'small',
          iconCls : 'addJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipAddJobButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_ADD ],
          qdNotAuthorizedIconCls : 'addJobNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'deleteJobButton',
          scale : 'small',
          iconCls : 'deleteJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipDeleteJobButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_DELETE ],
          qdNotAuthorizedIconCls : 'deleteJobNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'editJobButton',
          scale : 'small',
          iconCls : 'editJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipEditJobButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_UPDATE ],
          qdNotAuthorizedIconCls : 'editJobNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'cloneJobButton',
          scale : 'small',
          iconCls : 'cloneJob',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipCloneJobButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_ADD ],
          qdNotAuthorizedIconCls : 'cloneJobNAUTH'
        },
        '-',
        {
          xtype : 'xcombo',
          itemId : 'filterByJobGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.fieldLabelFilterByJobGroupCombo' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.emptyTextFilterByJobGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true,
          tpl : Ext.create( 'Ext.XTemplate',
              '<ul class="x-list-plain"><tpl for=".">',
              '<li role="option" class="x-boundlist-item">{name:htmlEncode}</li>',
              '</tpl></ul>' )
        },
        {
          xtype : 'button',
          itemId : 'resumeJobGroupButton',
          scale : 'small',
          iconCls : 'quartzResumeJobGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipResumeJobGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_GROUP_RESUME ],
          qdNotAuthorizedIconCls : 'quartzResumeJobGroupNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'pauseJobGroupButton',
          scale : 'small',
          iconCls : 'quartzPauseJobGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipPauseJobGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_GROUP_PAUSE ],
          qdNotAuthorizedIconCls : 'quartzPauseJobGroupNAUTH'
        },
        '-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.emptyTextFieldFilterQueryText' ),
          store : store,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },

        // TODO: expanding/collapsing all grid rows does not work reliably - produces "Uncaught TypeError: Cannot read property 'hasCls' of null"
        /*
         '-',
         {
         xtype : 'button',
         itemId : 'expandAllButton',
         scale : 'small',
         iconCls : 'expandAll',
         tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipExpandAllButton' ),
         disabled : true
         },
         {
         xtype : 'button',
         itemId : 'collapseAllButton',
         scale : 'small',
         iconCls : 'collapseAll',
         tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipCollapseAllButton' ),
         disabled : true
         },
         */

        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ]
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'groupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textGroupNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipGroupNameColumn' ),
          dataIndex : 'group.name',
          sortable : true,
          groupable : true,
          flex : 0.5,
          renderer : function ( value, metaData, record )
          {
            var jobGroup = record.get( 'group' );
            return Ext.String.htmlEncode( jobGroup.name );
          }
        },
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textJobClassColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipJobClassColumn' ),
          dataIndex : 'jobClass',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textJobDataMapColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var jobDetailRecord = grid.getStore().getAt( rowIndex );

                var jobDataMap = jobDetailRecord.get( 'jobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap );
              }
            }
          ]
        } );

    if ( licensedFeatures.ExecutionHistory )
    {
      if ( jvmAgentReady )
      {
        var maxExecHealthIndicatorEntries = config.qdesk.jvmAgentInfo.getConfigProperty(
            'quartz.execHealthIndicators.jobs.maxEntriesPerIndicator' );

        if ( publicApiReady )
        {
          columns.push(
              {
                xtype : 'widgetcolumn',
                itemId : 'execHealthIndicatorColumn',
                text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textExecHealthIndicatorColumn' ),
                tooltip : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipExecHealthIndicatorColumn',
                    maxExecHealthIndicatorEntries ),
                dataIndex : 'execHealthIndicatorValues',
                sortable : false,
                groupable : false,
                width : 115,
                widget : {
                  xtype : 'WExecHealthIndicator'
                }
              }
          );
        }
        else
        {
          // agent is ready, but public API is not ready
          columns.push(
              {
                itemId : 'execHealthIndicatorColumn',
                text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textExecHealthIndicatorColumn' ),
                tooltip : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipExecHealthIndicatorColumn',
                    maxExecHealthIndicatorEntries ),
                sortable : false,
                groupable : false,
                width : 113,
                renderer : function ()
                {
                  var tooltip = QDesk.controller.command.connection.ConnectionUtils.getPublicApiWarningTooltip(
                      config.qdesk.connectionId );

                  return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                      Ext.String.htmlEncode( tooltip ) + '"/>';
                }
              }
          );
        }
      }
      else
      {
        // agent is not ready
        columns.push(
            {
              itemId : 'execHealthIndicatorColumn',
              text : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textExecHealthIndicatorColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipExecHealthIndicatorColumn', 10 ),
              sortable : false,
              groupable : false,
              width : 113,
              renderer : function ()
              {
                var tooltip = QDesk.controller.command.connection.ConnectionUtils.getNotLicensedWarningTooltip();

                return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                    Ext.String.htmlEncode( tooltip ) + '"/>';
              }
            }
        );
      }
    }

    if ( jvmAgentReady )
    {
      columns.push(
          {
            xtype : 'WActionColumn',
            itemId : 'nextFireTimeColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textNextFireTimeColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipNextFireTimeColumn' ),
            dataIndex : 'nextFireTime',
            sortable : true,
            groupable : false,
            flex : 0.5,
            renderer : function ( value, metaData, record )
            {
              return QDesk.util.FormatUtils.formatTimestamp( value );
            },
            // hide icons in case the value is null (default is false)
            hideItemsIfValueNull : true,
            items : [
              {
                icon : 'images/icon/scheduler/quartz/view-next-job-fire-times-16x16.png',
                iconCls : 'clickable',
                tooltip : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipViewNextFireTimes' ),
                disabled : !jvmAgentReady || !publicApiReady,
                handler : function ( grid, rowIndex )
                {
                  var jobDetail = grid.getStore().getAt( rowIndex );
                  this.fireEvent( 'requestViewNextFireTimes', jobDetail.get( 'group' ).name, jobDetail.get( 'name' ) );
                }
              }
            ]
          }
      );
    }

    columns.push(
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'durabilityColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textDurabilityColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipDurabilityColumn' ),
          align : 'center',
          dataIndex : 'durability',
          sortable : false,
          groupable : false,
          width : 90
        },
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'shouldRecoverColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textShouldRecoverColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipShouldRecoverColumn' ),
          align : 'center',
          dataIndex : 'shouldRecover',
          sortable : false,
          groupable : false,
          width : 90
        },

        // include only for Quartz < 2.x
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'volatilityColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textVolatilityColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipVolatilityColumn' ),
          align : 'center',
          dataIndex : 'volatility',
          sortable : false,
          groupable : false,
          width : 90
        }
    );

    if ( jvmAgentReady )
    {
      columns.push(
          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'concurrentColumn',
            text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textConcurrentColumn' ),
            tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipConcurrentColumn' ),
            align : 'center',
            dataIndex : 'concurrentExecutionDisallowed',
            sortable : false,
            groupable : false,
            width : 90,
            renderer : function ( value, metaData )
            {
              // we need to invert the concurrentExecutionDisallowed value
              return this.defaultRenderer( !value, metaData );
            }
          },

          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'persistJobDataAfterExecutionColumn',
            text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.textPersistsJobDataColumn' ),
            tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.tooltipPersistsJobDataColumn' ),
            align : 'center',
            dataIndex : 'persistJobDataAfterExecution',
            sortable : false,
            groupable : false,
            width : 120
          }
      );
    }

    Ext.apply( config, {
      columns : columns,
      store : store,
      qdesk : {
        gridId : 'quartz.jobsGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobGroupStore' );

    jobGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByJobGroupCombo' ).store = jobGroupStore;

    // jobGroupStore.on( 'load', function ( store, records, successful, operation )
    // {
    //   if ( successful )
    //   {
    //     // all is good
    //   }
    //   else
    //   {
    //     var response = operation.getResponse();
    //     if ( response && response.responseText )
    //     {
    //       // application errors
    //       QDesk.util.UxUtils.showJsonResponseErrors( Ext.JSON.decode( response.responseText ) );
    //     }
    //     else
    //     {
    //       QDesk.util.UxUtils.showErrorDialog(
    //           QDesk.Msg.get(
    //               'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab.errorLoadingJobsMsg' ) );
    //     }
    //   }
    // } );

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  /**
   * Invoked to create a context menu for the specified job in the JobsGrid row.
   * @param jobDetail a job.
   * @private
   */
  _createRowCtxMenu : function ( jobDetail )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // trigger job
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.jobCtxMenuTrigger' ),
          iconCls : 'quartzTriggerJob',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_TRIGGER_MANUALLY ],
          qdNotAuthorizedIconCls : 'quartzTriggerJobNAUTH',
          handler : function ()
          {
            me.fireEvent( 'triggerJob', jobDetail );
          }
        },
        '-',
        // delete job
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.jobCtxMenuDelete' ),
          iconCls : 'deleteJob',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_DELETE ],
          qdNotAuthorizedIconCls : 'deleteJobNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteJob', jobDetail );
          }
        },
        // edit job
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.jobCtxMenuEdit' ),
          iconCls : 'editJob',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_UPDATE ],
          qdNotAuthorizedIconCls : 'editJobNAUTH',
          handler : function ()
          {
            me.fireEvent( 'editJob', jobDetail );
          }
        },
        // clone job
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobsGrid.jobCtxMenuClone' ),
          iconCls : 'cloneJob',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_JOB_ADD ],
          qdNotAuthorizedIconCls : 'cloneJobNAUTH',
          handler : function ()
          {
            me.fireEvent( 'cloneJob', jobDetail );
          }
        }
      ]
    } );
  }
} );
