/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsTab', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsGrid'
  ],

  alias : 'widget.QuartzPlannedJobExecutionsTab',

  iconCls : 'quartzPlannedJobExecutionsSubTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.PlannedJobExecutions )
    {
      if ( me.isJvmAgentReady( config.qdesk.jvmAgentInfo ) )
      {
        if ( me.isPublicApiReady( config.qdesk.schedulerInfo ) )
        {
          Ext.apply( config, {
            layout : 'fit',
            items : [
              {
                xtype : 'QuartzPlannedJobExecutionsGrid',
                itemId : 'plannedJobExecutionsGrid',
                flex : 1,
                qdesk : {
                  schedulerInfo : config.qdesk.schedulerInfo,
                  jvmAgentInfo : config.qdesk.jvmAgentInfo
                }
              }
            ]
          } );
        }
        else
        {
          Ext.apply( config, {
            items : [
              {
                xtype : 'WarningMessagePanel',
                style : {
                  margin : '5px'
                },
                qdesk : {
                  msg : me.getPublicApiWarningMessage( config.qdesk.schedulerInfo, config.qdesk.jvmAgentInfo )
                }
              }
            ]
          } );
        }
      }
      else
      {
        Ext.apply( config, {
          items : [
            {
              xtype : 'WarningMessagePanel',
              style : {
                margin : '5px'
              },
              qdesk : {
                msg : me.getJvmAgentWarningMessage( config.qdesk.schedulerInfo, config.qdesk.jvmAgentInfo )
              }
            }
          ]
        } );
      }
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsTab.tooltip' )
    } );

    me.callParent( [ config ] );
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.PlannedJobExecutionsTab.title' );
    if ( !licensedFeatures.PlannedJobExecutions )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );