/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.jobs.triggers.JobTriggersGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.JvmAgentInfoAccessor',
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzJobTriggerGroupStore',
    'QDesk.store.scheduler.quartz.QuartzJobTriggerStore',

    'QDesk.store.scheduler.quartz.QuartzTriggerGroupStore',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.quartz.QuartzEnumUtils',
    'QDesk.util.quartz.QuartzTriggerUtils',

    'QDesk.view.widget.WActionColumn',
    'QDesk.view.widget.WExecHealthIndicator',
    'QDesk.view.widget.WSearchField',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzJobTriggersGrid',

  cls : 'quartzJobTriggersGrid',

  border : false,
  scrollable : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobTriggerStore' );

    var schedulerVersion = config.qdesk.schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = config.qdesk.jvmAgentInfo.isReady();
    var publicApiReady = config.qdesk.schedulerInfo.isPublicApiReady();

    Ext.apply( config, {
      listeners : {
        itemcontextmenu : function ( grid, record, item, index, event )
        {
          var me = this;
          event.stopEvent();
          var rowCtxMenu = me._createRowCtxMenu( record, jvmAgentReady, publicApiReady );
          rowCtxMenu.showAt( event.getXY() );
        }
      }
    } );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'resumeTriggerButton',
          scale : 'small',
          iconCls : 'quartzResumeTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipResumeTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_RESUME ],
          qdNotAuthorizedIconCls : 'quartzResumeTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerButton',
          scale : 'small',
          iconCls : 'quartzPauseTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipPauseTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_PAUSE ],
          qdNotAuthorizedIconCls : 'quartzPauseTriggerNAUTH'
        },
        '-',

        {
          xtype : 'button',
          itemId : 'addTriggerButton',
          scale : 'small',
          iconCls : 'quartzAddTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipAddTriggerButton' ),
          menu : {
            items : QDesk.util.quartz.QuartzTriggerUtils.createAddTriggerButtonMenuItems( schedulerVersion )
          },
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ],
          qdNotAuthorizedIconCls : 'quartzAddTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'deleteTriggerButton',
          scale : 'small',
          iconCls : 'quartzDeleteTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipDeleteTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_DELETE ],
          qdNotAuthorizedIconCls : 'quartzDeleteTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'editTriggerButton',
          scale : 'small',
          iconCls : 'quartzEditTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipEditTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_UPDATE ],
          qdNotAuthorizedIconCls : 'quartzEditTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'cloneTriggerButton',
          scale : 'small',
          iconCls : 'quartzCloneTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipCloneTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ],
          qdNotAuthorizedIconCls : 'quartzCloneTriggerNAUTH'
        },
        '-',

        {
          xtype : 'xcombo',
          itemId : 'filterByTriggerGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.filterByJobGroupComboLabel' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.emptyTextFilterByTriggerGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true,
          tpl : Ext.create( 'Ext.XTemplate',
              '<ul class="x-list-plain"><tpl for=".">',
              '<li role="option" class="x-boundlist-item">{name:htmlEncode}</li>',
              '</tpl></ul>' )
        },
        {
          xtype : 'button',
          itemId : 'resumeTriggerGroupButton',
          scale : 'small',
          iconCls : 'quartzResumeTriggerGroup',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipResumeTriggerGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_GROUP_RESUME ],
          qdNotAuthorizedIconCls : 'quartzResumeTriggerGroupNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerGroupButton',
          scale : 'small',
          iconCls : 'quartzPauseTriggerGroup',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipPauseTriggerGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_GROUP_PAUSE ],
          qdNotAuthorizedIconCls : 'quartzPauseTriggerGroupNAUTH'
        },
        '-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.emptyTextFieldFilterQueryText' ),
          store : store,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ]
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipNameColumn' ),
          dataIndex : 'name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'groupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textGroupNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipGroupNameColumn' ),
          dataIndex : 'group.name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var jobGroup = record.get( 'group' );
            return Ext.String.htmlEncode( jobGroup.name );
          }
        }
    );


    if ( jvmAgentReady )
    {
      if ( publicApiReady )
      {
        columns.push(
            {
              itemId : 'typeColumn',
              text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textTypeColumn' ),
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipTypeColumn' ),
              dataIndex : 'type',
              sortable : true,
              groupable : false,
              flex : 0.5,
              renderer : function ( value )
              {
                return Ext.isEmpty( value ) ? QDesk.Msg.get( 'QDesk.unknown' ) :
                    QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( value );
              }
            }
        );
      }
      else
      {
        // public API is not ready
        columns.push(
            {
              itemId : 'typeColumn',
              text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textTypeColumn' ),
              tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipTypeColumn' ),
              dataIndex : 'type',
              sortable : false,
              groupable : false,
              flex : 0.5,
              renderer : function ()
              {
                var tooltip = QDesk.controller.command.connection.ConnectionUtils.getPublicApiWarningTooltip(
                    config.qdesk.connectionId );

                return '<img src="images/icon/not-available-16x16.png" width="16" height="16" data-qtip="' +
                    Ext.String.htmlEncode( tooltip ) +
                    '"/>';
              }
            }
        );
      }
    }
    else
    {
      // agent is not ready
      columns.push(
          {
            itemId : 'typeColumn',
            text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textTypeColumn' ),
            tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipTypeColumn' ),
            dataIndex : 'type',
            sortable : false,
            groupable : false,
            flex : 0.5,
            renderer : function ()
            {
              var tooltip = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentWarningTooltip(
                  config.qdesk.connectionId );

              return '<img src="images/icon/not-available-16x16.png" width="16" height="16" data-qtip="' +
                  Ext.String.htmlEncode( tooltip ) +
                  '"/>';
            }
          }
      );
    }


    columns.push(
        {
          itemId : 'stateColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textStateColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipStateColumn' ),
          dataIndex : 'state',
          sortable : true,
          groupable : false,
          flex : 0.5
        }
    );

    if ( jvmAgentReady )
    {
      var maxExecHealthIndicatorEntries = config.qdesk.jvmAgentInfo.getConfigProperty(
          'quartz.execHealthIndicators.triggers.maxEntriesPerIndicator' );

      if ( publicApiReady )
      {
        columns.push(
            {
              xtype : 'widgetcolumn',
              itemId : 'execHealthIndicatorColumn',
              text : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textExecHealthIndicatorColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipExecHealthIndicatorColumn',
                  maxExecHealthIndicatorEntries ),
              dataIndex : 'execHealthIndicatorValues',
              sortable : false,
              groupable : false,
              width : 115,
              widget : {
                xtype : 'WExecHealthIndicator'
              }
            }
        );
      }
      else
      {
        columns.push(
            {
              itemId : 'execHealthIndicatorColumn',
              text : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textExecHealthIndicatorColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipExecHealthIndicatorColumn',
                  maxExecHealthIndicatorEntries ),
              sortable : false,
              groupable : false,
              width : 115,
              renderer : function ()
              {
                var tooltip = QDesk.controller.command.connection.ConnectionUtils.getPublicApiWarningTooltip(
                    config.qdesk.connectionId );

                return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                    Ext.String.htmlEncode( tooltip ) + '"/>';
              }
            }
        );
      }
    }
    else
    {
      columns.push(
          {
            itemId : 'execHealthIndicatorColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textExecHealthIndicatorColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipExecHealthIndicatorColumn', 10 ),
            sortable : false,
            groupable : false,
            width : 115,
            renderer : function ()
            {
              var tooltip = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentWarningTooltip(
                  config.qdesk.connectionId );

              return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                  Ext.String.htmlEncode( tooltip ) + '"/>';
            }
          }
      );
    }

    columns.push(
        {
          xtype : 'actioncolumn',
          itemId : 'jobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textJobDataMapColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var triggerRecord = grid.getStore().getAt( rowIndex );

                var jobDataMap = triggerRecord.get( 'jobDataMap' );
                var mergedJobDataMap = triggerRecord.get( 'mergedJobDataMap' );
                this.fireEvent( 'requestViewJobDataMap', jobDataMap, mergedJobDataMap );
              }
            }
          ]
        },
        {
          xtype : 'datecolumn',
          itemId : 'startTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textStartTimeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipStartTimeColumn' ),
          dataIndex : 'startTime',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'endTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textEndTimeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipEndTimeColumn' ),
          dataIndex : 'endTime',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'datecolumn',
          itemId : 'previousFireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textPreviousFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipPreviousFireTimeColumn' ),
          dataIndex : 'previousFireTime',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          xtype : 'WActionColumn',
          itemId : 'nextFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textNextFireTimeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipNextFireTimeColumn' ),
          dataIndex : 'nextFireTime',
          sortable : true,
          groupable : false,
          flex : 1.1,
          renderer : function ( value, metaData, record )
          {
            return QDesk.util.FormatUtils.formatTimestamp( value );
          },
          items : [
            {
              icon : 'images/icon/scheduler/quartz/view-next-trigger-fire-times-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipViewTriggerNextFireTimes' ),
              disabled : !jvmAgentReady || !publicApiReady,
              handler : function ( grid, rowIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                this.fireEvent( 'requestViewNextFireTimes', record.get( 'group' ).name, record.get( 'name' ) );
              }
            }
          ]
        },

        {
          xtype : 'datecolumn',
          itemId : 'finalFireTimeColumn',
          format : QDesk.ApplicationConfig.get( 'timestampFormat' ),
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textFinalFireTimeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipFinalFireTimeColumn' ),
          dataIndex : 'finalFireTime',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'calendarNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textCalendarNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipCalendarNameColumn' ),
          dataIndex : 'calendar',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var calendar = record.get( 'calendar' );
            return calendar == null ? null : Ext.String.htmlEncode( calendar.displayName );
          }
        },
        {
          xtype : 'numbercolumn',
          itemId : 'priorityColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textPriorityColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipPriorityColumn' ),
          dataIndex : 'priority',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'numbercolumn',
          itemId : 'misfireInstructionColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.textMisfireInstructionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipMisfireInstructionColumn' ),
          dataIndex : 'misfireInstruction',
          sortable : false,
          groupable : false,
          /* Need custom renderer to display misfire instruction name */
          renderer : function ( value, metaData, record )
          {
            var triggerType = record.get( 'type' );

            if ( triggerType )
            {
              var misfireInstruction =
                  QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionFromNumber( triggerType,
                      value );

              return QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionName( triggerType,
                  misfireInstruction );
            }
            else
            {
              // if we do not know the trigger type, then we are not able to determine the misfire instruction name;
              // so we display the numeric constant instead
              return QDesk.Msg.get( 'QDesk.unknown' ) + ' (' + value.toString() + ')';
            }
          },
          flex : 1
        }
    );

    Ext.apply( config, {
      columns : columns,
      plugins : [
        {
          ptype : 'rowexpander',
          rowBodyTpl : [
            '<p>' +
            '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
            '<tpl if="description">{[Ext.String.htmlEncode(values.description)]}</tpl>' +
            '<tpl if="!description">{[Ext.String.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
            '</p>' +

            '{[QDesk.util.quartz.QuartzTriggerUtils.createRowDetailsTableHtml( values )]}' +

            '<p style="white-space: nowrap">' +
            '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
            '{[this.getMonitoringLink( values.monitoringUrl )]}' +
            '</p>',
            {
              getMonitoringLink : function ( monitoringUrl )
              {
                var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
                var tooltip;
                if ( licensedFeatures.PollMonitoring )
                {
                  tooltip =
                      Ext.String.htmlEncode( QDesk.Msg.get(
                          'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipMonitoringUrl' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
                }
                else
                {
                  tooltip =
                      Ext.String.htmlEncode( QDesk.Msg.get(
                          'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.tooltipMonitoringUrlNotLicensed' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                      tooltip + '">' + monitoringUrl + '</a>';
                }
              }
            }
          ]
        }
      ]
    } );

    Ext.apply( config, {
      store : store,

      qdesk : {
        gridId : 'quartz.jobTriggersGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobTriggerGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzJobTriggerGroupStore' );
    jobTriggerGroupStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;

    me.down( '#filterByTriggerGroupCombo' ).store = jobTriggerGroupStore;

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          var jobsTab = me.up( '#jobsTab' );

          // default empty text
          var emptyText = QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.emptyTextNoMatchingData' );

          if ( jobsTab )
          {
            var selectedJobs = jobsTab.down( '#jobsGrid' ).getSelectionModel().getSelection();
            if ( Ext.isEmpty( selectedJobs ) )
            {
              emptyText =
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.emptyTextNoJobSelected' );
            }
          }

          view.emptyText = emptyText;

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  /**
   * Invoked to create a context menu for the specified trigger in the JobTriggersGrid row.
   * @param trigger a trigger.
   * @param jvmAgentReady a flag indicating if the QuartzDesk JVM agent is ready.
   * @param publicApiReady a flag indicating if the QuartzDesk Public API is ready.
   * @private
   */
  _createRowCtxMenu : function ( trigger, jvmAgentReady, publicApiReady )
  {
    var me = this;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // resume trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuResume' ),
          iconCls : 'quartzResumeTrigger',
          disabled : trigger.get( 'state' ) != QDesk.model.scheduler.quartz.QuartzTriggerState.PAUSED,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_RESUME ],
          qdNotAuthorizedIconCls : 'quartzResumeTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'resumeTrigger', trigger );
          }
        },
        // pause trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuPause' ),
          iconCls : 'quartzPauseTrigger',
          disabled : trigger.get( 'state' ) != QDesk.model.scheduler.quartz.QuartzTriggerState.NORMAL,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_PAUSE ],
          qdNotAuthorizedIconCls : 'quartzPauseTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'pauseTrigger', trigger );
          }
        },
        '-',
        // delete trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuDelete' ),
          iconCls : 'quartzDeleteTrigger',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_DELETE ],
          qdNotAuthorizedIconCls : 'quartzDeleteTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteTrigger', trigger );
          }
        },
        // edit trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuEdit' ),
          iconCls : 'quartzEditTrigger',
          disabled : !jvmAgentReady || !publicApiReady,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_UPDATE ],
          qdNotAuthorizedIconCls : 'quartzEditTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'editTrigger', trigger );
          }
        },
        // clone trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.jobs.JobTriggersGrid.triggerCtxMenuClone' ),
          iconCls : 'quartzCloneTrigger',
          disabled : !jvmAgentReady || !publicApiReady,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ],
          qdNotAuthorizedIconCls : 'quartzCloneTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'cloneTrigger', trigger );
          }
        }
      ]
    } );
  }
} );
