/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedJobsChart', {
  extend : 'Ext.chart.CartesianChart',

  requires : [
    'QDesk.Msg',
    'QDesk.model.scheduler.ExecStatus',
    'QDesk.model.scheduler.quartz.stats.QuartzSchedulerStatisticsType',
    'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore',
    'QDesk.util.EnumUtils',

    'QDesk.view.work.scheduler.quartz.stats.theme.SlowestExecutedJobsChartTheme'
  ],

  style : 'background:#fff',

  animate : true,

  border : false,
  bodyBorder : false,

  flipXY : true,

  margin : '0 15 0 0',

  shadow : true,

  theme : 'QuartzSlowestExecutedJobsChartTheme',

  /*
   * Present to suppress warning: Using Sencha's download server could expose your data and pose a security risk. Please see Ext.draw.Container#download method docs for more info.
   */
  downloadServerUrl : 'http://localhost',

  qdesk : {
    /**
     * Required config parameter with the statistics period length for this chart.
     */
    periodLength : undefined,

    /**
     * Required config parameter with the statistics period start date for this chart.
     */
    periodStart : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.periodLength )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodLength' );

    if ( !config.qdesk.periodStart )
      Ext.Error.raise( 'Required config parameter missing: qdesk.periodStart' );

    Ext.apply( config, {
      store : Ext.create( 'QDesk.store.scheduler.quartz.stats.QuartzExecStatDataStore' ),

      axes : [
        {
          type : 'numeric',
          minimum : 0,
          position : 'bottom',
          fields : [ 'avgTimeSuccessAndError' ],
          title : {
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedJobsChart.labelValueAxis' ),
            fontSize : 16
          },
          minorTickSteps : 1,
          grid : {
            odd : {
              globalAlpha : 1,
              fillStyle : '#ddd',
              strokeStyle : '#bbb',
              lineWidth : 0.5
            }
          }
        },

        {
          type : 'category',
          position : 'left',
          fields : [ 'jobDisplayName' ],
          title : {
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedJobsChart.labelCategoryAxis' ),
            fontSize : 16
          }
        }
      ],

      series : [
        {
          type : 'bar',
          xField : 'jobDisplayName',
          yField : 'avgTimeSuccessAndError',
          highlight : {
            strokeStyle : '#505050',
            fillStyle : '#BF8073',
            lineWidth : 2,
            lineDash : [ 2, 2 ]
          },
          tooltip : {
            dismissDelay : 0,
            trackMouse : true,
            renderer : function ( tooltip, storeItem, ctx )
            {
              var html = QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.stats.SlowestExecutedJobsChart.tooltipBar',
                  Ext.String.htmlEncode( storeItem.get( 'jobGroupName' ) ),
                  Ext.String.htmlEncode( storeItem.get( 'jobName' ) ),
                  Ext.util.Format.number( storeItem.get( 'countSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeSuccess' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeSuccess' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'timeError' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeError' ), '0,000.00' ),
                  Ext.util.Format.number( storeItem.get( 'countVeto' ), '0,000' ),
                  Ext.util.Format.number( storeItem.get( 'avgTimeSuccessAndError' ), '0,000.00' ) );

              tooltip.setHtml( html );
            }
          }
        }
      ]
    } );

    me.callParent( [ config ] );
  }
} );