/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm', {
  extend : 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.quartz.QuartzIntervalType',

    'QDesk.store.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstructionStore',
    'QDesk.store.enums.scheduler.quartz.QuartzIntervalTypeStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.base.BaseFormPanel'
  ],

  alias : 'widget.QuartzEditNthIncludedDayTriggerForm',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    me.callParent( [ config ] );
  },


  /**
   * @private
   * @override
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    if ( trigger.get( 'timeZone' ) != null )
    {
      me.getForm().setValues( {
        timeZone : trigger.get( 'timeZone' ).id
      } );
    }
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    var intervalTypeStore = Ext.create( 'QDesk.store.enums.scheduler.quartz.QuartzIntervalTypeStore', {
      qdesk : {
        schedulerVersion : qdesk.schedulerInfo.getSchedulerVersion()
      }
    } );

    var timeZoneStore = Ext.create( 'Ext.data.ArrayStore', {
      model : 'QDesk.model.common.TimeZone'
    } );

    timeZoneStore.loadData( qdesk.jvmAgentInfo.getAvailableTimeZones() );

    return [
      {
        xtype : 'combo',
        itemId : 'intervalType',
        name : 'intervalType',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldLabelIntervalType' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldTooltipIntervalType' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldEmptyTxtIntervalType' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : intervalTypeStore,
        valueField : 'value',
        displayField : 'valueLabel'
      },
      {
        xtype : 'numberfield',
        itemId : 'nthDay',
        name : 'nthDay',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldLabelNthDay' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldTooltipNthDay' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldEmptyTxtNthDay' ),
        minValue : 1,
        allowBlank : false
      },

      {
        xtype : 'numberfield',
        itemId : 'nextFireCutoffInterval',
        name : 'nextFireCutoffInterval',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldLabelNextFireCutoffInterval' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldTooltipNextFireCutoffInterval' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldEmptyTxtNextFireCutoffInterval' ),
        minValue : 1,
        allowBlank : false
      },

      {
        xtype : 'timefield',
        itemId : 'fireAt',
        name : 'fireAt',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldLabelFireAt' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldTooltipFireAt' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldEmptyTxtFireAt' ),
        allowBlank : false,
        format : QDesk.util.constant.DateFormat.dayTimeFormat
      },

      {
        xtype : 'combo',
        itemId : 'timeZone',
        name : 'timeZone',
        fieldLabel : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldLabelTimeZone' ),
        fieldTooltip : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldTooltipTimeZone' ),
        emptyText : QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.EditNthIncludedDayTriggerForm.fieldEmptyTxtTimeZone' ),
        allowBlank : false,
        editable : false,
        forceSelection : true,
        queryMode : 'local',
        store : timeZoneStore,
        valueField : 'id',
        displayField : 'displayName',
        value : qdesk.jvmAgentInfo.getTimeZone().id
      }
    ];
  },


  /**
   * @private
   * @override
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    return Ext.create( 'QDesk.store.enums.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstructionStore', {
      qdesk : {
        schedulerVersion : schedulerVersion
      }
    } );
  },


  /**
   * @private
   * @override
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
    var me = this;
    me.callParent( arguments );

    /* QDesk.model.scheduler.quartz.QuartzNthIncludedDayTriggerMisfireInstruction.SMART_POLICY */
    me.down( '#misfireInstruction' ).setValue( 0 );

    me.down( '#intervalType' ).setValue( QDesk.model.scheduler.quartz.QuartzIntervalType.MONTHLY );
    me.down( '#nthDay' ).setValue( 1 );

    me.down( '#nextFireCutoffInterval' ).setValue( 12 );

    me.down( '#fireAt' ).setValue( '00:00:00' );
  }
} );
