/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.scheduler.quartz.QuartzCalendarStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.util.quartz.QuartzUtils',
    'QDesk.util.quartz.QuartzEnumUtils',

    'QDesk.view.work.scheduler.quartz.JobDataMapGrid',
    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode',

    'Ext.ux.form.field.DateTime',
    'Ext.ux.form.ClearableCombo'
  ],

  bodyPadding : '5 10 10 10',

  fieldDefaults : {
    labelWidth : 100
  },

  defaultButton : 'actionButton',

  qdesk : {
    /**
     * Scheduler connection ID.
     */
    connectionId : undefined,

    /*
     * Mode of the form (see EditTriggerFormMode).
     */
    mode : undefined,

    /**
     * Trigger type (see QDesk.model.scheduler.quartz.QuartzTriggerType).
     */
    triggerType : undefined,

    /*
     * Trigger job group name.
     */
    triggerJobGroupName : undefined,

    /*
     * Trigger job name.
     */
    triggerJobName : undefined,

    /*
     * JVM Agent info object (see QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo).
     */
    jvmAgentInfo : undefined,

    /*
     * Scheduler info object (see QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo).
     */
    schedulerInfo : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    var jvmAgentReady = config.qdesk.jvmAgentInfo.isReady();
    var publicApiReady = config.qdesk.schedulerInfo.isPublicApiReady();

    var propertiesItems = me._getTriggerSpecificFormItems( config.qdesk );

    var calendarStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzCalendarStore', {
      autoLoad : false
    } );

    calendarStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      items : [
        /* Start of hidden fields */
        {
          xtype : 'hiddenfield',
          itemId : 'type',
          name : 'type',
          value : config.qdesk.triggerType
        },
        /* End of hidden fields */

        {
          xtype : 'displayfield',
          cls : 'formHeadingText',
          value : me._getTriggerSpecificFormHeadingText( config.qdesk.triggerType ),
          listeners : {
            render : function ( c )
            {
              var field = c.getEl().select( 'div.x-form-display-field' );
              if ( field )
              {
                var tooltip = me._getTriggerSpecificFormHeadingTooltip( config.qdesk.triggerType );
                field.createChild( {
                  tag : 'img',
                  src : 'images/icon/tooltip-10x10.png',
                  style : 'margin-bottom: -1px; margin-left: 5px; padding: 0px; width: 10px; height: 10px;',
                  'data-qtip' : Ext.String.htmlEncode( tooltip )
                } );
              }
            }
          }
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldsetLabelJob' ),
          padding : '0 10 0 10',
          defaults : {
            labelWidth : 55,
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'jobName',
              name : 'jobName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelJobName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtJobName' ),
              allowBlank : false,
              readOnly : true,
              value : config.qdesk.triggerJobName
            },
            {
              xtype : 'textfield',
              itemId : 'jobGroupName',
              name : 'jobGroupName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelJobGroup' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtJobGroup' ),
              allowBlank : false,
              readOnly : true,
              value : config.qdesk.triggerJobGroupName
            }
          ]
        },  // end of Job fieldset


        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldsetLabelTrigger' ),
          padding : '0 10 0 10',
          defaults : {
            labelWidth : 115,
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              labelWidth : 55,
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelName' ),
              fieldTooltip : config.qdesk.mode == QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT ?
                  null :
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtName' ),
              allowBlank : false,
              readOnly : config.qdesk.mode == QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT
            },
            {
              xtype : 'textfield',
              itemId : 'group',
              name : 'group',
              labelWidth : 55,
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelGroup' ),
              fieldTooltip : config.qdesk.mode == QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT ?
                  null :
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipGroup' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtGroup' ),
              allowBlank : false,
              readOnly : config.qdesk.mode == QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT
            },

            {
              xtype : 'textareafield',
              itemId : 'description',
              name : 'description',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelDescription' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipDescription' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtDescription' ),
              labelAlign : 'top',
              allowBlank : true
            },

            /* Trigger start time */
            {
              xtype : 'xdatetime',
              itemId : 'startTime',
              name : 'startTime',
              allowBlank : false,
              value : new Date(),
              timeFormat : QDesk.util.constant.DateFormat.timeFormatLong,
              dateConfig : {
                fieldLabel : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelStartTime' ),
                fieldTooltip : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipStartTime' ),
                emptyText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtStartTimeDate' ),
                formatText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldFormatTxtStartTimeDate' ),
                labelWidth : 115
              },
              timeConfig : {
                emptyText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtStartTimeTime' ),
                formatText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldFormatTxtStartTimeTime' ),
                increment : 30 /* 30 minutes */
              }
            },

            /* Trigger end time */
            {
              xtype : 'xdatetime',
              itemId : 'endTime',
              name : 'endTime',
              allowBlank : true,
              timeFormat : QDesk.util.constant.DateFormat.timeFormatLong,
              dateConfig : {
                fieldLabel : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelEndTime' ),
                fieldTooltip : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipEndTime' ),
                emptyText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtEndTimeDate' ),
                formatText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldFormatTxtEndTimeDate' ),
                labelWidth : 115
              },
              timeConfig : {
                emptyText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtEndTimeTime' ),
                formatText : QDesk.Msg.get(
                    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldFormatTxtEndTimeTime' ),
                increment : 30 /* 30 minutes */
              }
            },

            {
              xtype : 'numberfield',
              itemId : 'priority',
              name : 'priority',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelPriority' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipPriority' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtPriority' ),
              value : 5, /* default priority value */
              minValue : 1,
              maxValue : 65535,
              allowBlank : false
            },

            {
              xtype : 'combo',
              itemId : 'misfireInstruction',
              name : 'misfireInstruction',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelMisfireInstruction' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipMisfireInstruction' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtMisfireInstruction' ),
              allowBlank : false,
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : me._getTriggerSpecificMisfireInstructionStore( config.qdesk.schedulerInfo.getSchedulerVersion() ),
              valueField : 'value',
              displayField : 'valueLabel'
            },

            {
              xtype : 'xcombo',
              itemId : 'calendarName',
              name : 'calendarName',
              fieldLabel : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelCalendarName' ),
              fieldTooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldTooltipCalendarName' ),
              emptyText : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtCalendarName' ),
              allowBlank : true,
              editable : false,
              forceSelection : false,
              queryMode : 'local',
              store : calendarStore,
              valueField : 'id',
              displayField : 'displayName',
              tpl : Ext.create( 'Ext.XTemplate',
                  '<ul class="x-list-plain"><tpl for=".">',
                  '<li role="option" class="x-boundlist-item">{displayName:htmlEncode}</li>',
                  '</tpl></ul>' )
            },

            {
              xtype : 'tabpanel',
              padding : '0 0 10 0',
              items : [
                {
                  xtype : 'panel',
                  padding : '10 10 5 10',
                  border : false,
                  title : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.tabLabelProperties' ),
                  tooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.tabTooltipProperties' ),
                  layout : 'anchor',
                  defaults : {
                    labelWidth : 135,
                    anchor : '100%'
                  },
                  items : propertiesItems
                },
                {
                  xtype : 'panel',
                  bodyPadding : 0,
                  border : false,
                  iconCls : 'quartzJobDataMapTabIcon',
                  title : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.tabLabelJobDataMap' ),
                  tooltip : QDesk.Msg.get(
                      'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.tabTooltipJobDataMap' ),
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%'
                  },
                  items : [
                    {
                      xtype : 'QuartzJobDataMapGrid',
                      border : false,
                      itemId : 'jobDataMapGrid',
                      height : 150,

                      qdesk : {
                        editable : true,
                        jobDataMap : []
                      }
                    }
                  ]
                }
              ]
            }
          ]
        }  // end of Trigger fieldset
      ],

      buttons : [
        {
          itemId : 'viewNextFireTimesButton',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.buttonLabelViewNextFireTimes' ),
          iconCls : 'quartzViewNextFireTimes',
          formBind : jvmAgentReady && publicApiReady,
          disabled : !jvmAgentReady || !publicApiReady
        },
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          reference : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : jvmAgentReady && publicApiReady,
          disabled : !jvmAgentReady || !publicApiReady,
          qdRequiresAnyPermissions : me._getRequiredPermissions( config.qdesk.mode )
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();

    me._setDefaultFormValues();
  },


  /**
   * Returns an array of all form stores that must be loaded before this form can be opened.
   */
  getFormStores : function ()
  {
    var me = this;

    var stores = [ me.down( '#calendarName' ).getStore() ];

    // append trigger type specific stores
    return stores.concat( me._getTriggerSpecificStores() );
  },


  /**
   * Loads the specified trigger record ({@link QuartzTrigger} instance) into this form.
   * @param trigger the trigger record.
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    me.getForm().setValues( {
      group : trigger.get( 'group' ).name,
      calendarName : trigger.get( 'calendar' ) ? trigger.get( 'calendar' ).id : null
    } );

    QDesk.util.quartz.QuartzUtils.populateJobDataMap( me.down( '#jobDataMapGrid' ), trigger.get( 'jobDataMap' ) );
  },


  /**
   * Returns the {@link QuartzTrigger} instance with data from this form.
   */
  getTriggerRecord : function ()
  {
    var me = this;

    var form = me.getForm();
    var jobDataMap = me.getJobDataMap();

    return QDesk.util.quartz.QuartzTriggerUtils.createTrigger( form.getFieldValues(), jobDataMap );
  },


  /**
   * Returns the job data map as an array.
   * @returns {Array} the array of job data map entries.
   */
  getJobDataMap : function ()
  {
    var me = this;
    return QDesk.util.quartz.QuartzUtils.createJobDataMap( me.down( '#jobDataMapGrid' ) );
  },


  /**
   * Binds event handlers to all form widgets. Sub-classes must not override this method, they
   * should override the _bindTriggerSpecificEventHandlers method.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    me.down( '#viewNextFireTimesButton' ).on( 'click', function ()
    {
      var trigger = me.getTriggerRecord();

      Ext.create( 'QDesk.controller.command.scheduler.quartz.triggers.ViewNextFireTimes', {
        connectionId : me.qdesk.connectionId,
        trigger : trigger,
        maskEl : me.up( 'window' )
      } ).execute();
    } );

    // nothing to bind here for generic trigger form fields
    me._bindTriggerSpecificEventHandlers();
  },


  /**
   * Sets the default values of individual form fields. Sub-classes must not override this method, they
   * should override the _setTriggerSpecificFormDefaultValues method to set trigger type specific default
   * form values.
   */
  _setDefaultFormValues : function ()
  {
    var me = this;
    me._setTriggerSpecificFormDefaultValues();
  },


  /**
   * Returns the array of trigger type specific form items. Sub-classes must
   * override this method.
   * @param qdesk a QuartzDesk object wrapping various parameters passed to the form upon its creation.
   * @returns {Array} the array of trigger type specific form items.
   * @private
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    Ext.Error.raise( 'Sub-classes must override the _getTriggerSpecificFormItems method.' );
  },


  /**
   * Returns the trigger type specific misfire instruction store. Sub-classes must
   * override this method.
   * @param schedulerVersion the scheduler version.
   * @returns {Store} the misfire instruction store.
   * @private
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    Ext.Error.raise( 'Sub-classes must override the _getTriggerSpecificMisfireInstructionStore method.' );
  },


  /**
   * Binds event handlers to trigger type specific form widgets. Sub-classes can override
   * this method.
   */
  _bindTriggerSpecificEventHandlers : function ()
  {
  },


  /**
   * Sets the default values of individual form fields. Sub-classes can override
   * this method.
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
  },


  /**
   * Invoked by the base form to obtain the array of all trigger type specific stores from
   * sub-classes. Sub-classes can override this method.
   */
  _getTriggerSpecificStores : function ()
  {
    return [];
  },


  _getTriggerSpecificFormHeadingText : function ( triggerType )
  {
    return QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( triggerType );
  },


  _getTriggerSpecificFormHeadingTooltip : function ( triggerType )
  {
    return QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.EditTriggerBaseForm.titleTooltip.' + triggerType );
  },


  _getActionButtonText : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  },


  _getRequiredPermissions : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.ADD:
        return [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ];

      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.EDIT:
        return [ QDesk.model.security.Permission.QUARTZ_TRIGGER_UPDATE ];

      case QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.CLONE:
        return [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ];

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
