/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.JvmAgentInfoAccessor',
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.util.FormatUtils',

    'QDesk.store.scheduler.quartz.QuartzTriggerGroupStore',
    'QDesk.store.scheduler.quartz.QuartzTriggerWithJobDetailStore',

    'QDesk.view.widget.WActionColumn',
    'QDesk.view.widget.WExecHealthIndicator',
    'QDesk.view.widget.WSearchField',
    'QDesk.view.widget.WReadOnlyCheckColumn',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.quartz.QuartzEnumUtils',
    'QDesk.util.quartz.QuartzTriggerUtils',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzTriggersGrid',

  cls : 'quartzTriggersGrid',

  border : false,
  scrollable : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzTriggerWithJobDetailStore' );

    store.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    var schedulerVersion = config.qdesk.schedulerInfo.getSchedulerVersion();
    var jvmAgentReady = config.qdesk.jvmAgentInfo.isReady();
    var publicApiReady = config.qdesk.schedulerInfo.isPublicApiReady();

    Ext.apply( config, {
      listeners : {
        itemcontextmenu : function ( grid, record, item, index, event )
        {
          var me = this;
          event.stopEvent();
          var rowCtxMenu = me._createRowCtxMenu( record, jvmAgentReady, publicApiReady );
          rowCtxMenu.showAt( event.getXY() );
        }
      }
    } );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'button',
          itemId : 'resumeTriggerButton',
          scale : 'small',
          iconCls : 'quartzResumeTrigger',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipResumeTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_RESUME ],
          qdNotAuthorizedIconCls : 'quartzResumeTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerButton',
          scale : 'small',
          iconCls : 'quartzPauseTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipPauseTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_PAUSE ],
          qdNotAuthorizedIconCls : 'quartzPauseTriggerNAUTH'
        },
        '-',

        {
          xtype : 'button',
          itemId : 'addTriggerButton',
          scale : 'small',
          iconCls : 'quartzAddTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipAddTriggerButton' ),
          menu : {
            items : QDesk.util.quartz.QuartzTriggerUtils.createAddTriggerButtonMenuItems( schedulerVersion )
          },
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ],
          qdNotAuthorizedIconCls : 'quartzAddTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'deleteTriggerButton',
          scale : 'small',
          iconCls : 'quartzDeleteTrigger',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipDeleteTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_DELETE ],
          qdNotAuthorizedIconCls : 'quartzDeleteTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'editTriggerButton',
          scale : 'small',
          iconCls : 'quartzEditTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipEditTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_UPDATE ],
          qdNotAuthorizedIconCls : 'quartzEditTriggerNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'cloneTriggerButton',
          scale : 'small',
          iconCls : 'quartzCloneTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipCloneTriggerButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ],
          qdNotAuthorizedIconCls : 'quartzCloneTriggerNAUTH'
        },
        '-',

        {
          xtype : 'xcombo',
          itemId : 'filterByTriggerGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.filterByJobGroupComboLabel' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.emptyTextFilterByTriggerGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true,
          tpl : Ext.create( 'Ext.XTemplate',
              '<ul class="x-list-plain"><tpl for=".">',
              '<li role="option" class="x-boundlist-item">{name:htmlEncode}</li>',
              '</tpl></ul>' )
        },
        {
          xtype : 'button',
          itemId : 'resumeTriggerGroupButton',
          scale : 'small',
          iconCls : 'quartzResumeTriggerGroup',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipResumeTriggerGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_GROUP_RESUME ],
          qdNotAuthorizedIconCls : 'quartzResumeTriggerGroupNAUTH'
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerGroupButton',
          scale : 'small',
          iconCls : 'quartzPauseTriggerGroup',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipPauseTriggerGroupButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_GROUP_PAUSE ],
          qdNotAuthorizedIconCls : 'quartzPauseTriggerGroupNAUTH'
        },
        '-',
        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.emptyTextFieldFilterQueryText' ),
          store : store,
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ]
    } );

    var columns = [];

    columns.push(
        {
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerNameColumn' ),
          dataIndex : 'trigger.name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return Ext.String.htmlEncode( trigger.name );
          }
        },
        {
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerGroupNameColumn' ),
          dataIndex : 'trigger.group.name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return Ext.String.htmlEncode( trigger.group.name );
          }
        }
    );


    if ( jvmAgentReady )
    {
      if ( publicApiReady )
      {
        columns.push(
            {
              itemId : 'triggerTypeColumn',
              text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerTypeColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerTypeColumn' ),
              dataIndex : 'trigger.type',
              sortable : true,
              groupable : false,
              flex : 0.5,
              renderer : function ( value, metaData, record )
              {
                var trigger = record.get( 'trigger' );
                return Ext.isEmpty( trigger.type ) ? QDesk.Msg.get( 'QDesk.unknown' ) :
                    QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( trigger.type );
              }
            }
        );
      }
      else
      {
        // public API is not ready
        columns.push(
            {
              itemId : 'triggerTypeColumn',
              text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerTypeColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerTypeColumn' ),
              dataIndex : 'trigger.type',
              sortable : false,
              groupable : false,
              flex : 0.5,
              renderer : function ()
              {
                var tooltip = QDesk.controller.command.connection.ConnectionUtils.getPublicApiWarningTooltip(
                    config.qdesk.connectionId );

                return '<img src="images/icon/not-available-16x16.png" width="16" height="16" data-qtip="' +
                    Ext.String.htmlEncode( tooltip ) +
                    '"/>';
              }
            }
        );
      }
    }
    else
    {
      // agent is not ready
      columns.push(
          {
            itemId : 'triggerTypeColumn',
            text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerTypeColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerTypeColumn' ),
            dataIndex : 'trigger.type',
            sortable : false,
            groupable : false,
            flex : 0.5,
            renderer : function ()
            {
              var tooltip = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentWarningTooltip(
                  config.qdesk.connectionId );

              return '<img src="images/icon/not-available-16x16.png" width="16" height="16" data-qtip="' +
                  Ext.String.htmlEncode( tooltip ) +
                  '"/>';
            }
          }
      );
    }


    columns.push(
        {
          itemId : 'triggerStateColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerStateColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerStateColumn' ),
          dataIndex : 'trigger.state',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return trigger.state;
          }
        }
    );

    if ( jvmAgentReady )
    {
      var maxExecHealthIndicatorEntries = config.qdesk.jvmAgentInfo.getConfigProperty(
          'quartz.execHealthIndicators.triggers.maxEntriesPerIndicator' );

      if ( publicApiReady )
      {
        columns.push(
            {
              xtype : 'widgetcolumn',
              itemId : 'triggerExecHealthIndicatorColumn',
              text : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerExecHealthIndicatorColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerExecHealthIndicatorColumn',
                  maxExecHealthIndicatorEntries ),
              dataIndex : 'trigger.execHealthIndicatorValues',
              sortable : false,
              groupable : false,
              width : 115,
              widget : {
                xtype : 'WExecHealthIndicator'
              },
              onWidgetAttach : function ( column, widget, record )
              {
                var trigger = record.get( 'trigger' );
                if ( trigger.execHealthIndicatorValues )
                {
                  widget.setValues( trigger.execHealthIndicatorValues );
                }
              }
            }
        );
      }
      else
      {
        // agent is ready, but public API is not ready
        columns.push(
            {
              itemId : 'triggerExecHealthIndicatorColumn',
              text : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerExecHealthIndicatorColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerExecHealthIndicatorColumn',
                  maxExecHealthIndicatorEntries ),
              dataIndex : 'trigger.execHealthIndicatorValues',
              sortable : false,
              groupable : false,
              width : 115,
              renderer : function ()
              {
                var tooltip = QDesk.controller.command.connection.ConnectionUtils.getPublicApiWarningTooltip(
                    config.qdesk.connectionId );

                return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                    Ext.String.htmlEncode( tooltip ) + '"/>';
              }
            }
        );
      }
    }
    else
    {
      // agent is not ready
      columns.push(
          {
            itemId : 'triggerExecHealthIndicatorColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerExecHealthIndicatorColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerExecHealthIndicatorColumn', 10 ),
            dataIndex : 'trigger.execHealthIndicatorValues',
            sortable : false,
            groupable : false,
            width : 115,
            renderer : function ()
            {
              var tooltip = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentWarningTooltip(
                  config.qdesk.connectionId );

              return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                  Ext.String.htmlEncode( tooltip ) + '"/>';
            }
          }
      );
    }

    columns.push(
        {
          xtype : 'actioncolumn',
          itemId : 'triggerJobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerJobDataMapColumn' ),
          header : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerJobDataMapColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/scheduler/quartz/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerJobDataMapColumnIcon' ),
              handler : function ( grid, rowIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var trigger = record.get( 'trigger' );
                this.fireEvent( 'requestViewJobDataMap', trigger.jobDataMap, trigger.mergedJobDataMap );
              }
            }
          ]
        },
        {
          itemId : 'triggerStartTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerStartTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerStartTimeColumn' ),
          dataIndex : 'trigger.startTime',
          groupable : false,
          flex : 1.1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return QDesk.util.FormatUtils.parseAndFormatTimestamp( trigger.startTime );
          }
        },
        {
          itemId : 'triggerEndTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerEndTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerEndTimeColumn' ),
          dataIndex : 'trigger.endTime',
          sortable : true,
          groupable : false,
          flex : 1.1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return QDesk.util.FormatUtils.parseAndFormatTimestamp( trigger.endTime );
          }
        },
        {
          itemId : 'triggerPreviousFireTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerPreviousFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerPreviousFireTimeColumn' ),
          dataIndex : 'trigger.previousFireTime',
          sortable : true,
          groupable : false,
          flex : 1.1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return QDesk.util.FormatUtils.parseAndFormatTimestamp( trigger.previousFireTime );
          }
        },

        {
          xtype : 'WActionColumn',
          itemId : 'triggerNextFireTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerNextFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerNextFireTimeColumn' ),
          dataIndex : 'trigger.nextFireTime',
          sortable : true,
          groupable : false,
          flex : 1.25,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return QDesk.util.FormatUtils.parseAndFormatTimestamp( trigger.nextFireTime );
          },
          items : [
            {
              icon : 'images/icon/scheduler/quartz/view-next-trigger-fire-times-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipViewTriggerNextFireTimes' ),
              disabled : !jvmAgentReady || !publicApiReady,
              handler : function ( grid, rowIndex )
              {
                var record = grid.getStore().getAt( rowIndex );
                var trigger = record.get( 'trigger' );
                this.fireEvent( 'requestViewNextFireTimes', trigger.group.name, trigger.name );
              }
            }
          ]
        },

        {
          itemId : 'triggerFinalFireTimeColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerFinalFireTimeColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerFinalFireTimeColumn' ),
          dataIndex : 'trigger.finalFireTime',
          sortable : true,
          groupable : false,
          flex : 1.1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return QDesk.util.FormatUtils.parseAndFormatTimestamp( trigger.finalFireTime );
          }
        },
        {
          itemId : 'triggerCalendarNameColumn',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerCalendarNameColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerCalendarNameColumn' ),
          dataIndex : 'trigger.calendar',
          sortable : false,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return trigger.calendar == null ? null : trigger.calendar.displayName;
          }
        },
        {
          itemId : 'triggerPriorityColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerPriorityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerPriorityColumn' ),
          dataIndex : 'trigger.priority',
          sortable : false,
          groupable : false,
          flex : 0.5,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return trigger.priority;
          }
        },
        {
          itemId : 'triggerMisfireInstructionColumn',
          format : '0',
          text : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textTriggerMisfireInstructionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerMisfireInstructionColumn' ),
          dataIndex : 'trigger.misfireInstruction',
          sortable : false,
          groupable : false,
          /* Need custom renderer to display misfire instruction name */
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );

            if ( trigger.type )
            {
              var misfireInstruction =
                  QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionFromNumber( trigger.type,
                      trigger.misfireInstruction );

              return QDesk.util.quartz.QuartzEnumUtils.getTriggerMisfireInstructionName( trigger.type,
                  misfireInstruction );
            }
            else
            {
              // if we do not know the trigger type, then we are not able to determine the misfire instruction name;
              // so we display the numeric constant instead
              return QDesk.Msg.get( 'QDesk.unknown' ) + ' (' + trigger.misfireInstruction.toString() + ')';
            }
          },
          flex : 1
        },
        {
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobNameColumn' ),
          dataIndex : 'jobDetail.name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.name );
          }
        },
        {
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobGroupNameColumn' ),
          dataIndex : 'jobDetail.group.name',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.group.name );
          }
        }
    );


    if ( jvmAgentReady )
    {
      var maxExecHealthIndicatorEntries = config.qdesk.jvmAgentInfo.getConfigProperty(
          'quartz.execHealthIndicators.jobs.maxEntriesPerIndicator' );

      if ( publicApiReady )
      {
        columns.push(
            {
              xtype : 'widgetcolumn',
              itemId : 'jobExecHealthIndicatorColumn',
              text : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobExecHealthIndicatorColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobExecHealthIndicatorColumn',
                  maxExecHealthIndicatorEntries ),
              dataIndex : 'jobDetail.execHealthIndicatorValues',
              sortable : false,
              groupable : false,
              width : 115,
              widget : {
                xtype : 'WExecHealthIndicator'
              },
              onWidgetAttach : function ( column, widget, record )
              {
                var jobDetail = record.get( 'jobDetail' );
                if ( jobDetail.execHealthIndicatorValues )
                {
                  widget.setValues( jobDetail.execHealthIndicatorValues );
                }
              }
            }
        );
      }
      else
      {
        // agent is ready, but public API is not ready
        columns.push(
            {
              itemId : 'jobExecHealthIndicatorColumn',
              text : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobExecHealthIndicatorColumn' ),
              tooltip : QDesk.Msg.get(
                  'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobExecHealthIndicatorColumn',
                  maxExecHealthIndicatorEntries ),
              dataIndex : 'jobDetail.execHealthIndicatorValues',
              sortable : false,
              groupable : false,
              width : 115,
              renderer : function ()
              {
                var tooltip = QDesk.controller.command.connection.ConnectionUtils.getPublicApiWarningTooltip(
                    config.qdesk.connectionId );

                return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                    Ext.String.htmlEncode( tooltip ) + '"/>';
              }
            }
        );
      }
    }
    else
    {
      // agent is not ready
      columns.push(
          {
            itemId : 'jobExecHealthIndicatorColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobExecHealthIndicatorColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobExecHealthIndicatorColumn', 10 ),
            dataIndex : 'jobDetail.execHealthIndicatorValues',
            sortable : false,
            groupable : false,
            width : 115,
            renderer : function ()
            {
              var tooltip = QDesk.controller.command.connection.ConnectionUtils.getJvmAgentWarningTooltip(
                  config.qdesk.connectionId );

              return '<img src="images/exec-health-indicator-na.png" width="103" height="13" data-qtip="' +
                  Ext.String.htmlEncode( tooltip ) + '"/>';
            }
          }
      );
    }


    columns.push(
        {
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobClassColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobClassColumn' ),
          dataIndex : 'jobDetail.jobClass',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return Ext.String.htmlEncode( jobDetail.jobClass );
          }
        },

        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobDurabilityColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobDurabilityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobDurabilityColumn' ),
          dataIndex : 'jobDetail.durability',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.durability, metaData );  // does not work for nested properties - we use renderer
          }
        },
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobShouldRecoverColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobShouldRecoverColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobShouldRecoverColumn' ),
          dataIndex : 'jobDetail.shouldRecover',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.shouldRecover, metaData );  // does not work for nested properties - we use renderer
          }
        },
        {
          xtype : 'WReadOnlyCheckColumn',
          itemId : 'jobVolatilityColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobVolatilityColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobVolatilityColumn' ),
          dataIndex : 'jobDetail.volatility',
          sortable : false,
          groupable : false,
          width : 90,
          renderer : function ( value, metaData, record )
          {
            var jobDetail = record.get( 'jobDetail' );
            return this.defaultRenderer( jobDetail.volatility, metaData );  // does not work for nested properties - we use renderer
          }
        }
    );

    if ( jvmAgentReady )
    {
      columns.push(
          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'jobConcurrentColumn',
            text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobConcurrentColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobConcurrentColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.concurrentExecutionDisallowed',  // does not work for nested properties - we use renderer
            sortable : false,
            groupable : false,
            width : 110,
            renderer : function ( value, metaData, record )
            {
              var jobDetail = record.get( 'jobDetail' );
              // we need to invert the concurrentExecutionDisallowed value
              return this.defaultRenderer( !jobDetail.concurrentExecutionDisallowed, metaData );
            }
          },

          // available only if the agent is present, otherwise the value is unknown
          {
            xtype : 'WReadOnlyCheckColumn',
            itemId : 'jobPersistJobDataAfterExecutionColumn',
            text : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.textJobPersistsJobDataColumn' ),
            tooltip : QDesk.Msg.get(
                'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipJobPersistsJobDataColumn' ),
            align : 'center',
            dataIndex : 'jobDetail.persistJobDataAfterExecution',  // does not work for nested properties - we use renderer
            sortable : false,
            groupable : false,
            width : 140,
            renderer : function ( value, metaData, record )
            {
              var jobDetail = record.get( 'jobDetail' );
              return this.defaultRenderer( jobDetail.persistJobDataAfterExecution, metaData );
            }
          }
      );
    }


    Ext.apply( config, {
      plugins : [
        {
          ptype : 'rowexpander',
          rowBodyTpl : [
            '<p>' +
            '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
            '<tpl if="trigger.description">{[Ext.String.htmlEncode(values.trigger.description)]}</tpl>' +
            '<tpl if="!trigger.description">{[Ext.String.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
            '</p>' +

            '<p style="white-space: nowrap">' +
            '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
            '{[this.getMonitoringLink( values.trigger.monitoringUrl )]}' +
            '</p>' +

            '{[QDesk.util.quartz.QuartzTriggerUtils.createRowDetailsTableHtml( values.trigger )]}',
            {
              getMonitoringLink : function ( monitoringUrl )
              {
                var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
                var tooltip;
                if ( licensedFeatures.PollMonitoring )
                {
                  tooltip =
                      Ext.String.htmlEncode( QDesk.Msg.get(
                          'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipMonitoringUrl' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
                }
                else
                {
                  tooltip =
                      Ext.String.htmlEncode( QDesk.Msg.get(
                          'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.tooltipMonitoringUrlNotLicensed' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                      tooltip + '">' + monitoringUrl + '</a>';
                }
              }
            }
          ]
        }
      ]
    } );

    Ext.apply( config, {
      columns : columns,
      store : store,

      qdesk : {
        gridId : 'quartz.triggersGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var triggerGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzTriggerGroupStore' );

    triggerGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByTriggerGroupCombo' ).store = triggerGroupStore;

    me.getStore().on( 'load', function ( store, records, successful, operation )
    {
      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  },


  /**
   * Invoked to create a context menu for the specified trigger in the TriggersGrid row.
   * @param triggerWithJobDetail a trigger with a job detail.
   * @param jvmAgentReady a flag indicating if the QuartzDesk JVM agent is ready.
   * @param publicApiReady a flag indicating if the QuartzDesk Public API is ready.
   * @private
   */
  _createRowCtxMenu : function ( triggerWithJobDetail, jvmAgentReady, publicApiReady )
  {
    var me = this;

    var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger', triggerWithJobDetail.get( 'trigger' ) );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // resume trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuResume' ),
          iconCls : 'quartzResumeTrigger',
          disabled : trigger.get( 'state' ) != QDesk.model.scheduler.quartz.QuartzTriggerState.PAUSED,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_RESUME ],
          qdNotAuthorizedIconCls : 'quartzResumeTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'resumeTrigger', trigger );
          }
        },
        // pause trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuPause' ),
          iconCls : 'quartzPauseTrigger',
          disabled : trigger.get( 'state' ) != QDesk.model.scheduler.quartz.QuartzTriggerState.NORMAL,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_PAUSE ],
          qdNotAuthorizedIconCls : 'quartzPauseTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'pauseTrigger', trigger );
          }
        },
        '-',
        // delete trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuDelete' ),
          iconCls : 'quartzDeleteTrigger',
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_DELETE ],
          qdNotAuthorizedIconCls : 'quartzDeleteTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'deleteTrigger', trigger );
          }
        },
        // edit trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuEdit' ),
          iconCls : 'quartzEditTrigger',
          disabled : !jvmAgentReady || !publicApiReady,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_UPDATE ],
          qdNotAuthorizedIconCls : 'quartzEditTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'editTrigger', trigger );
          }
        },
        // clone trigger
        {
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.TriggersGrid.triggerCtxMenuClone' ),
          iconCls : 'quartzCloneTrigger',
          disabled : !jvmAgentReady || !publicApiReady,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.QUARTZ_TRIGGER_ADD ],
          qdNotAuthorizedIconCls : 'quartzCloneTriggerNAUTH',
          handler : function ()
          {
            me.fireEvent( 'cloneTrigger', trigger );
          }
        }
      ]
    } );
  }
} );
