/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.triggers.chain.TriggerJobChainsView', {
  extend : 'QDesk.view.work.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.widget.message.WarningMessagePanel',
    'QDesk.view.work.scheduler.quartz.chain.JobChainsGrid'
  ],

  alias : 'widget.QuartzTriggerJobChainsView',

  iconCls : 'quartzTriggerJobChainsTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.JobChaining )
    {
      if ( me.isJvmAgentReady( config.qdesk.jvmAgentInfo ) )
      {
        var items = [];

        if ( !me.isPublicApiReady( config.qdesk.schedulerInfo ) )
        {
          items.push(
              {
                xtype : 'WarningMessagePanel',
                style : {
                  margin : '5px'
                },
                qdesk : {
                  msg : me.getPublicApiWarningMessage( config.qdesk.schedulerInfo, config.qdesk.jvmAgentInfo )
                }
              }
          );
        }

        items.push(
            {
              xtype : 'QuartzJobChainsGrid',
              itemId : 'jobChainsGrid',
              flex : 1,
              qdesk : {
                gridId : 'quartz.triggerJobChainsGrid'
              }
            }
        );

        /*
         * Intentionally over-nested a bit so that we use identical layout to other tabs that
         * have a license restrictions related message panel above the grid.
         */
        Ext.apply( config, {
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : 'fit',
              flex : 1,
              items : [
                {
                  xtype : 'panel',
                  border : false,
                  layout : {
                    type : 'vbox',
                    align : 'stretch'
                  },
                  items : items
                }
              ]
            }
          ]
        } );
      }
      else
      {
        Ext.apply( config, {
          items : [
            {
              xtype : 'WarningMessagePanel',
              style : {
                margin : '5px'
              },
              qdesk : {
                msg : me.getJvmAgentWarningMessage( config.qdesk.schedulerInfo, config.qdesk.jvmAgentInfo )
              }
            }
          ]
        } );
      }
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'WarningMessagePanel',
            style : {
              margin : '5px'
            },
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : me._getTitle(),
      tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.chain.TriggerJobChainsView.tooltip' )
    } );

    me.callParent( [ config ] );
  },


  /**
   * Overridden method from the base class that returns the warning message shown at the top of the job chains grid.
   * @param schedulerInfo the scheduler info.
   * @param jvmAgentInfo the QuartzDesk JVM Agent info.
   * @returns the warning message.
   */
  getPublicApiWarningMessage : function ( schedulerInfo, jvmAgentInfo )
  {
    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.chain.TriggerJobChainsView.publicApiNotLoadedCorrectlyMsg' );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get(
            'QDesk.view.work.scheduler.quartz.triggers.chain.TriggerJobChainsView.publicApiIncompatibleVersionMsg',
            QDesk.util.FormatUtils.formatProductVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatProductVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get(
          'QDesk.view.work.scheduler.quartz.triggers.chain.TriggerJobChainsView.publicApiNotDetectedMsg' );
    }
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.triggers.chain.TriggerJobChainsView.title' );
    if ( !licensedFeatures.JobChaining )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );