/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.Msg',

    'QDesk.model.security.EffectiveUserPermissionSource',
    'QDesk.model.security.Permission',

    'QDesk.model.security.EffectiveUserPermission',

    'QDesk.store.security.users.EffectiveUserPermissionStore',

    'QDesk.util.EnumUtils',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.EffectiveUserPermissionsGrid',

  cls : 'effectiveUserPermissionsGrid',

  //scrollable : true,

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
        '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
        '{[Ext.String.htmlEncode(QDesk.util.EnumUtils.getPermissionDescription(values.permission))]}' +
        '</p>'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var store = Ext.create( 'QDesk.store.security.users.EffectiveUserPermissionStore' );

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
      ],

      columns : [
        // permission
        {
          itemId : 'permissionColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.textPermissionColumn' ),
          tooltip : QDesk.Msg.get(
              'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.tooltipPermissionColumn' ),
          dataIndex : 'permission',
          sortable : true,
          groupable : false,
          flex : 1
        },

        // source
        {
          itemId : 'sourceColumn',
          menuText : QDesk.Msg.get( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.textSourceColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.textSourceColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.tooltipSourceColumn' ),
          dataIndex : 'source',
          sortable : false,
          groupable : false,
          flex : 0.6,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var iconClass, tooltipText;

            var source = value;
            var sourceId = record.get( 'sourceId' );
            var sourceName = record.get( 'sourceName' );

            if ( source == QDesk.model.security.EffectiveUserPermissionSource.USER )
              iconClass = 'effectiveUserPermissionSourceUser';
            else if ( source == QDesk.model.security.EffectiveUserPermissionSource.GROUP )
              iconClass = 'effectiveUserPermissionSourceGroup';
            else
              Ext.Error.raise( 'Unexpected effective user permission source value: ' + source );

            var text = QDesk.util.EnumUtils.getEffectiveUserPermissionSourceName( source );
            tooltipText = text;

            text += ' (' + Ext.String.htmlEncode( sourceName ) + ')';

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>' +
                '<span style="padding-left: 5px;">' + text + '</span>';
          }
        }
      ],

      store : store,

      qdesk : {
        gridId : 'effectiveUserPermissionsGrid'
      }
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  /**
   * Refreshes this grid.
   */
  refresh : function ()
  {
    var me = this;
    var store = me.getStore();
    store.loadPage( 1 );
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var store = me.getStore();

    // refresh button
    var refreshButton = me.down( '#refreshButton' );
    refreshButton.on( 'click',
        function ()
        {
          me.refresh();
        } );

    store.on( 'beforeload', function ( store, operation )
    {
      refreshButton.disable();

      /*
       * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
       * Therefore we need to force-fire this event by clearing the selection.
       */
      me.getSelectionModel().deselectAll();
    } );

    store.on( 'load', function ( store, records, successful, operation )
    {
      refreshButton.enable();

      var view = me.getView();

      if ( successful )
      {
        if ( Ext.isEmpty( records ) )
        {
          view.emptyText =
              QDesk.Msg.get( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.emptyTextNoMatchingData' );

          view.refresh();
        }
      }
      else
      {
        store.removeAll();

        view.emptyText =
            QDesk.Msg.get( 'QDesk.view.work.security.users.EffectiveUserPermissionsGrid.emptyTextErrorLoadingData' );

        view.refresh();
      }
    } );
  }
} );
