/*
 * Copyright (c) 2013-2018 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.top.LicenseInfoPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.util.FormatUtils'
  ],

  alias : 'widget.LicenseInfoPanel',

  cls : 'licenseInfoPanel',

  layout : {
    type : 'vbox',
    align : 'stretchmax'
  },

  height : 40,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.product )
      Ext.Error.raise( 'Required config parameter missing: qdesk.product' );

    if ( !config.qdesk.license )
      Ext.Error.raise( 'Required config parameter missing: qdesk.license' );

    var productInfo = config.qdesk.product;
    var licenseInfo = config.qdesk.license;

    // we do not want the work panel to cache the qdesk data
    delete config.qdesk;

    Ext.apply( config, {
      items : [
        {
          xtype : 'panel',
          itemId : 'productAndVersion',
          cls : 'productAndVersion'
        },
        {
          xtype : 'panel',
          itemId : 'licenseeNameAndExpiry',
          cls : 'licenseeNameAndExpiry'
        }
      ]
    } );

    me.callParent( [ config ] );

    me.updateProductInfo( productInfo );
    me.updateLicenseInfo( licenseInfo );
  },


  updateProductInfo : function ( product )
  {
    var me = this;
    var productAndVersionPanel = me.down( '#productAndVersion' );
    productAndVersionPanel.update(
        QDesk.util.FormatUtils.formatProductNameAndVersion( product.name, product.version )
    );
  },


  updateLicenseInfo : function ( license )
  {
    var me = this;
    var licenseeNameAndExpiryPanel = me.down( '#licenseeNameAndExpiry' );
    var content = '<span class="licenseeNameLabel">'
        + QDesk.Msg.get( 'QDesk.view.work.top.LicenseInfoPanel.fieldLabelLicenseeName' )
        + ':</span>'
        + '<span class="licenseeName">' + license.licensee.name + '</span>';

    if ( license.type === QDesk.model.license.LicenseType.TEMPORARY )
    {
      if ( license.expiresInDays )
      {
        if ( license.expiresInDays > 0 )
        {
          // license has not expired yet - show number of days till expiration
          content += ' | <span class="expiresInDaysLabel">'
              + QDesk.Msg.get( 'QDesk.view.work.top.LicenseInfoPanel.fieldLabelLicenseExpiresInDays' )
              + ':</span>'
              + '<span class="expiresInDaysText">' + license.expiresInDaysText + '</span>'
        }
        else
        {
          // license expired
          content += ' | <span class="expiredText">'
              + QDesk.Msg.get( 'QDesk.view.work.top.LicenseInfoPanel.textLicenseExpired' )
              + '</span>'
        }
      }
    }
    else if ( license.type === QDesk.model.license.LicenseType.PERPETUAL )
    {
      if ( license.swMaintenanceExpiresInDays )
      {
        if ( license.swMaintenanceExpiresInDays > 0 )
        {
          // SW Maintenance has not expired yet - show number of days till expiration
          content += ' | <span class="expiresInDaysLabel">'
              + QDesk.Msg.get( 'QDesk.view.work.top.LicenseInfoPanel.fieldLabelSwMaintenanceExpiresInDays' )
              + ':</span>'
              + '<span class="expiresInDaysText">' + license.swMaintenanceExpiresInDaysText + '</span>'
        }
        else
        {
          // SW Maintenance expired
          content += ' | <span class="expiredText">'
              + QDesk.Msg.get( 'QDesk.view.work.top.LicenseInfoPanel.textSwMaintenanceExpired' )
              + '</span>'
        }
      }
    }

    licenseeNameAndExpiryPanel.update( content );
  }
} );